* = $0000

; --- Jump to init
	nop
	nop
	nop	
	jmp hi_color_demo_init

; reserve the remainder of zero page for run-time data
DEMO_PTR:
DEMO_PTR_L: !fi 1
DEMO_PTR_H  !fi 1
VIDEO_ADR_TEMP: !fi 1

DEMO_ADR_LAST_SECOND: !fi 1
DEMO_ADR_IDLE_SECONDS: !fi 1
DEMO_IDLE_TIMEOUT = 30

VIDEO_RAM_A0 = $7FFE
VIDEO_RAM_A0_L = $7FFE
VIDEO_RAM_A0_H = $7FFF

* = $0100
; includes
DRIVER_RAM_BASE = hi_color_demo_end_of_program 
!source "app.s"
!source "lcd.s"
!source "keyhit.s"
!source "video_defs.s"
	
; --- init
hi_color_demo_init:	
	lda #<hi_color_demo_isr
	sta $00FFFE
	lda #>hi_color_demo_isr
	sta $00FFFF	
	+app_init
			
	jsr lcd_standard_init
	jsr keyhit_standard_init
	jsr print_title
	
	cli
	
	; start the program
	jsr hi_color_sample
	
	; wait for key
	clr DEMO_ADR_IDLE_SECONDS
-	lda KEYBOARD_ADR_KEY_READY
	bne +
	
	lda $003F20 ; RTC_ADR_SECONDS
	cmp DEMO_ADR_LAST_SECOND
	beq -
	sta DEMO_ADR_LAST_SECOND
	inc DEMO_ADR_IDLE_SECONDS
	lda DEMO_ADR_IDLE_SECONDS
	cmp #DEMO_IDLE_TIMEOUT
	bcc -
	
	; exit
+	jml app_warm_boot
	
; --- Interrupt service routine	
hi_color_demo_isr:		
	php
	phb
	pha
	phy
	phx
	
	lda APP_ADR_PROGRAM_BANK
	bank
	
	; was the interrupt triggered by the keyboard?
	lda KEYBOARD_ADR_CONTROL
	and #$1
	bne +
	lda KEYBOARD_ADR_DATA
	jsr keyboard_handle_scan_code
			
+	plx
	ply
	pla
	plb
	plp
	
	rtl ; returns to page 0 wrapper
		

; --- Print title string to LCD	 
print_title:
	ldx #$00
	
-	lda demo_titlestr,X
	beq +
	
	phx
	jsr lcd_write_char
	plx
	inx
	
	; terminate after 256 chars, even if NULL is not reached
	bne -

+	rts

	
demo_titlestr:	
!text "256 color image demo", 13, "Press any key to exit", 0
	 

; --- 256 color sample
; @noparams
; 128x240 @ 256 colors
hi_color_sample:
	lda #$40 ; 240 lines, mode byte per frame
	sta VIDEO_ADR_VSIZE 
	
	jsr video_zero_vram
	jsr video_palette_8bpp
	
	lda #$30 ; 256 colors, palette 0
	phb
	bank #VIDEO_RAM_BANK
	sta VIDEO_ADR_FRAME_MODE_0 
	sta VIDEO_ADR_FRAME_MODE_1 
	sta VIDEO_ADR_FRAME_MODE_2 
	sta VIDEO_ADR_FRAME_MODE_3 
	sta VIDEO_ADR_FRAME_MODE_4 
	plb
	
	; init source pointer
	lda #<(video_image_data+54+256*4)
	sta DEMO_PTR_L
	lda #>(video_image_data+54+256*4)
	sta DEMO_PTR_H
	
	; init dest pointer = start of line 479 = $7781
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$77
	sta VIDEO_RAM_A0_H
	plb
	
.copy_loop:
	jsr copy_one_line
	
	; increment source pointer
	lda DEMO_PTR_L
	clc
	adc #$80
	sta DEMO_PTR_L
	bcc +
	inc DEMO_PTR_H
+	

	; decrement dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda VIDEO_RAM_A0_L
	sec
	sbc #$80
	sta VIDEO_RAM_A0_L
	bcs +
	dec VIDEO_RAM_A0_H
	bmi .done
+
	plb
	bra .copy_loop

.done	
	plb
	rts
	

; --- Copy one line
copy_one_line:	
	; last byte of source line goes at first byte of dest line
	ldx #127
	lda (DEMO_PTR),X
	phb
	bank #VIDEO_RAM_BANK
	ldx #0
	sta (VIDEO_RAM_A0), X
	plb
	
	; other bytes are shifted one to the right
	ldx #126
-	lda (DEMO_PTR),X
	phb
	bank #VIDEO_RAM_BANK
	inx
	sta (VIDEO_RAM_A0), X
	dex
	plb
	
	dex
	bpl -
	
	rts
	
			
; --- Video zero VRAM
; @noparams
; Clears contents of VRAM to all zeroes.
video_zero_vram:
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$7E
	sta VIDEO_RAM_A0_H
	
--	ldx #$7F
	lda #0
	
-	sta (VIDEO_RAM_A0),X
	dex
	bpl -
	
	; decrement row pointer
	lda VIDEO_RAM_A0_L
	sec
	sbc #$80
	sta VIDEO_RAM_A0_L
	bcs --
	dec VIDEO_RAM_A0_H
	bpl --
	
	plb 
	rts
	
			
; --- Video setup 8bpp palette
; @noparams
video_palette_8bpp:
install_palette:	
	; src pointer
	lda #<(video_image_data+56)
	sta DEMO_PTR_L
	lda #>(video_image_data+56)
	sta DEMO_PTR_H
	
	ldx #$00
	ldy #$00
	
.next_entry	
	tya
	sta VIDEO_ADR_PAL_INDEX
	
	lda (DEMO_PTR),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; R
	dec DEMO_PTR_L
	lda DEMO_PTR_L
	cmp #$FF
	bne +
	dec DEMO_PTR_H
+	lda (DEMO_PTR),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; G
	dec DEMO_PTR_L
	lda DEMO_PTR_L
	cmp #$FF
	bne +
	dec DEMO_PTR_H
+	lda (DEMO_PTR),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; B
	
	; inc 6
	inc DEMO_PTR_L
	bne +
	inc DEMO_PTR_H
+	inc DEMO_PTR_L
	bne +
	inc DEMO_PTR_H
+	inc DEMO_PTR_L
	bne +
	inc DEMO_PTR_H
+	inc DEMO_PTR_L
	bne +
	inc DEMO_PTR_H
+	inc DEMO_PTR_L
	bne +
	inc DEMO_PTR_H
+	inc DEMO_PTR_L
	bne +
	inc DEMO_PTR_H
			
+	iny
	bne .next_entry
					
	rts	
	
		
; --- Data		
video_image_data:
!binary "HalfDome.bmp"	

hi_color_demo_end_of_program:
	