* = $0000

; --- Jump to init
	nop
	nop
	nop
	jmp audio_init

; reserve the remainder of zero page for run-time data
FRAME_COUNT_L: !fi 1
FRAME_COUNT_H: !fi 1
REG_COUNTS: !fi 28
REG_MODES: !fi 28
REG_VALUES: !fi 28
REG_DATA_PTRS: !fi 28
AUDIO_TEMP: !fi 1
VOICE_HPOS: !fi 3

; MODE_COPY = 0
; MODE_REPEAT = non-zero

* = $0100
; includes
DRIVER_RAM_BASE = audio_end_of_program 
!source "lcd.s"
!source "keyhit.s"
!source "audio.s"
!source "console.s"
!source "app.s"
!source "video_defs.s"
	
!source "tim_follin_agent_x2.ym.data.s"
	
; --- Simple init
audio_init:	
	lda #<audio_isr
	sta $00FFFE
	lda #>audio_isr
	sta $00FFFF	
	+app_init
			
	jsr lcd_standard_init
	jsr keyhit_standard_init
	jsr video_zero_vram
	jsr print_title
	
	; init voice hpos
	lda #2
	sta VOICE_HPOS
	sta VOICE_HPOS+1
	sta VOICE_HPOS+2
	
	; set video mode
	lda #$00
	sta VIDEO_ADR_VSIZE ; 240 lines, mode byte per line
	
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$67
	sta VIDEO_RAM_A0_H
	
	; set modes for graphic lines
	ldx #0
-	lda #$B0 ; 128x120 @ 256 colors 
	sta (VIDEO_RAM_A0),X
	inx
	sta (VIDEO_RAM_A0),X
	dex
	lda VIDEO_RAM_A0_L
	sec
	sbc #$80
	sta VIDEO_RAM_A0_L
	bcs -
	dec VIDEO_RAM_A0_H
	bpl -	
	
	; setup palette: 0 = black
	; black background
	lda #$00
	sta VIDEO_ADR_PAL_INDEX
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	
	; 1 = blue
	lda #1
	sta VIDEO_ADR_PAL_INDEX
	lda #0
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR

	; 2 = yellow
	lda #2
	sta VIDEO_ADR_PAL_INDEX
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	lda #0
	sta VIDEO_ADR_PAL_COLOR
	
	; 3 = green
	lda #3
	sta VIDEO_ADR_PAL_INDEX
	lda #0
	sta VIDEO_ADR_PAL_COLOR
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	lda #0
	sta VIDEO_ADR_PAL_COLOR
	
	; set 4-15 to black
	ldx #4
-	txa
	sta VIDEO_ADR_PAL_INDEX
	lda #0
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	inx
	cpx #$10
	bne -
	
	; set palette 1 to white on blue
	ldx #$10
	
-	; background color
	txa
	sta VIDEO_ADR_PAL_INDEX
	
	; blue background
	lda #$00
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	
	inx
	
	; foreground color
	txa
	sta VIDEO_ADR_PAL_INDEX
	
	; foreground.rgb
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
		
	inx
	cpx #$20
	bne -
			
	; set palette 2 to white on black
	ldx #$20
	
-	; background color
	txa
	sta VIDEO_ADR_PAL_INDEX
	
	; black background
	lda #$00
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	
	inx
	
	; foreground color
	txa
	sta VIDEO_ADR_PAL_INDEX
	
	; foreground.rgb
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
		
	inx
	cpx #$30
	bne -
					
	; set all other palette entries to black
	ldx #$30
-	txa
	sta VIDEO_ADR_PAL_INDEX
	lda #0
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	inx
	bne -
	
	; set mode for text lines
	lda #$42 ; text, 1bpp, palette 2
	sta $6400
	sta $6401
	
	lda #$41 ; text, 1bpp, palette 1
	sta $6C00
	sta $6C01
	sta $7000
	sta $7001
	plb
	
	jsr console_enable_mixed_mode
	lda #0
	jsr console_enable_cursor
	lda #27
	jsr console_set_row			
	cli
	
	; clear the text lines
	phb
	bank #VIDEO_RAM_BANK
	lda #' '
	ldx #$02
-	sta $6400,X
	inx
	bpl -

	ldx #$02
-	sta $6C00,X
	inx
	bpl -
	
	ldx #$02
-	sta $7000,X
	inx
	bpl -
	plb
	
	; title and artist
	lda #<audio_titlestr
	sta CONSOLE_ADR_A0_L
	lda #>audio_titlestr
	sta CONSOLE_ADR_A0_H
	jsr console_write_string_line
	
	lda #<audio_artiststr
	sta CONSOLE_ADR_A0_L
	lda #>audio_artiststr
	sta CONSOLE_ADR_A0_H
	jsr console_write_string
	
	; start the program
	jsr audio_run 

	halt

; --- Interrupt service routine	
audio_isr:
	php
	phb
	pha
	phy
	phx
	
	lda APP_ADR_PROGRAM_BANK
	bank
	
	; was the interrupt triggered by the keyboard?
	lda KEYBOARD_ADR_CONTROL
	and #$1
	bne +
	lda KEYBOARD_ADR_DATA
	jsr keyboard_handle_scan_code
		
	; exit on keypress
	lda KEYBOARD_ADR_KEY_READY
	beq +
	
	; zero all audio registers and exit
	ldy #13
-	tya
	sta AUDIO_REGISTER   
	lda #0
	sta AUDIO_DATA 
	dey
	bpl -  
	jml app_warm_boot
				
+	plx
	ply
	pla
	plb
	plp
	
	rtl ; returns to page 0 wrapper
	
		
; --- Video zero VRAM
; @noparams
; Clears contents of VRAM to all zeroes.
video_zero_vram:
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$7E
	sta VIDEO_RAM_A0_H
	
--	ldx #$7F
	lda #0
	
-	sta (VIDEO_RAM_A0),X
	dex
	bpl -
	
	; decrement row pointer
	lda VIDEO_RAM_A0_L
	sec
	sbc #$80
	sta VIDEO_RAM_A0_L
	bcs --
	dec VIDEO_RAM_A0_H
	bpl --
	
	plb 
	rts

			
audio_run:
	; clear the block counts
	lda #0
	ldx #26
-	sta REG_COUNTS,X
	dex
	dex
	bpl -

	; init the data pointers
	ldx #0
-	lda #<musicData
	clc
	adc musicData,X
	sta REG_DATA_PTRS,X
	inx
	lda #>musicData
	adc musicData,X
	sta REG_DATA_PTRS,X
	inx
	cpx #28
	bne -
	
	; init the frame count
	lda musicData+28
	sta FRAME_COUNT_L
	lda musicData+29
	sta FRAME_COUNT_H	
	
.begin_frame:	
	; wait for non-vblank
-	lda VIDEO_ADR_VBLANK
	bpl -

	ldx #0 ; register offset
	ldy #0 ; register index
	
.set_registers:
	lda REG_COUNTS,X
	beq .new_block
	
	lda REG_MODES,X
	bne .repeat_byte
	
.copy_byte:
	; ignore $FF values for register 13
	cpx #26
	bne +
	lda (REG_DATA_PTRS,X)
	cmp #$FF
	beq ++
		
+	tya
	sta AUDIO_REGISTER   
	lda (REG_DATA_PTRS,X)
	sta AUDIO_DATA       
	sta REG_VALUES,X ; save it off, for visualization

++	; increment data pointer
	inc REG_DATA_PTRS,X
	bne +
	inc REG_DATA_PTRS+1,X
+	
	; decrement count
	dec REG_COUNTS,X
	bra .next_register

.repeat_byte:
	; ignore $FF values for register 13
	cpx #26
	bne +
	lda REG_VALUES,X
	cmp #$FF
	beq ++
	
+	tya
	sta AUDIO_REGISTER   
	lda REG_VALUES,X
	sta AUDIO_DATA       

++	; decrement count
	dec REG_COUNTS,X
	bra .next_register
		
.new_block:
	; get block header
	lda (REG_DATA_PTRS,X)       

	; increment data pointer
	inc REG_DATA_PTRS,X
	bne +
	inc REG_DATA_PTRS+1,X
+

	; determine the block type
	cmp #$80
	bmi .new_copy_block
	
.new_repeat_block
	and #$7F
	inc ; block size = N+1
	sta REG_COUNTS,X
	sta REG_MODES,X ; any non-zero value will do
	
	; get repeat value
	lda (REG_DATA_PTRS,X)       
	sta REG_VALUES, X
	
	; increment data pointer
	inc REG_DATA_PTRS,X
	bne +
	inc REG_DATA_PTRS+1,X
+
	bra .repeat_byte
	
.new_copy_block
	inc ; block size = N+1
	sta REG_COUNTS,X
	lda #0
	sta REG_MODES,X 
	bra .copy_byte
					
.next_register:
	iny
	inx
	inx
	cpx #28
	bne .set_registers
		
.next_frame:
	jsr draw_visualizer	
	jsr waitNext
	
	; decrement frame count
	dec FRAME_COUNT_L
	lda FRAME_COUNT_L
	cmp #$FF
	bne +
	dec FRAME_COUNT_H
	lda FRAME_COUNT_H
	cmp #$FF
	bne +

	; zero all audio registers and exit
	ldy #13
-	tya
	sta AUDIO_REGISTER   
	lda #0
	sta AUDIO_DATA 
	dey
	bpl -  
	jml app_warm_boot
		
+	jmp .begin_frame
	
	
waitNext:
	; wait 20ms
	lda #$0D ; #$1D
--	ldx #$ff
-	dex
	bne -
	dec
	bne --
	rts
	 
print_title:
	ldx #$00
	
-	lda audio_lcdstr,X
	beq +
	
	phx
	jsr lcd_write_char
	plx
	inx
	
	; terminate after 256 chars, even if NULL is not reached
	bne -

+	rts

draw_visualizer:
	; wait for vblank
-	lda VIDEO_ADR_VBLANK
	bmi -
	
	; erase old bars
	; voice A
	lda VOICE_HPOS
	jsr erase_amplitude_bar
	; voice B
	lda VOICE_HPOS+1
	jsr erase_amplitude_bar
	; voice C
	lda VOICE_HPOS+2
	jsr erase_amplitude_bar
			
	; voice A
	ldx REG_VALUES+(1*2)
	lda REG_VALUES+(0*2)
	jsr get_bar_freq_offset
	sta VOICE_HPOS
	ldy REG_VALUES+(8*2)
	ldx #1
	jsr draw_amplitude_bar
	
	; voice B
	ldx REG_VALUES+(3*2)
	lda REG_VALUES+(2*2)
	jsr get_bar_freq_offset
	sta VOICE_HPOS+1
	ldy REG_VALUES+(9*2)
	ldx #2
	jsr draw_amplitude_bar
	
	; voice C
	ldx REG_VALUES+(5*2)
	lda REG_VALUES+(4*2)
	jsr get_bar_freq_offset
	sta VOICE_HPOS+2
	ldy REG_VALUES+(10*2)
	ldx #3
	jsr draw_amplitude_bar
	
	; register values
	lda #25
	jsr console_set_row
	lda #24
	phb
	bank #VIDEO_RAM_BANK
	sta VIDEO_RAM_CURSOR_PTR_L
	plb
	ldx #0
	
-	lda REG_VALUES,X
	phx
	jsr console_write_hex
	lda #' '
	jsr console_write_char
	plx
	inx
	inx
	cpx #28
	bne -
	
	rts
	

; --- get bar freq offset 
; @param: X = course period
; @param: A = fine period
get_bar_freq_offset:
	; fine period >>= 2
	; ROL * 7 followed by AND should be faster than LSR * 2
	rol
	rol
	rol
	rol
	rol
	rol
	rol
	and #$3F
	sta AUDIO_TEMP
	
	; course period <<= 6
	txa
	asl
	asl
	asl
	asl
	asl
	asl
	
	; sum
	clc
	adc AUDIO_TEMP
	tax
	lda period_to_freq,X
	rts
	
; --- erase amplitude bar
; @param: A = horizontal offset for bar
erase_amplitude_bar:
	phb
	bank #VIDEO_RAM_BANK
	sta VIDEO_RAM_A0_L
	lda #56
	sta VIDEO_RAM_A0_H
	
	ldx #0
-	lda VIDEO_RAM_A0_H
	cmp #40
	bmi +
	lda #0
	sta (VIDEO_RAM_A0),X	
	inx
	sta (VIDEO_RAM_A0),X	
	dex
	dec VIDEO_RAM_A0_H
	bra -

+	plb
	rts
	
	
; --- draw amplitude bar
; @param: X = bar color
; @param: Y = bar height
; @param: A = horizontal offset for bar
draw_amplitude_bar:
	phb
	bank #VIDEO_RAM_BANK
	sta VIDEO_RAM_A0_L
	lda #56
	sta VIDEO_RAM_A0_H
	cpy #0
	beq +
	
-	bmi +					; 1
	txa						; 1
	phx						; 2
	ldx #0					; 2
	sta (VIDEO_RAM_A0),X    ; 8
	inx
	sta (VIDEO_RAM_A0),X
	dex
	plx						; 3
	dec VIDEO_RAM_A0_H		; 6
	dey						; 1
	bne -					; 5

+	ldx #0					
-	lda VIDEO_RAM_A0_H		; 4
	cmp #40					; 2
	bmi +					; 2
	lda #0					; 2
	sta (VIDEO_RAM_A0),X	; 8
	inx
	sta (VIDEO_RAM_A0),X
	dex
	dec VIDEO_RAM_A0_H		; 6
	bra -					; 5
	
+	plb
	rts
	
	
audio_lcdstr:	
!text  "Audio demo", 13, "Press any key to exit", 0

audio_titlestr:
!text "                    Now Playing: Agent X II", 0

audio_artiststr:
!text "                      Artist: Tom Follin", 0
	
period_to_freq:
	; period 255 -> freq 2
	; period 6 -> freq 126
	!byte $78,$78,$78,$78,$78,$78
	!for Period, 256-6 {
		!byte (126*6)/(Period+5)
	}

audio_end_of_program:
	