; Application init routines

app_warm_boot = $000100

APP_ADR_PROGRAM_BANK = $00FFFC

; --- Application init
; On entry, app_init expects to find an interrupt service routine address at $00FFFE.
; It replaces this with a wrapper routine in bank zero, which vectors to the program's
; original ISR in the program bank. Part of this wrapper includes the program's bank
; number, which can be examined by the running program using the constant APP_ADR_PROGRAM_BANK.
!macro app_init {
	; init stack pointer to $00FFFD
	ldx #$00
	ldy #$FF
	lda #$FD
	trs	
	
	; Create a subroutine in bank 0, which will return PCBANK in the accumulator.
	lda #$A3 ; LDA stack-relative
	sta $00FFF8
	lda #3 ; stack offset to return address bank byte
	sta $00FFF9
	lda #$6B ; RTL
	sta $00FFFA
	
	; call the subroutine in bank 0, to get PCBANK in A
	jsl $00FFF8
	bank ; use the program bank for all future 16-bit memory references
	sta APP_ADR_PROGRAM_BANK
	
	; re-init stack pointer to $XXFFF6
	tax
	ldy #$FF
	lda #$F6
	trs	
	
	; create a wrapper routine in bank 0, which performs a long jump to the real ISR
	lda #$22 ; JSL
	sta $00FFF9
	lda $00FFFE ; low byte of real ISR's address
	sta $00FFFA
	lda $00FFFF ; high byte of real ISR's address
	sta $00FFFB	
	; bank byte of real ISR's address is already at $00FFFC
	lda #$40 ; RTI
	sta $00FFFD 
	
	; install wrapper routine's address in the interrupt vector
	lda #$F9
	sta $00FFFE
	lda #$FF
	sta $00FFFF	
}