* = $0000

; Bootloader for BMOW
; Downloads the main program via USB and runs it

BOOTLOADER_ADR_WARM_BOOT_FLAG = $00FFF7 
GLOBAL_ADR_NEXT_DEMO_LOOP = $08EFFF

cold_start:
	sei
	lda #0
	sta BOOTLOADER_ADR_WARM_BOOT_FLAG
	sta GLOBAL_ADR_NEXT_DEMO_LOOP
	jmp bootloader_init

; skip to start of page 1, leaving most of page 0 free		
* = $0100		
warm_start: 
	sei
	lda #1
	sta BOOTLOADER_ADR_WARM_BOOT_FLAG
	jmp bootloader_init
			
; symbols from other banks
!source "bootloader_1.sym"
!source "bootloader_2.sym"
!source "bootloader_3.sym"
!source "bootloader_4.sym"
!source "bootloader_5.sym"
!source "bootloader_6.sym"
!source "bootloader_7.sym"

; driver includes
DRIVER_RAM_BASE = $FF20
!source "app.s"
!source "usb.s"
!source "lcd.s"
!source "keyboard.s"
!source "console.s"
!source "rtc.s"
MONITOR_ADR_END = CONSOLE_ADR_END
!source "bootlogo.s"
!source "audio.s"

; bootloader RAM addresses
BOOTLOADER_ADR_BASE = $FF00
BOOTLOADER_ADR_A0_L = BOOTLOADER_ADR_BASE
BOOTLOADER_ADR_A0_H = BOOTLOADER_ADR_BASE+1
BOOTLOADER_ADR_A0 = BOOTLOADER_ADR_BASE
BOOTLOADER_ADR_A1_L = BOOTLOADER_ADR_BASE+2
BOOTLOADER_ADR_A1_H = BOOTLOADER_ADR_BASE+3
BOOTLOADER_ADR_A1_B = BOOTLOADER_ADR_BASE+4
BOOTLOADER_ADR_A1 = BOOTLOADER_ADR_BASE+2
BOOTLOADER_ADR_BOOTSTATE = BOOTLOADER_ADR_BASE+5
BOOTLOADER_ADR_CHECKSUM = BOOTLOADER_ADR_BASE+6
BOOTLOADER_ADR_TGTBANK = BOOTLOADER_ADR_BASE+7
BOOTLOADER_ADR_DEST_L = BOOTLOADER_ADR_BASE+8
BOOTLOADER_ADR_DEST_H = BOOTLOADER_ADR_BASE+9
BOOTLOADER_ADR_DEST_B = BOOTLOADER_ADR_BASE+10
BOOTLOADER_ADR_STACKSAVE_LO = BOOTLOADER_ADR_BASE+11
BOOTLOADER_ADR_STACKSAVE_HI = BOOTLOADER_ADR_BASE+12
BOOTLOADER_ADR_STACKSAVE_BANK = BOOTLOADER_ADR_BASE+13
BOOTLOADER_ADR_LOADSIZE_L = BOOTLOADER_ADR_BASE+14
BOOTLOADER_ADR_LOADSIZE_H = BOOTLOADER_ADR_BASE+15
BOOTLOADER_ADR_LOADSIZE_B = BOOTLOADER_ADR_BASE+16
BOOTLOADER_ADR_PERCENT_THRESHHOLD_L = BOOTLOADER_ADR_BASE+17
BOOTLOADER_ADR_PERCENT_THRESHHOLD_H = BOOTLOADER_ADR_BASE+18
BOOTLOADER_ADR_PERCENT_PROGRESS_L = BOOTLOADER_ADR_BASE+19
BOOTLOADER_ADR_PERCENT_PROGRESS_H = BOOTLOADER_ADR_BASE+20
BOOTLOADER_ADR_CURRENT_PERCENT = BOOTLOADER_ADR_BASE+21
BOOTLOADER_ADR_TEMP = BOOTLOADER_ADR_BASE+21
BOOTLOADER_ADR_MENU_CHOICE = BOOTLOADER_ADR_BASE+22
BOOTLOADER_ADR_A0_B = BOOTLOADER_ADR_BASE+23
BOOTLOADER_ADR_LAST_MINUTE = BOOTLOADER_ADR_BASE+24
BOOTLOADER_ADR_LAST_SECOND = BOOTLOADER_ADR_BASE+25
BOOTLOADER_ADR_IDLE_SECONDS = BOOTLOADER_ADR_BASE+26

; bootloader states
BOOTLOADER_STATE_AWAITING_USB_ENTER = 0
BOOTLOADER_STATE_AWAITING_USB = 1
BOOTLOADER_STATE_AWAITING_SIGNATURE0 = 2
BOOTLOADER_STATE_AWAITING_SIGNATURE1 = 3
BOOTLOADER_STATE_AWAITING_SIZE0 = 4
BOOTLOADER_STATE_AWAITING_SIZE1 = 5
BOOTLOADER_STATE_AWAITING_SIZE2 = 6
BOOTLOADER_STATE_AWAITING_DATA = 7
BOOTLOADER_STATE_AWAITING_CHECKSUM = 8
	
; constants
BOOTLOADER_NUM_MENU_OPTIONS = 12
BOOTLOADER_NUM_MENU_OPTIONS_2 = 6
BOOTLOADER_NUM_MENU_OPTIONS_3 = 4
BOOTLOADER_MENU_COLUMN_OFFSET = 52
BOOTLOADER_MENU_COLUMN_OFFSET_2 = 102
BOOTLOADER_MENU_TOP_ROW = 17
BOOTLOADER_MENU_IDLE_TIMEOUT = 30
BOOTLOADER_NUM_DEMO_LOOP_ENTRIES = 10
	
	
; --- Bootloader init
bootloader_init:
	lda #<bootloader_isr
	sta $00FFFE
	lda #>bootloader_isr
	sta $00FFFF	
	+app_init
	
	; select the target bank: 2 or 4 (use the opposite bank from the one this is running in)
	lda APP_ADR_PROGRAM_BANK
	cmp #2
	beq +
	ldx #2
	stx BOOTLOADER_ADR_TGTBANK	
	bra ++
+	ldx #4
	stx BOOTLOADER_ADR_TGTBANK	
++		
	
	jsr keyboard_standard_init
	jsr lcd_standard_init
	
	clr BOOTLOADER_ADR_IDLE_SECONDS
	
	; display LCD boot message
	lda #<bootloader_initstr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_initstr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
	
	lda #' '
	jsr lcd_write_char
	lda #'v'
	jsr lcd_write_char
	lda #'.'
	jsr lcd_write_char
	lda bootlogo_buildate+4
	jsr lcd_write_char
	lda bootlogo_buildate+5
	jsr lcd_write_char
	lda bootlogo_buildate+7
	jsr lcd_write_char
	lda bootlogo_buildate+8
	jsr lcd_write_char
	lda bootlogo_buildate+12
	jsr lcd_write_char
	lda bootlogo_buildate+13
	jsr lcd_write_char
	
	; show video boot logo
	jsr bootlogo_init
	
	; modify palette
	; special case: palette 3 = dark green on black
	ldx #$30
	
-	; background color
	txa
	sta VIDEO_ADR_PAL_INDEX
	
	; black background
	lda #$00
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	
	inx
	
	; foreground color
	txa
	sta VIDEO_ADR_PAL_INDEX
	
	; foreground.rgb
	lda #0
	sta VIDEO_ADR_PAL_COLOR
	lda #$18
	sta VIDEO_ADR_PAL_COLOR
	lda #0
	sta VIDEO_ADR_PAL_COLOR
		
	inx
	cpx #$40
	bne -
	
	; set palette
	lda #7
	ldy #$0E ; gray on black
	jsr console_set_row_palette
	lda #8
	ldy #$0E ; gray on black
	jsr console_set_row_palette
	lda #9
	ldy #$0E ; gray on black
	jsr console_set_row_palette
	lda #10
	ldy #$0E ; gray on black
	jsr console_set_row_palette			
	
	; disable cursor
	lda #0
	jsr console_enable_cursor
	
	; init states
	clr BOOTLOADER_ADR_MENU_CHOICE
	lda #BOOTLOADER_STATE_AWAITING_USB_ENTER
	sta BOOTLOADER_ADR_BOOTSTATE
	
	; play boot sound, if cold boot
	lda BOOTLOADER_ADR_WARM_BOOT_FLAG
	bne +
	jsr bootloader_play_boot_sound
+
	
	; show menu
	jsr bootloader_print_menu
	
	cli ; enable interrupts
	
	; fall through to start of the state machine
		
; --- Bootloader state machine
bootloader_state_machine:
	lda BOOTLOADER_ADR_BOOTSTATE	
	cmp #BOOTLOADER_STATE_AWAITING_USB_ENTER
	beq bootloader_awaiting_usb_enter
	cmp #BOOTLOADER_STATE_AWAITING_USB
	beq bootloader_awaiting_usb
	cmp #BOOTLOADER_STATE_AWAITING_SIGNATURE0
	beq bootloader_awaiting_signature0_idle
	
	; other state: make sure USB is still up
	lda USB_ADR_CONTROL
	and #$04
	beq bootloader_state_machine
	
	; USB went down down: reset
	lda #BOOTLOADER_STATE_AWAITING_USB_ENTER
	sta BOOTLOADER_ADR_BOOTSTATE
	bra bootloader_state_machine
			
bootloader_awaiting_usb_enter:
	; print state message
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #<bootloader_usbdownstr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_usbdownstr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
	
	lda #BOOTLOADER_STATE_AWAITING_USB
	sta BOOTLOADER_ADR_BOOTSTATE
	bra bootloader_state_machine
		
bootloader_awaiting_usb:
	lda USB_ADR_CONTROL
	and #$04
	bne bootloader_state_machine
	
	; print state message
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #<bootloader_usbupstr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_usbupstr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
	
	lda #BOOTLOADER_STATE_AWAITING_SIGNATURE0
	sta BOOTLOADER_ADR_BOOTSTATE
	
	bra bootloader_state_machine
	

bootloader_awaiting_signature0_idle:
	lda RTC_ADR_SECONDS
	cmp BOOTLOADER_ADR_LAST_SECOND
	beq +
	sta BOOTLOADER_ADR_LAST_SECOND
	inc BOOTLOADER_ADR_IDLE_SECONDS
	lda BOOTLOADER_ADR_IDLE_SECONDS
	cmp #BOOTLOADER_MENU_IDLE_TIMEOUT
	bcc +
	jmp run_next_demo_loop
		
+	lda RTC_ADR_MINUTES
	cmp BOOTLOADER_ADR_LAST_MINUTE
	beq bootloader_state_machine
	
	sta BOOTLOADER_ADR_LAST_MINUTE	
	phb
	bank #VIDEO_RAM_BANK
	lda #42
	sta VIDEO_RAM_CURSOR_PTR_L	
	lda #40
	sta VIDEO_RAM_CURSOR_PTR_H
	plb			
	
	jsr rtc_print_time
	jsr console_advance_row
	
	jmp bootloader_state_machine
	
	
bootloader_awaiting_boot:
	jmp bootloader_boot	
	

bootloader_play_boot_sound:
	; init pointer to start of music data
	lda #<bootsound_data
	sta BOOTLOADER_ADR_A0_L
	lda #>bootsound_data
	sta BOOTLOADER_ADR_A0_H
	
	; set register values for this frame
--	ldx #13
	ldy (BOOTLOADER_ADR_A0),X
	cpy #0 ; finished?
	beq ++
	cpy #$ff ; skip envelope update?
	beq +

	; set envelope register
	txa
	jsr audio_set_register

	; set other registers
+	ldx #12	
-	ldy (BOOTLOADER_ADR_A0),X
	txa
	jsr audio_set_register
	dex
	bpl -
		
	; advance pointer to next frame
	lda #14
	clc
	adc BOOTLOADER_ADR_A0_L
	sta BOOTLOADER_ADR_A0_L
	bcc +
	inc BOOTLOADER_ADR_A0_H		
+	jsr wait20
	jmp --
	
++	rts
	
	
wait20:
	; wait ~20ms
	lda #$1B 
--	ldx #$ff
-	dex
	bne -
	dec
	bne --
	rts
	
		
; --- Bootloader interrupt service routine	
bootloader_isr:	
	php
	phb
	pha
	phy
	phx

	lda APP_ADR_PROGRAM_BANK
	bank
	
	; was the interrupt triggered by USB?
	lda USB_ADR_CONTROL
	and #$1
	bne +
	jsr bootloader_handler_usbdata
	bra bootloader_isr_done
	
	; was the interrupt triggered by the keyboard?
+	lda KEYBOARD_ADR_CONTROL
	and #$1
	bne bootloader_isr_done	
	lda KEYBOARD_ADR_DATA
	
	; extended key?
	ldx KEYBOARD_ADR_EXTENDED_FLAG
	beq ++
	; not break?
	ldx KEYBOARD_ADR_NOT_BREAK_FLAG
	beq ++
	
	; up arrow?
	cmp #$75
	bne +
	pha
	jsr bootloader_handle_arrow_up
	pla
	bra ++
	
	; down arrow
+	cmp #$72
	bne +
	pha
	jsr bootloader_handle_arrow_down
	pla

+	; left arrow
	cmp #$6B
	bne +
	pha
	jsr bootloader_handle_arrow_left
	pla
	
+	; right arrow
	cmp #$74
	bne ++
	pha
	jsr bootloader_handle_arrow_right
	pla

++	jsr keyboard_handle_scan_code
	
	; process the key
	lda #1
	cmp KEYBOARD_ADR_KEY_READY
	bne bootloader_isr_done
	lda KEYBOARD_ADR_KEY_BUFFER
	clr KEYBOARD_ADR_KEY_READY
	cmp #$0D ; test for carriage return
	bne bootloader_isr_done

	; abort bootloader and boot built-in ROM program
	jmp bootloader_builtin_boot
	
bootloader_isr_done:
	plx
	ply
	pla
	plb
	plp
	
	rtl ; returns to page 0 wrapper
	
; --- Bootloader handle arrow keys
bootloader_handle_arrow_up:
	jsr bootloader_dim_current_selection

	dec BOOTLOADER_ADR_MENU_CHOICE
	bpl +
	
	lda #BOOTLOADER_NUM_MENU_OPTIONS	
	dec
	sta BOOTLOADER_ADR_MENU_CHOICE
	
+	jsr bootloader_highlight_current_selection
	clr BOOTLOADER_ADR_IDLE_SECONDS
	rts
		
		
bootloader_handle_arrow_down:	
	jsr bootloader_dim_current_selection

	inc BOOTLOADER_ADR_MENU_CHOICE
	lda BOOTLOADER_ADR_MENU_CHOICE
	cmp #BOOTLOADER_NUM_MENU_OPTIONS
	bne +
	
	clr BOOTLOADER_ADR_MENU_CHOICE
	
+	jsr bootloader_highlight_current_selection
	clr BOOTLOADER_ADR_IDLE_SECONDS
	rts
	

bootloader_handle_arrow_right:	
	jsr bootloader_dim_current_selection

	lda BOOTLOADER_ADR_MENU_CHOICE
	clc
	adc #BOOTLOADER_NUM_MENU_OPTIONS_3
	cmp #BOOTLOADER_NUM_MENU_OPTIONS
	bcc +
	
	sec
	sbc #BOOTLOADER_NUM_MENU_OPTIONS
	
+	sta BOOTLOADER_ADR_MENU_CHOICE
	jsr bootloader_highlight_current_selection
	clr BOOTLOADER_ADR_IDLE_SECONDS
	rts
	
	
bootloader_handle_arrow_left:	
	jsr bootloader_dim_current_selection

	lda BOOTLOADER_ADR_MENU_CHOICE
	sec
	sbc #BOOTLOADER_NUM_MENU_OPTIONS_3
	bpl +
	
	clc
	adc #BOOTLOADER_NUM_MENU_OPTIONS
	
+	sta BOOTLOADER_ADR_MENU_CHOICE
	jsr bootloader_highlight_current_selection
	clr BOOTLOADER_ADR_IDLE_SECONDS
	rts
	
	
bootloader_set_console_pos_for_menu_item:
	cmp #BOOTLOADER_NUM_MENU_OPTIONS_3
	bcc + ; choices 0 - N/3
	cmp #(BOOTLOADER_NUM_MENU_OPTIONS_3*2)
	bcc ++ ; choices N/3 - 2N/3	
	sec
	sbc #(BOOTLOADER_NUM_MENU_OPTIONS_3*2)
	clc
	adc #BOOTLOADER_MENU_TOP_ROW
	asl
	asl
	phb
	bank #VIDEO_RAM_BANK
	sta VIDEO_RAM_CURSOR_PTR_H
	lda #BOOTLOADER_MENU_COLUMN_OFFSET_2
	sta VIDEO_RAM_CURSOR_PTR_L
	plb
	rts
	
+	clc
	adc #BOOTLOADER_MENU_TOP_ROW
	asl
	asl
	phb
	bank #VIDEO_RAM_BANK
	sta VIDEO_RAM_CURSOR_PTR_H
	lda VIDEO_RAM_LEFT_MARGIN
	sta VIDEO_RAM_CURSOR_PTR_L
	plb
	rts
		
++	sec
	sbc #BOOTLOADER_NUM_MENU_OPTIONS_3
	clc
	adc #BOOTLOADER_MENU_TOP_ROW
	asl
	asl
	phb
	bank #VIDEO_RAM_BANK
	sta VIDEO_RAM_CURSOR_PTR_H
	lda #BOOTLOADER_MENU_COLUMN_OFFSET
	sta VIDEO_RAM_CURSOR_PTR_L
	plb
	rts
	
	
bootloader_dim_current_selection:
	lda BOOTLOADER_ADR_MENU_CHOICE
	jsr bootloader_set_console_pos_for_menu_item
	
	; start of table
	lda #<menu_table
	sta BOOTLOADER_ADR_A1_L
	lda #>menu_table
	sta BOOTLOADER_ADR_A1_H
	
	lda BOOTLOADER_ADR_MENU_CHOICE
	asl ; * 2
	tax
	inx
	lda (BOOTLOADER_ADR_A1),X ; high byte
	sta BOOTLOADER_ADR_A0_H
	dex
	lda (BOOTLOADER_ADR_A1),X
	sta BOOTLOADER_ADR_A0_L
			
	; wait for vblank
-	lda VIDEO_ADR_VBLANK
	bmi -	
	
	; print entry name
	ldx #5 ; entry offset to name string	
	jsr bootloader_print_console_string
	rts
	
	
bootloader_highlight_current_selection:
	lda BOOTLOADER_ADR_MENU_CHOICE
	jsr bootloader_set_console_pos_for_menu_item
	
	; start of table
	lda #<menu_table
	sta BOOTLOADER_ADR_A1_L
	lda #>menu_table
	sta BOOTLOADER_ADR_A1_H
	
	lda BOOTLOADER_ADR_MENU_CHOICE
	asl ; * 2
	tax
	inx
	lda (BOOTLOADER_ADR_A1),X ; high byte
	sta BOOTLOADER_ADR_A0_H
	dex
	lda (BOOTLOADER_ADR_A1),X
	sta BOOTLOADER_ADR_A0_L
		
	; wait for vblank
-	lda VIDEO_ADR_VBLANK
	bmi -	
					
	; print entry name
	ldx #5 ; entry offset to name string	
	jsr bootloader_print_console_string_highlighted
	rts
	
		
; --- Bootloader handle USB data	
bootloader_handler_usbdata:
	lda BOOTLOADER_ADR_BOOTSTATE
	cmp #BOOTLOADER_STATE_AWAITING_SIGNATURE0
	bne +
	jmp bootloader_awaiting_signature0
+	cmp #BOOTLOADER_STATE_AWAITING_SIGNATURE1
	bne +
	jmp bootloader_awaiting_signature1
+	cmp #BOOTLOADER_STATE_AWAITING_SIZE0
	bne +
	jmp bootloader_awaiting_size0
+	cmp #BOOTLOADER_STATE_AWAITING_SIZE1
	bne +
	jmp bootloader_awaiting_size1
+	cmp #BOOTLOADER_STATE_AWAITING_SIZE2
	bne +
	jmp bootloader_awaiting_size2	
+	cmp #BOOTLOADER_STATE_AWAITING_DATA
	bne +
	jmp bootloader_awaiting_data
+	cmp #BOOTLOADER_STATE_AWAITING_CHECKSUM
	bne +
	jmp bootloader_awaiting_checksum
+
	; other state: read and discard the data
	lda USB_ADR_DATA
	
	; log the unexpected data
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #<bootloader_unexpecteddatastr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_unexpecteddatastr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
	
	rts
	
bootloader_awaiting_signature0:
	lda USB_ADR_DATA
	cmp #'B'
	beq +
	
	; unexpected data: reset
	; print state message
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #<bootloader_badheaderstr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_badheaderstr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
	
	lda #BOOTLOADER_STATE_AWAITING_SIGNATURE0
	sta BOOTLOADER_ADR_BOOTSTATE
	rts
	
+	; next state
	; print state message
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #<bootloader_readingheader1str
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_readingheader1str
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
	
	lda #BOOTLOADER_STATE_AWAITING_SIGNATURE1
	sta BOOTLOADER_ADR_BOOTSTATE
	rts
	
bootloader_awaiting_signature1:
	lda USB_ADR_DATA
	cmp #'M'
	beq +
	
	; unexpected data: reset
	; print state message
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #<bootloader_badheaderstr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_badheaderstr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
	
	lda #BOOTLOADER_STATE_AWAITING_SIGNATURE0
	sta BOOTLOADER_ADR_BOOTSTATE
	rts
	
+	; next state
	; print state message
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #<bootloader_readingheader2str
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_readingheader2str
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
	
	lda #BOOTLOADER_STATE_AWAITING_SIZE0
	sta BOOTLOADER_ADR_BOOTSTATE
	rts
	
bootloader_awaiting_size0:
	lda USB_ADR_DATA
	sta BOOTLOADER_ADR_A1_L
	sta LCD_WRITE_DECIMAL_IN_L
	
	; next state
	; print state message
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #<bootloader_readingsize1str
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_readingsize1str
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
		
	lda #BOOTLOADER_STATE_AWAITING_SIZE1
	sta BOOTLOADER_ADR_BOOTSTATE
	rts
	
bootloader_awaiting_size1:
	lda USB_ADR_DATA
	sta BOOTLOADER_ADR_A1_H
	sta LCD_WRITE_DECIMAL_IN_H
	
	; next state
	; print state message
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #<bootloader_readingsize2str
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_readingsize2str
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
		
	lda #BOOTLOADER_STATE_AWAITING_SIZE2
	sta BOOTLOADER_ADR_BOOTSTATE
	rts
		
bootloader_awaiting_size2:
	lda USB_ADR_DATA
	and #$01 ; only bank size of 0 or 1 is valid
	sta LCD_WRITE_DECIMAL_IN_B
	clc
	adc BOOTLOADER_ADR_TGTBANK
	sta BOOTLOADER_ADR_A1_B
	
	; convert download size to decmial, and save the result
	jsr lcd_convert_decimal_24
	lda LCD_WRITE_DECIMAL_OUT_L
	sta BOOTLOADER_ADR_LOADSIZE_L
	lda LCD_WRITE_DECIMAL_OUT_H
	sta BOOTLOADER_ADR_LOADSIZE_H
	lda LCD_WRITE_DECIMAL_OUT_B
	sta BOOTLOADER_ADR_LOADSIZE_B
	
	; initialize state
	clr BOOTLOADER_ADR_CHECKSUM
	
	; initialize destination pointer
	clr BOOTLOADER_ADR_DEST_L
	clr BOOTLOADER_ADR_DEST_H
	lda BOOTLOADER_ADR_TGTBANK
	sta BOOTLOADER_ADR_DEST_B
	
	; initialize percent progress
	clr BOOTLOADER_ADR_CURRENT_PERCENT
	clr BOOTLOADER_ADR_PERCENT_PROGRESS_L
	clr BOOTLOADER_ADR_PERCENT_PROGRESS_H
	jsr bootloader_compute_percent_threshhold
	
	; print download progress
	jsr bootloader_write_progress
		
	; padding
-	lda LCD_ADR_CURSOR
	cmp #47
	beq +
	lda #' '
	jsr lcd_write_char	
	bra -
+
	
	; next state
	lda #BOOTLOADER_STATE_AWAITING_DATA
	sta BOOTLOADER_ADR_BOOTSTATE
	rts
		
bootloader_awaiting_data:
	; save SP and setup BOOTLOADER_ADR_DEST
	tsr
	sta BOOTLOADER_ADR_STACKSAVE_LO
	sty BOOTLOADER_ADR_STACKSAVE_HI
	stx BOOTLOADER_ADR_STACKSAVE_BANK
	lda BOOTLOADER_ADR_DEST_L
	ldy BOOTLOADER_ADR_DEST_H
	ldx BOOTLOADER_ADR_DEST_B
	trs	
	
	lda USB_ADR_DATA
	sta_s ; store byte and increment target pointer
		
	; update checksum
	clc
	adc BOOTLOADER_ADR_CHECKSUM
	sta BOOTLOADER_ADR_CHECKSUM
			
	; save BOOTLOADER_ADR_DEST and setup SP
	tsr
	sta BOOTLOADER_ADR_DEST_L
	sty BOOTLOADER_ADR_DEST_H
	stx BOOTLOADER_ADR_DEST_B
	lda BOOTLOADER_ADR_STACKSAVE_LO
	ldy BOOTLOADER_ADR_STACKSAVE_HI
	ldx BOOTLOADER_ADR_STACKSAVE_BANK
	trs	
		
	; update progress
	inc BOOTLOADER_ADR_PERCENT_PROGRESS_L
	bne +
	inc BOOTLOADER_ADR_PERCENT_PROGRESS_H
	
	; compare to threshhold
+	lda BOOTLOADER_ADR_PERCENT_PROGRESS_H
	cmp BOOTLOADER_ADR_PERCENT_THRESHHOLD_H
	bne +
	lda BOOTLOADER_ADR_PERCENT_PROGRESS_L
	cmp BOOTLOADER_ADR_PERCENT_THRESHHOLD_L
	bne +
	
	; step to new percent
	lda #5
	clc 	
	adc BOOTLOADER_ADR_CURRENT_PERCENT
	sta BOOTLOADER_ADR_CURRENT_PERCENT
	
	clr BOOTLOADER_ADR_PERCENT_PROGRESS_L
	clr BOOTLOADER_ADR_PERCENT_PROGRESS_H
	jsr bootloader_write_progress
+
		
	; finished?
	lda BOOTLOADER_ADR_A1_L
	cmp BOOTLOADER_ADR_DEST_L
	bne +
	lda BOOTLOADER_ADR_A1_H
	cmp BOOTLOADER_ADR_DEST_H 
	bne +
	lda BOOTLOADER_ADR_A1_B
	cmp BOOTLOADER_ADR_DEST_B 
	bne +
	
	; finished: go to next state
	
	; print state message
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #<bootloader_awaitingchecksumstr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_awaitingchecksumstr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
	
	lda #BOOTLOADER_STATE_AWAITING_CHECKSUM
	sta BOOTLOADER_ADR_BOOTSTATE
	rts
	
+	; not finished: stay in this state
	rts	
		
bootloader_awaiting_checksum:
	lda USB_ADR_DATA
	cmp BOOTLOADER_ADR_CHECKSUM
	beq +
	
	; bad checksum			
	; print state message
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #<bootloader_badchecksumstr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_badchecksumstr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
	
	lda #BOOTLOADER_STATE_AWAITING_SIGNATURE0
	sta BOOTLOADER_ADR_BOOTSTATE
	rts

+	; checksum OK: boot
	jmp bootloader_boot
	 	
	 	
; --- Bootloader computer percent threshhold
bootloader_compute_percent_threshhold:
	; save A1
	lda BOOTLOADER_ADR_A1_L
	pha
	lda BOOTLOADER_ADR_A1_H
	pha
	lda BOOTLOADER_ADR_A1_B
	pha
	
	; subtract target bank from A1_B
	sec
	sbc BOOTLOADER_ADR_TGTBANK
	sta BOOTLOADER_ADR_A1_B
	
	; clear the count
	clr BOOTLOADER_ADR_PERCENT_THRESHHOLD_L
	clr BOOTLOADER_ADR_PERCENT_THRESHHOLD_H
			
	; repeatedly subtract 20 from the download size, counting the number of subtractions
-	lda BOOTLOADER_ADR_A1_L
	sec
	sbc #20
	sta BOOTLOADER_ADR_A1_L
	bcs +
	
	lda BOOTLOADER_ADR_A1_H
	sec
	sbc #1
	sta BOOTLOADER_ADR_A1_H
	bcs +		
	
	lda BOOTLOADER_ADR_A1_B
	sec
	sbc #1
	sta BOOTLOADER_ADR_A1_B
	
	; increment count
+	inc BOOTLOADER_ADR_PERCENT_THRESHHOLD_L
	bne +
	inc BOOTLOADER_ADR_PERCENT_THRESHHOLD_H		
	
+	bcs -

	pla 
	sta BOOTLOADER_ADR_A1_B
	pla
	sta BOOTLOADER_ADR_A1_H
	pla
	sta BOOTLOADER_ADR_A1_L
	
	rts
	
		 	
; --- Bootloader write progress	 	
bootloader_write_progress:
	; clear status line
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #'>'
	jsr lcd_write_char
	
	; total size
	lda BOOTLOADER_ADR_LOADSIZE_L
	sta LCD_WRITE_DECIMAL_OUT_L
	lda BOOTLOADER_ADR_LOADSIZE_H
	sta LCD_WRITE_DECIMAL_OUT_H
	lda BOOTLOADER_ADR_LOADSIZE_B
	sta LCD_WRITE_DECIMAL_OUT_B
	clr LCD_WRITE_DECIMAL_OUT_BB
	jsr lcd_write_decimal_result
	
	; bytes
	lda #<bootloader_readingbytesstr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_readingbytesstr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
		
	; percent number
	lda BOOTLOADER_ADR_CURRENT_PERCENT
	sta LCD_WRITE_DECIMAL_IN_L
	clr LCD_WRITE_DECIMAL_IN_H
	clr LCD_WRITE_DECIMAL_IN_B
	jsr lcd_convert_decimal_24
	jsr lcd_write_decimal_result
	
	; percent sign
	lda #<bootloader_readingpercentstr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_readingpercentstr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
	
	rts
		
; --- Bootloader boot	
; Boot the program in the target bank, at address $XX0000
bootloader_boot:	
	; print state message
	ldx #24
	stx LCD_ADR_CURSOR
	jsr lcd_sync_cursor
	
	lda #<bootloader_bootingstr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_bootingstr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_string
	
	sei ; disable interrupts
	
	; create a program in page 0 to long jump to the loaded program at $XX0000
	lda #$5C ; JML
	sta $00FFFC
	lda #$00
	sta $00FFFD
	lda #$00
	sta $00FFFE
	lda BOOTLOADER_ADR_TGTBANK
	sta $00FFFF
	
	; init SP to $XXFFFF
	ldx BOOTLOADER_ADR_TGTBANK
	ldy #$FF
	lda #$FF
	trs	
	
	lda BOOTLOADER_ADR_TGTBANK
	bank 
	
	; begin program execution by jumping to the page 0 program
	jml $00FFFC
	
	
; --- Blink wait
blink_wait:
	lda #$a0
--	ldx #$ff
-	dex
	bne -
	dec
	bne --
	rts
				
; --- Bootloader builtin boot	
; Load the selected built-in program to the target bank, and run it.
bootloader_builtin_boot:
	cli ; re-enable interrupts, so key-up will be processed
	
	; wait for break
-	lda KEYBOARD_ADR_NOT_BREAK_FLAG
	bne -
	
	; wait for non-break
-	lda KEYBOARD_ADR_NOT_BREAK_FLAG
	beq -
		
	sei ; disable interrupts
	
bootloader_builtin_boot_nokey:	
	; highlight the selected program
	jsr bootloader_dim_current_selection
	jsr blink_wait
	jsr bootloader_highlight_current_selection
	jsr blink_wait
	jsr bootloader_dim_current_selection
	jsr blink_wait
	jsr bootloader_highlight_current_selection
	jsr blink_wait
	jsr bootloader_dim_current_selection
	jsr blink_wait
	jsr bootloader_highlight_current_selection
	jsr blink_wait
	
	; start of table
	lda #<menu_table
	sta BOOTLOADER_ADR_A1_L
	lda #>menu_table
	sta BOOTLOADER_ADR_A1_H
	
	; add 2*entry_index
	ldx BOOTLOADER_ADR_MENU_CHOICE
	
-	cpx #0
	beq ++
	
	dex
	lda #2
	clc
	adc BOOTLOADER_ADR_A1_L
	sta BOOTLOADER_ADR_A1_L
	bcc -
	inc BOOTLOADER_ADR_A1_H
	bra -
	
++	; load entry pointer
	ldx #0
	lda (BOOTLOADER_ADR_A1),X
	sta BOOTLOADER_ADR_TEMP
	inx
	lda (BOOTLOADER_ADR_A1),X
	sta BOOTLOADER_ADR_A1_H
	lda BOOTLOADER_ADR_TEMP
	sta BOOTLOADER_ADR_A1_L	
	
	; load src pointer to A0
	ldx #0
	lda (BOOTLOADER_ADR_A1),X
	sta BOOTLOADER_ADR_TEMP
	lda APP_ADR_PROGRAM_BANK 
	clc
	adc BOOTLOADER_ADR_TEMP
	sta BOOTLOADER_ADR_A0_B
	
	; Change source bank index by +16 if program bank is 0
	lda APP_ADR_PROGRAM_BANK
	bne +
	lda #16
	clc
	adc BOOTLOADER_ADR_A0_B
	sta BOOTLOADER_ADR_A0_B
	
+	inx
	lda (BOOTLOADER_ADR_A1),X
	sta BOOTLOADER_ADR_A0_L
	inx
	lda (BOOTLOADER_ADR_A1),X
	sta BOOTLOADER_ADR_A0_H
	
	; load end pointer to A1 
	inx
	lda (BOOTLOADER_ADR_A1),X
	tay ; low byte
	
	inx
	lda (BOOTLOADER_ADR_A1),X
	sta BOOTLOADER_ADR_A1_H
	sty BOOTLOADER_ADR_A1_L
	; built-in programs are less than 64K, so end bank = target bank
	lda BOOTLOADER_ADR_TGTBANK
	sta BOOTLOADER_ADR_A1_B
	
	; initialize DEST ptr
	clr BOOTLOADER_ADR_DEST_L
	clr BOOTLOADER_ADR_DEST_H
	lda BOOTLOADER_ADR_TGTBANK
	sta BOOTLOADER_ADR_DEST_B
	
	; -------
	; copy bytes from A0++ to DEST, until DEST == A1
	; save SP 
-	tsr
	sta BOOTLOADER_ADR_STACKSAVE_LO
	sty BOOTLOADER_ADR_STACKSAVE_HI
	stx BOOTLOADER_ADR_STACKSAVE_BANK
	
	; get next byte
	lda BOOTLOADER_ADR_A0_L
	ldy BOOTLOADER_ADR_A0_H
	ldx BOOTLOADER_ADR_A0_B
	trs	
	lda_s ; get byte
	sta BOOTLOADER_ADR_TEMP 
	; update A0
	tsr
	sta BOOTLOADER_ADR_A0_L
	sty BOOTLOADER_ADR_A0_H
	stx BOOTLOADER_ADR_A0_B	
	
	; setup dest ptr
	lda BOOTLOADER_ADR_DEST_L
	ldy BOOTLOADER_ADR_DEST_H
	ldx BOOTLOADER_ADR_DEST_B
	trs	
	lda BOOTLOADER_ADR_TEMP
	sta_s ; store byte and increment target pointer			
	; save BOOTLOADER_ADR_DEST and restore SP
	tsr
	sta BOOTLOADER_ADR_DEST_L
	sty BOOTLOADER_ADR_DEST_H
	stx BOOTLOADER_ADR_DEST_B
	lda BOOTLOADER_ADR_STACKSAVE_LO
	ldy BOOTLOADER_ADR_STACKSAVE_HI
	ldx BOOTLOADER_ADR_STACKSAVE_BANK
	trs	
		
	; finished?
	lda BOOTLOADER_ADR_A1_L
	cmp BOOTLOADER_ADR_DEST_L
	bne +
	lda BOOTLOADER_ADR_A1_H
	cmp BOOTLOADER_ADR_DEST_H 
	bne +
	lda BOOTLOADER_ADR_A1_B
	cmp BOOTLOADER_ADR_DEST_B 
	bne +	
	
	jsr bootloader_boot
+	jmp -
	
	
; --- Debug print string 
; Print the null-terminated string pointed to by BOOTLOADER_ADR_A0 to the LCD 
bootloader_print_string:
	ldx #$00
	
-	lda (BOOTLOADER_ADR_A0),X
	beq +
	
	phx
	jsr lcd_write_char
	plx
	inx
	
	; terminate after 256 chars, even if NULL is not reached
	bne -

+	rts


; --- bootloader_print_console_string
; @param: X = string offset
; @param: Y = option index
; Print the null-terminated string pointed to by BOOTLOADER_ADR_A0 to the console 
bootloader_print_console_string:
-	lda (BOOTLOADER_ADR_A0),X
	beq +
	
	phx
	jsr console_write_char
	plx
	inx
	
	; terminate after 256 chars, even if NULL is not reached
	bne -

+	rts

; --- bootloader_print_console_string highlighted
; @param: X = string offset
; @param: Y = option index
; Print the null-terminated string pointed to by BOOTLOADER_ADR_A0 to the console 
bootloader_print_console_string_highlighted:
-	lda (BOOTLOADER_ADR_A0),X
	beq ++
	
	phx
	cmp #13
	beq +
	ora #$80
+	jsr console_write_char
	plx
	inx
	
	; terminate after 256 chars, even if NULL is not reached
	bne -

++	rts

; --- Bootloader print menu
bootloader_print_menu:
	lda #13
	jsr console_set_row
	
	; divider
	ldx #61	
-	lda #18
	phx
	jsr console_write_char
	plx
	dex
	bpl -
	lda #13
	jsr console_write_char
		
	; instructions
	lda #<bootloader_menuinstructionstr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_menuinstructionstr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_console_string
	
	; blank line
	lda #13
	jsr console_write_char
	
	; table header
	lda #(BOOTLOADER_MENU_TOP_ROW-1)
	ldy #$03 ; dark green on black
	jsr console_set_row_palette
	lda #<bootloader_tableheaderstr
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_tableheaderstr
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_console_string
	
	
	; print the table
	lda #<menu_table
	sta BOOTLOADER_ADR_A1_L
	lda #>menu_table
	sta BOOTLOADER_ADR_A1_H
		
	; entry number
	ldy #0
	
	; set palette
-	tya
	cmp #BOOTLOADER_NUM_MENU_OPTIONS_3
	bcs +
	clc
	adc #BOOTLOADER_MENU_TOP_ROW
	phy
	ldy #$0A ; green on black
	jsr console_set_row_palette
	ply
			
	; read table address
+	tya
	cmp #BOOTLOADER_NUM_MENU_OPTIONS
	beq +++ ; done?
	
	; set screen pos
	jsr bootloader_set_console_pos_for_menu_item
	
	ldx #1
	lda (BOOTLOADER_ADR_A1),X ; high byte
	sta BOOTLOADER_ADR_A0_H
	dex
	lda (BOOTLOADER_ADR_A1),X
	sta BOOTLOADER_ADR_A0_L
			
	; print entry name
	ldx #5 ; entry offset to name string
	phy
	
	;Y == BOOTLOADER_ADR_MENU_CHOICE?
	tya
	cmp BOOTLOADER_ADR_MENU_CHOICE
	beq +
	
	jsr bootloader_print_console_string
	bra ++
	
+	jsr bootloader_print_console_string_highlighted
	
++	ply
	
	; next table entry
	iny
	lda #2
	clc
	adc BOOTLOADER_ADR_A1_L
	sta BOOTLOADER_ADR_A1_L
	bcc +
	inc BOOTLOADER_ADR_A1_H
+	bra -
	
+++	; blank line
	lda #13
	jsr console_write_char
	
	; instructions
	lda #<bootloader_menuinstruction2str
	sta BOOTLOADER_ADR_A0_L
	lda #>bootloader_menuinstruction2str
	sta BOOTLOADER_ADR_A0_H
	jsr bootloader_print_console_string
	
	rts




; --- LCD convert decimal 24
; On entry, converts a 24-bit binary number in LCD_WRITE_DECIMAL_IN to a 32-bit BCD number in LCD_WRITE_DECIMAL_OUT.
lcd_convert_decimal_24:   
	clr LCD_WRITE_DECIMAL_OUT_BB   ; Begin by storing 0 in the output bytes.
	clr LCD_WRITE_DECIMAL_OUT_B
	clr LCD_WRITE_DECIMAL_OUT_H 
	clr LCD_WRITE_DECIMAL_OUT_L     
	ldx #24

-	asl LCD_WRITE_DECIMAL_IN_L
	rol LCD_WRITE_DECIMAL_IN_H
	rol LCD_WRITE_DECIMAL_IN_B
	rol LCD_WRITE_DECIMAL_OUT_L
	rol LCD_WRITE_DECIMAL_OUT_H
	rol LCD_WRITE_DECIMAL_OUT_B
	rol LCD_WRITE_DECIMAL_OUT_BB

	dex             ; The shifting will happen 24 times.  After
	beq ++          ; the last shift, you don't check for digits of 5 or more.
    
	jsr lcd_write_decimal_check_overflow	
	bra -       

++	rts


; --- LCD write decimal result
; On entry, writes the 32-bit BCD number in LCD_WRITE_DECIMAL_OUT to the LCD.
lcd_write_decimal_result: 
	clr LCD_WRITE_DECIMAL_FLAG
	lda LCD_WRITE_DECIMAL_OUT_BB
	jsr lcd_write_bcd
	lda LCD_WRITE_DECIMAL_OUT_B
	jsr lcd_write_bcd
	lda LCD_WRITE_DECIMAL_OUT_H
	jsr lcd_write_bcd
	lda LCD_WRITE_DECIMAL_OUT_L
	jsr lcd_write_bcd_last_byte
	rts
	

lcd_write_decimal_check_overflow:
	lda LCD_WRITE_DECIMAL_OUT_L
	and #$0F
	cmp #5
	bmi +

	clc
	lda LCD_WRITE_DECIMAL_OUT_L
	adc #3
	sta LCD_WRITE_DECIMAL_OUT_L

+	lda LCD_WRITE_DECIMAL_OUT_L
	cmp #$50
	bmi +

	clc
	adc #$30
	sta LCD_WRITE_DECIMAL_OUT_L

+	lda LCD_WRITE_DECIMAL_OUT_H
	and #$0F
	cmp #5
	bmi +
	
	clc
	lda LCD_WRITE_DECIMAL_OUT_H
	adc #3
	sta LCD_WRITE_DECIMAL_OUT_H
	
+	lda LCD_WRITE_DECIMAL_OUT_H
	cmp #$50
	bmi +

	clc
	adc #$30
	sta LCD_WRITE_DECIMAL_OUT_H
		
+	lda LCD_WRITE_DECIMAL_OUT_B
	and #$0F
	cmp #5
	bmi +
	
	clc
	lda LCD_WRITE_DECIMAL_OUT_B
	adc #3
	sta LCD_WRITE_DECIMAL_OUT_B
	
+	lda LCD_WRITE_DECIMAL_OUT_B
	cmp #$50
	bmi +

	clc
	adc #$30
	sta LCD_WRITE_DECIMAL_OUT_B
	
+	lda LCD_WRITE_DECIMAL_OUT_BB
	and #$0F
	cmp #5
	bmi +
	
	clc
	lda LCD_WRITE_DECIMAL_OUT_BB
	adc #3
	sta LCD_WRITE_DECIMAL_OUT_BB
	
+	lda LCD_WRITE_DECIMAL_OUT_BB
	cmp #$50
	bmi +

	clc
	adc #$30
	sta LCD_WRITE_DECIMAL_OUT_BB
	
+	rts

	
; --- LCD write bcd	
lcd_write_bcd:
	ldx LCD_WRITE_DECIMAL_FLAG
	beq +
	; output already started: write byte
	jsr lcd_write_hex
	rts
	
+	pha
	and #$F0
	beq +
	pla
	; byte begins with non-zero: set flag and write byte
	ldx #1
	stx LCD_WRITE_DECIMAL_FLAG	
	jsr lcd_write_hex
	rts
	
+	pla
	pha
	and #$0F
	beq +
	pla
	; byte ends with non-zero: set flag and write nibble
	ldx #1
	stx LCD_WRITE_DECIMAL_FLAG	
	jsr lcd_write_hex_nibble
	rts		
	
+	pla
	rts	
	

; --- LCD write bcd last byte	
lcd_write_bcd_last_byte:
	ldx LCD_WRITE_DECIMAL_FLAG
	beq +
	; output already started: write byte
	jsr lcd_write_hex
	rts
	
+	pha
	and #$F0
	beq +
	pla
	; byte begins with non-zero: set flag and write byte
	ldx #1
	stx LCD_WRITE_DECIMAL_FLAG	
	jsr lcd_write_hex
	rts
	
+	; write last nibble regardless of its value
	pla
	ldx #1
	stx LCD_WRITE_DECIMAL_FLAG	
	jsr lcd_write_hex_nibble
	rts		
	
	
; --- Next demo loop
run_next_demo_loop:
	lda GLOBAL_ADR_NEXT_DEMO_LOOP	
	cmp #BOOTLOADER_NUM_DEMO_LOOP_ENTRIES
	bcc +
	lda #0
+	tax
	inc
	sta GLOBAL_ADR_NEXT_DEMO_LOOP
	lda demo_loop_entries,X
	pha
	jsr bootloader_dim_current_selection
	pla
	sta BOOTLOADER_ADR_MENU_CHOICE	
	jmp bootloader_builtin_boot_nokey
	
		
bootloader_initstr:	
!text "BMOW bootloader", 0

bootloader_usbdownstr:	
!text ">USB not ready         ", 0

bootloader_usbupstr:	
!text ">Ready for download    ", 0

bootloader_readingbytesstr:	
!text " byte file: ", 0

bootloader_readingpercentstr:	
!text "%", 0

bootloader_unexpecteddatastr:
!text ">Unknown state         ", 0

bootloader_awaitingchecksumstr:
!text ">Reading checksum      ", 0

bootloader_badchecksumstr:
!text ">Bad checksum          ", 0

bootloader_bootingstr:
!text ">Starting program      ", 0

bootloader_readingheader1str:
!text ">Reading header 1/2    ", 0

bootloader_readingheader2str:
!text ">Reading header 2/2    ", 0

bootloader_readingsize1str:
!text ">Reading size 1/3      ", 0

bootloader_readingsize2str:
!text ">Reading size 2/3      ", 0

bootloader_badheaderstr:
!text ">Bad header            ", 0

bootloader_menuinstructionstr:
!text "Begin download now, or select a built-in program to start:", 13, 0

bootloader_tableheaderstr:
!text "Interactive              Video                    Audio", 13, 0

bootloader_menuinstruction2str:
!text "Use the arrow keys to move the highlight to your choice.", 13
!text "Press ENTER to choose.", 13, 0

; --- Boot menu options 
menu_option1:
!byte 0 ; source bank offset
!word system_monitor_data
!word (system_monitor_data_end - system_monitor_data)
!text "System Monitor", 13, 0

menu_option2: 
!byte 0 ; source bank offset
!word microchess_data
!word (microchess_data_end - microchess_data)
!text "Microchess", 13, 0

menu_option3: 
!byte 0 ; source bank offset
!word basic_data
!word (basic_data_end - basic_data)
!text "BASIC", 13, 0

menu_option4: 
!byte 0 ; source bank offset
!word mixed_mode_data
!word (mixed_mode_data_end - mixed_mode_data)
!text "Mixed Modes", 13, 0

menu_option5: 
!byte 1 ; source bank offset
!word dateandtime_data
!word (dateandtime_data_end - dateandtime_data)
!text "Date and Time", 13, 0

menu_option6: 
!byte 1 ; source bank offset
!word color_depth_demo_data
!word (color_depth_demo_data_end - color_depth_demo_data)
!text "Color Depths", 13, 0

menu_option7: 
!byte 1 ; source bank offset
!word xxl_data
!word (xxl_data_end - xxl_data)
!text "XXL", 13, 0

menu_option8: 
!byte 2 ; source bank offset
!word powerm_data
!word (powerm_data_end - powerm_data)
!text "Power M", 13, 0

menu_option9: 
!byte 3 ; source bank offset
!word hi_res_demo_data
!word (hi_res_demo_data_end - hi_res_demo_data)
!text "Hi-Res", 13, 0

menu_option10: 
!byte 3 ; source bank offset
!word c256_color_demo_data
!word (c256_color_demo_data_end - c256_color_demo_data)
!text "256 Colors", 13, 0

menu_option11: 
!byte 4 ; source bank offset
!word agentx_data
!word (agentx_data_end - agentx_data)
!text "Agent X II", 13, 0

menu_option12: 
!byte 4 ; source bank offset
!word destine_data
!word (destine_data_end - destine_data)
!text "Destine", 13, 0

menu_table:
!word menu_option1
!word menu_option5
!word menu_option3
!word menu_option2
!word menu_option4
!word menu_option6
!word menu_option9
!word menu_option10
!word menu_option7
!word menu_option8
!word menu_option12
!word menu_option11

demo_loop_entries:
!byte 3, 4, 11, 5, 10, 2, 8, 6, 9, 7

bootsound_data:
!source "bootsound.s"

code_end:
!if code_end > (16384-256) {
	!error "Bootloader code segment overlaps memory-mapped hardware devices"
}

!align 16383, 0, 0

; --- Program data 
system_monitor_data:
!binary "bmow_monitor.bin"
system_monitor_data_end:

microchess_data:
!binary "microchess.bin"
microchess_data_end:

basic_data:
!binary "..\..\msbasic\bmow\bmow.bin"
basic_data_end:

mixed_mode_data:
!binary "mixed_mode_demo.bin"
mixed_mode_data_end:

lo_data_end:
!if lo_data_end > (65536-256) {
	!error "Bootloader lo data segment overlaps RAM"
}

!align 65535, 0, 0

