; Boot logo for BMOW
; Displays a logo image

; driver RAM addresses
BOOTLOGO_BASE = MONITOR_ADR_END+1
BOOTLOGO_ADR_A0_L = BOOTLOGO_BASE
BOOTLOGO_ADR_A0_H = BOOTLOGO_BASE+1
BOOTLOGO_ADR_A0 = BOOTLOGO_BASE
BOOTLOGO_ADR_FORMAT = BOOTLOGO_BASE+2
BOOTLOGO_ADR_END = BOOTLOGO_ADR_FORMAT

; --- bootlogo init
bootlogo_init:	
	jsr console_standard_init

	jsr console_enable_mixed_mode
	lda #$07
	jsr console_set_row
	jsr bootlogo_draw			
		
	; display BMOW
	lda #<bootlogo_bmowstr
	sta CONSOLE_ADR_A0_L
	lda #>bootlogo_bmowstr
	sta CONSOLE_ADR_A0_H
	jsr console_write_string_line
		
	; display ROM version
	lda #<bootlogo_buildstr
	sta CONSOLE_ADR_A0_L
	lda #>bootlogo_buildstr
	sta CONSOLE_ADR_A0_H
	jsr console_write_string

	lda #<bootlogo_buildate
	sta CONSOLE_ADR_A0_L
	lda #>bootlogo_buildate
	sta CONSOLE_ADR_A0_H
	jsr console_write_string
			
	lda #<bootlogo_buildtime
	sta CONSOLE_ADR_A0_L
	lda #>bootlogo_buildtime
	sta CONSOLE_ADR_A0_H
	jsr console_write_string_line
	
	; display installed memory
	lda #<bootlogo_memorystr
	sta CONSOLE_ADR_A0_L
	lda #>bootlogo_memorystr
	sta CONSOLE_ADR_A0_H
	jsr console_write_string_line
		
	; display current time
	lda #<bootlogo_currenttimestr
	sta CONSOLE_ADR_A0_L
	lda #>bootlogo_currenttimestr
	sta CONSOLE_ADR_A0_H
	jsr console_write_string
				
	jsr rtc_print_time
	
	lda #$0D		
	jsr console_write_char
	
	rts
	


; --- bootlogo draw
; Logo image width is assumed to be full-screen width.
; Bit depth and height are determined from the image data.
; Sets the current video mode to 240 lines, mode byte per line.
bootlogo_draw:	
	; set the video resolution
	lda #$00 ; 240 lines, mode byte per line
	sta VIDEO_ADR_VSIZE 
	
	; determine the bit depth
	lda bootlogo_image+28
	cmp #1
	bne +
	clr BOOTLOGO_ADR_FORMAT
	bra ++
+	cmp #2
	bne +
	lda #$10
	sta BOOTLOGO_ADR_FORMAT
	bra ++
+	cmp #4
	bne +
	lda #$20
	sta BOOTLOGO_ADR_FORMAT
	bra ++
+	cmp #8
	bne +
	lda #$30
	sta BOOTLOGO_ADR_FORMAT
				
++	; setup image palette
	jsr bootlogo_install_palette
   	
	; init source pointer
	jsr bootlogo_init_source_pointer
	
	; init dest pointer
	phb
	bank #VIDEO_RAM_BANK
	clr VIDEO_RAM_A0_L
	clr VIDEO_RAM_A0_H
	plb

	; get the image height
	ldy bootlogo_image+22
	
--	; copy one row

	; copy the last byte of the source row to dest row index 0
	ldx #$7F
	lda (BOOTLOGO_ADR_A0), X
	phb
	bank #VIDEO_RAM_BANK
	ldx #$00
	sta (VIDEO_RAM_A0), X
	plb
		
	; set the line mode 
	ldx #$01
	lda BOOTLOGO_ADR_FORMAT
	phb
	bank #VIDEO_RAM_BANK
	sta (VIDEO_RAM_A0), X
	plb
	
	; copy the rest of the line's image data 
	; (byte 0 and 1 of source image is not drawn)
	ldx #$03
-	dex
	lda (BOOTLOGO_ADR_A0), X
	inx
	phb
	bank #VIDEO_RAM_BANK
	sta (VIDEO_RAM_A0), X
	plb
	inx
	bpl -	
	
	dey
	beq ++
	
	; decrement source pointer
	lda BOOTLOGO_ADR_A0_L
	sec
	sbc #$80
	sta BOOTLOGO_ADR_A0_L
	bcs +
	dec BOOTLOGO_ADR_A0_H
+	
	; increment dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda VIDEO_RAM_A0_L
	eor #$80
	sta VIDEO_RAM_A0_L
	bne +
	inc VIDEO_RAM_A0_H	
+	plb
	bra --
	
++	jsr bootlogo_init_text_region

	; set the line mode for the 1st non-visible line
	lda BOOTLOGO_ADR_FORMAT
	phb
	bank #VIDEO_RAM_BANK
	sta $7800 
	sta $7A80
	sta $7A81
	sta $0000 ; hack-o-rama: this introduces a drawing problem
	plb
	
	rts
		

; --- bootlogo install palette
;initialize the color palette from offset 54 of the BMP
bootlogo_install_palette:	
	; source pointer
	lda #<(bootlogo_image+54)
	sta BOOTLOGO_ADR_A0_L
	lda #>(bootlogo_image+54)
	sta BOOTLOGO_ADR_A0_H
	
	ldy #$00
		
-	tya
	sta VIDEO_ADR_PAL_INDEX
	
	;entries are BGRA format
	ldx #$02
	lda (BOOTLOGO_ADR_A0),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; R
	dex
	lda (BOOTLOGO_ADR_A0),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; G
	dex
	lda (BOOTLOGO_ADR_A0),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; B
	
	; increment source pointer + 4
	lda #$04
	clc
	adc BOOTLOGO_ADR_A0_L
	sta BOOTLOGO_ADR_A0_L
	bcc +
	inc BOOTLOGO_ADR_A0_H	
			
+	iny
	cpy bootlogo_image+46 ; all colors installed?
	
	bne -
	
	rts
	
	
; --- bootlogo init source pointer
bootlogo_init_source_pointer:   	
	; init source pointer
	lda #<(bootlogo_image+58)
	sta BOOTLOGO_ADR_A0_L
	lda #>(bootlogo_image+58)
	sta BOOTLOGO_ADR_A0_H
	
	; add 4 * number of palette entries
	lda bootlogo_image+46
	dec
	asl
	asl
	clc
	adc BOOTLOGO_ADR_A0_L
	sta BOOTLOGO_ADR_A0_L
	bcc +
	inc BOOTLOGO_ADR_A0_H
+	lda bootlogo_image+46
	dec
	lsr
	lsr
	lsr
	lsr
	lsr
	lsr
	clc
	adc BOOTLOGO_ADR_A0_H
	sta BOOTLOGO_ADR_A0_H	
	
	; get the image height
	ldy bootlogo_image+22
	
	; start on last line of source image
	; low byte
	tya
	dec
	and #$01
	beq +
	lda BOOTLOGO_ADR_A0_L
	clc
	adc #$80
	sta BOOTLOGO_ADR_A0_L	
	bcc +
	inc BOOTLOGO_ADR_A0_H
+
	; high byte
	tya
	dec
	lsr
	clc
	adc BOOTLOGO_ADR_A0_H
	sta BOOTLOGO_ADR_A0_H

	rts
	
	
; --- bootlogo init text region
bootlogo_init_text_region:
	; get the image height
	ldx #22
	ldy bootlogo_image, X
	
	phb
	bank #VIDEO_RAM_BANK
	
	; init dest pointer
	; assume row count is a multiple of 8
	clr VIDEO_RAM_A0_L
	tya
	lsr
	sta VIDEO_RAM_A0_H
	
	; set mode for one text row
-	ldx #$00
	lda #$41 ; text, 1bpp, palette 1
	sta (VIDEO_RAM_A0), X
	inx
	sta (VIDEO_RAM_A0), X
	
	; next row
	lda #$04
	clc
	adc VIDEO_RAM_A0_H
	sta VIDEO_RAM_A0_H
	
	cmp #(30*4)
	bne -
	
	plb
	rts
	
bootlogo_image:
!binary "bootlogo_16color.bmp"

bootlogo_bmowstr:	
!text "Big Mess o' Wires", 0	

bootlogo_memorystr:	
!text "512K bytes ROM, 512K bytes RAM installed", 0	

bootlogo_currenttimestr:	
!text "The current time is ", 0	
	
bootlogo_buildstr:
!text "Boot ROM version ", 0	
	
bootlogo_buildate:
!binary "bootloader_builddate.txt"

bootlogo_buildtime:
!binary "bootloader_buildtime.txt"