* = $0000

; --- Jump to init
	nop
	nop
	nop
	jmp video_demo_init

; reserve the remainder of zero page for run-time data
VIDEO_DEMO_PTR:
VIDEO_DEMO_PTR_L: !fi 1
VIDEO_DEMO_PTR_H  !fi 1
VIDEO_ADR_SPSAVE: !fi 3
VIDEO_ADR_A0: 
VIDEO_ADR_A0_L: !fi 1
VIDEO_ADR_A0_H: !fi 1
VIDEO_ADR_TEMP: !fi 1

* = $0100
; includes
DRIVER_RAM_BASE = video_demo_end_of_program 
!source "app.s"
!source "lcd.s"
!source "keyhit.s"
!source "video_defs.s"
!source "audio.s"


; --- init
video_demo_init:	
	lda #<video_demo_isr
	sta $00FFFE
	lda #>video_demo_isr
	sta $00FFFF	
	+app_init
			
	jsr lcd_standard_init
	jsr keyhit_standard_init
	jsr print_title
	
	cli
	
	; start the program
	jsr video_image_test
	
	; wait for key
-	lda KEYBOARD_ADR_KEY_READY
	beq -
	
	; exit
	jml app_warm_boot

; --- Interrupt service routine	
video_demo_isr:
	php
	phb
	pha
	phy
	phx
	
	lda APP_ADR_PROGRAM_BANK
	bank
	
	; was the interrupt triggered by the keyboard?
	lda KEYBOARD_ADR_CONTROL
	and #$1
	bne +
	lda KEYBOARD_ADR_DATA
	jsr keyboard_handle_scan_code
			
+	plx
	ply
	pla
	plb
	plp
	
	rtl ; returns to page 0 wrapper
		

; --- Print title	 
print_title:
	ldx #$00
	
-	lda demo_titlestr,X
	beq +
	
	phx
	jsr lcd_write_char
	plx
	inx
	
	; terminate after 256 chars, even if NULL is not reached
	bne -

+	rts


demo_titlestr:	
!text "Bouncing image demo", 13, "Press any key to exit", 0


; --- Video zero VRAM
; @noparams
; Clears contents of VRAM to all zeroes.
video_zero_vram:
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$7E
	sta VIDEO_RAM_A0_H
	
--	ldx #$7F
	lda #0
	
-	sta (VIDEO_RAM_A0),X
	dex
	bpl -
	
	; decrement row pointer
	lda VIDEO_RAM_A0_L
	sec
	sbc #$80
	sta VIDEO_RAM_A0_L
	bcs --
	dec VIDEO_RAM_A0_H
	bpl --
	
	plb 
	rts
	
	
; --- Video image test
; @noparams
; 128x100 @ 256 colors
video_image_test:	 
	lda #$00
	sta VIDEO_ADR_VSIZE ; 200 lines, mode byte per line
   
    jsr video_zero_vram
   
	; src pointer
	lda #<(video_image_bmp+56)
	sta VIDEO_ADR_A0_L
	lda #>(video_image_bmp+56)
	sta VIDEO_ADR_A0_H
	
	ldx #$00
	ldy #$00
	
install_palette:	
	tya
	sta VIDEO_ADR_PAL_INDEX
	
	lda (VIDEO_ADR_A0),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; R
	dec VIDEO_ADR_A0_L
	lda VIDEO_ADR_A0_L
	cmp #$FF
	bne +
	dec VIDEO_ADR_A0_H
+	lda (VIDEO_ADR_A0),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; G
	dec VIDEO_ADR_A0_L
	lda VIDEO_ADR_A0_L
	cmp #$FF
	bne +
	dec VIDEO_ADR_A0_H
+	lda (VIDEO_ADR_A0),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; B
	
	; inc 6
	inc VIDEO_ADR_A0_L
	bne +
	inc VIDEO_ADR_A0_H
+	inc VIDEO_ADR_A0_L
	bne +
	inc VIDEO_ADR_A0_H
+	inc VIDEO_ADR_A0_L
	bne +
	inc VIDEO_ADR_A0_H
+	inc VIDEO_ADR_A0_L
	bne +
	inc VIDEO_ADR_A0_H
+	inc VIDEO_ADR_A0_L
	bne +
	inc VIDEO_ADR_A0_H
+	inc VIDEO_ADR_A0_L
	bne +
	inc VIDEO_ADR_A0_H
			
+	iny
	bne install_palette
   
init_mode:   
	phb
	bank #$20
	
	; setup a pointer in $7FFE-$7FFF
	lda #$81
	sta $7FFE
	lda #$63
	sta $7FFF
	
	ldx #$00

-	lda #$30
	sta ($7FFE),X ; 128 columns, 8bpp
	lda $7FFE
	sec
	sbc #$80
	sta $7FFE
	bcs -

	dec $7FFF
	lda $7FFF
	cmp #$FF
	bne -

init_position:
	;bank #$20
    ; img.x
    lda #32
    sta $7FFC
    ; img.y
    lda #50
    sta $7FFD
	
	; dest pointer
	lda #$02
	clc
	adc $7FFC ; add x
	; add $80 if y is even
	sta $7FF8
	lda $7FFD
	and #$01
	bne +
	lda #$80
	clc
	adc $7FF8
	sta $7FF8
	
+	lda #$31
	sta $7FF9
	; add y/2
	lda $7FFD
	lsr
	clc
	adc $7FF9
	sta $7FF9
	
init_direction:
	lda #$02
	sta $7FF6
	lda #$01
	sta $7FF7
	plb
		
draw_image:		
	; dest pointer
	phb
	bank #$20
	lda $7FF8
	sta $7FFE
	lda $7FF9
	sta $7FFF
	plb
	
	; init src pointer
	lda #<(video_image_bmp+1078)
	sta VIDEO_ADR_A0_L
	lda #>(video_image_bmp+1078)
	sta VIDEO_ADR_A0_H
	
	; rownum/2
	phb
	bank #$20
    lda #50
    sta $7FFB
    
!if 1 {
	; adjust dest pointer to end of line (+61 bytes)
	lda $7FFE
	clc
	adc #61
	sta $7FFE
	bcc +
	inc $7FFF
+		
}

    
newline:	
	; phb, bank #$20 in effect

	; save SP
	plb
	tsr
	sta VIDEO_ADR_SPSAVE
	stx VIDEO_ADR_SPSAVE+1
	sty VIDEO_ADR_SPSAVE+2
	phb
	bank #$20
	
	sei	 ; disable interrupts
	
	; set dest pointer in SP
	ldx #$20
	ldy $7FFF
	lda $7FFE
	trs
	
	; column index
    ldx #63
		
	; restore program bank
	lda APP_ADR_PROGRAM_BANK
	bank
		
newpixel:	
	; wait for vblank	
-	lda VIDEO_ADR_VBLANK	; 4
	bmi -					; 2
	
	; copy 8 pixels
	lda (VIDEO_ADR_A0),X	; 8
	pha						; 2
	dex						; 1
	lda (VIDEO_ADR_A0),X	; 8
	pha						; 2
	dex						; 1
	lda (VIDEO_ADR_A0),X	; 8
	pha						; 2
	dex						; 1
	lda (VIDEO_ADR_A0),X	; 8
	pha						; 2
	dex						; 1
	lda (VIDEO_ADR_A0),X	; 8
	pha						; 2
	dex						; 1
	lda (VIDEO_ADR_A0),X	; 8
	pha						; 2
	dex						; 1
	lda (VIDEO_ADR_A0),X	; 8
	pha						; 2
	dex						; 1
	lda (VIDEO_ADR_A0),X	; 8
	pha						; 2
	dex						; 1				
	
	bpl newpixel			; 5
	
	; 99 clock cycles for 8 pixel loop = ~12 cycles/pixel	
		
	; restore SP
	lda VIDEO_ADR_SPSAVE
	ldx VIDEO_ADR_SPSAVE+1
	ldy VIDEO_ADR_SPSAVE+2
	trs
	
	cli ; re-enable interrupts
		
	; increment source pointer +64
	lda VIDEO_ADR_A0_L
	clc
	adc #64
	sta VIDEO_ADR_A0_L
	bcc +
	inc VIDEO_ADR_A0_H	
+	
	; compute next line address
	phb
	bank #$20
	lda $7FFE
	sec
	;sbc #(62+128) ; go back 62 pixels to the start of the line, plus 128 to the next line
	sbc #128 ; go back 128 to the next line
	sta $7FFE
	bcs newline
	dec $7FFF
	
	dec $7FFB
	bne newline
	
	; shift the image and run again
	lda $7FF8
	adc $7FF6 ; x += dx
	sta $7FF8
	and #$7F
	cmp #64
	bne +
	lda #$FE ; dx = -2
	sta $7FF6	
+	cmp #04
	bne +
	lda #02 ; dx = 2
	sta $7FF6	
+	

	lda $7FF7
	cmp #$01
	bne yminus
	
	; y += 1
	lda $7FF8
	clc
	adc #$80 ; y += dy
	sta $7FF8
	bcc ydone
	inc $7FF9	
	bra ydone	
	
yminus:
	; y -= 1
	lda $7FF8
	sec
	sbc #$80 ; y += dy
	sta $7FF8
	bcs ydone
	dec $7FF9	
	bra ydone	
	
ydone:	
	lda $7FF9
	cmp #$31
	bne +
	lda #$01 ; dy = 1
	sta $7FF7	
+	cmp #$63
	bne +
	lda #$FF ; dy = -1
	sta $7FF7	
	
+
	plb
	
	; check for key
	lda KEYBOARD_ADR_KEY_READY
	beq +
	
	; exit
	jml app_warm_boot
	
+	jmp draw_image
	
	rts	
	
	 
video_image_bmp:
!binary "testimg_64x100.bmp"		
	
video_demo_end_of_program:
	