* = $0000

; --- Jump to init
	nop
	nop
	nop
	jmp color_depth_demo_init

; reserve the remainder of zero page for run-time data
DEMO_PTR:
DEMO_PTR_L: !fi 1
DEMO_PTR_H  !fi 1
DEMO_DEST:
DEMO_DEST_L: !fi 1
DEMO_DEST_H  !fi 1
DEMO_PALETTE_SIZE: !fi 1
DEMO_PALETTE_BASE: !fi 1
DEMO_TEMP: !fi 1

DEMO_ADR_LAST_SECOND: !fi 1
DEMO_ADR_IDLE_SECONDS: !fi 1
DEMO_IDLE_TIMEOUT = 30

VIDEO_RAM_A0 = $7FFE
VIDEO_RAM_A0_L = $7FFE
VIDEO_RAM_A0_H = $7FFF

* = $0100
; includes
DRIVER_RAM_BASE = color_depth_demo_end_of_program 
!source "app.s"
!source "lcd.s"
!source "keyhit.s"
!source "video_defs.s"
	
; --- init
color_depth_demo_init:	
	lda #<color_depth_demo_isr
	sta $00FFFE
	lda #>color_depth_demo_isr
	sta $00FFFF	
	+app_init
			
	jsr lcd_standard_init
	jsr keyhit_standard_init
	jsr print_title
	
	cli
	
	; start the program
	jsr color_depth_sample
	
	; wait for key
	clr DEMO_ADR_IDLE_SECONDS
-	lda KEYBOARD_ADR_KEY_READY
	bne +
	
	lda $003F20 ; RTC_ADR_SECONDS
	cmp DEMO_ADR_LAST_SECOND
	beq -
	sta DEMO_ADR_LAST_SECOND
	inc DEMO_ADR_IDLE_SECONDS
	lda DEMO_ADR_IDLE_SECONDS
	cmp #DEMO_IDLE_TIMEOUT
	bcc -
	
	; exit
+	jml app_warm_boot
	
; --- Interrupt service routine	
color_depth_demo_isr:		
	php
	phb
	pha
	phy
	phx
	
	lda APP_ADR_PROGRAM_BANK
	bank
	
	; was the interrupt triggered by the keyboard?
	lda KEYBOARD_ADR_CONTROL
	and #$1
	bne +
	lda KEYBOARD_ADR_DATA
	jsr keyboard_handle_scan_code
			
+	plx
	ply
	pla
	plb
	plp
	
	rtl ; returns to page 0 wrapper
			

; --- Print title string to LCD	 
print_title:
	ldx #$00
	
-	lda demo_titlestr,X
	beq +
	
	phx
	jsr lcd_write_char
	plx
	inx
	
	; terminate after 256 chars, even if NULL is not reached
	bne -

+	rts

	
demo_titlestr:	
!text "Color depth demo", 13, "Press any key to exit", 0
	 

; Color depth sample
color_depth_sample:
	lda #$00
	sta VIDEO_ADR_VSIZE ; 240 lines, mode byte per line

	jsr video_zero_vram
		
	; draw images	
	jsr draw_sample_8bpp
	jsr draw_sample_4bpp
	jsr draw_sample_2bpp

	; draw text
	lda #<demo_text_2bpp
	ldy #>demo_text_2bpp
	ldx #9
	jsr video_text_print
	
	lda #<demo_text_4bpp
	ldy #>demo_text_4bpp
	ldx #19
	jsr video_text_print
	
	lda #<demo_text_8bpp
	ldy #>demo_text_8bpp
	ldx #29
	jsr video_text_print
	
	rts

		
; --- Video zero VRAM
; @noparams
; Clears contents of VRAM to all zeroes.
video_zero_vram:
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$7E
	sta VIDEO_RAM_A0_H
	
--	ldx #$7F
	lda #0
	
-	sta (VIDEO_RAM_A0),X
	dex
	bpl -
	
	; decrement row pointer
	lda VIDEO_RAM_A0_L
	sec
	sbc #$80
	sta VIDEO_RAM_A0_L
	bcs --
	dec VIDEO_RAM_A0_H
	bpl --
	
	plb 
	rts
	
		
; --- Draw sample 8bpp
draw_sample_8bpp:
	lda #$E0 ; 224 colors
	sta DEMO_PALETTE_SIZE
	lda #$00
	sta DEMO_PALETTE_BASE
	
	; src pointer for palette
	lda #<(sample_8bpp_image+54)
	sta DEMO_PTR_L
	lda #>(sample_8bpp_image+54)
	sta DEMO_PTR_H
	jsr setup_palette
	
	; set the bottom 80 lines to 256 color 8bpp mode
	; dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda #$00
	sta VIDEO_RAM_A0_L
	lda #$50
	sta VIDEO_RAM_A0_H
	plb
	
	ldy #80 ; line count
	lda #$30 ; 256 colors
	jsr setup_mode

	; copy the image to the bottom 71 lines
	; dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$73
	sta VIDEO_RAM_A0_H
	plb	
	
	; source pointer
	lda #<(sample_8bpp_image+54+4*224)
	sta DEMO_PTR_L
	lda #>(sample_8bpp_image+54+4*224)
	sta DEMO_PTR_H
		
	ldy #71 ; line count
	jsr copy_image
	
	rts
	
	
; --- Draw sample 4bpp
draw_sample_4bpp:
	lda #$10 ; 16 colors
	sta DEMO_PALETTE_SIZE
	lda #$E0 ; offset E
	sta DEMO_PALETTE_BASE
	
	; src pointer for palette
	lda #<(sample_4bpp_image+54)
	sta DEMO_PTR_L
	lda #>(sample_4bpp_image+54)
	sta DEMO_PTR_H
	jsr setup_palette
	
	; set the middle 80 lines to 16 color 4bpp mode
	; dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda #$00
	sta VIDEO_RAM_A0_L
	lda #$28
	sta VIDEO_RAM_A0_H
	plb
	
	ldy #80 ; line count
	lda #$2E ; 16 colors, palette E
	jsr setup_mode

	; copy the image to the middle 71 lines
	; dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$4B
	sta VIDEO_RAM_A0_H
	plb	
	
	; source pointer
	lda #<(sample_4bpp_image+54+4*16)
	sta DEMO_PTR_L
	lda #>(sample_4bpp_image+54+4*16)
	sta DEMO_PTR_H
		
	ldy #71 ; line count
	jsr copy_image
	
	rts
	
	

; --- Draw sample 2bpp
draw_sample_2bpp:
	ldy #$04 ; 4 repetitions
	
	lda #$04 ; 4 colors
	sta DEMO_PALETTE_SIZE
	lda #$F0 ; offset E
	sta DEMO_PALETTE_BASE
	
-	; src pointer for palette
	lda #<(sample_2bpp_image+54)
	sta DEMO_PTR_L
	lda #>(sample_2bpp_image+54)
	sta DEMO_PTR_H
	phy
	jsr setup_palette
	ply
	dey
	beq +
	
	lda DEMO_PALETTE_BASE
	clc
	adc #$04
	sta DEMO_PALETTE_BASE
	bra -	
	
+	; pack the bitmap data to 4 pixels/byte
	; source pointer
	lda #<(sample_2bpp_image+54+4*4)
	sta DEMO_PTR_L
	lda #>(sample_2bpp_image+54+4*4)
	sta DEMO_PTR_H
	; dest pointer
	lda #<(sample_2bpp_image+54+4*4)
	sta DEMO_DEST_L
	lda #>(sample_2bpp_image+54+4*4)
	sta DEMO_DEST_H	
	jsr pack_image_2bpp
	
	; set the top 80 lines to 4 color 2bpp mode
	; dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda #$00
	sta VIDEO_RAM_A0_L
	lda #$00
	sta VIDEO_RAM_A0_H
	plb
	
	ldy #80 ; line count
	lda #$1F ; 4 colors, palette F
	jsr setup_mode

	; copy the image to the top 72 lines
	; dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$23
	sta VIDEO_RAM_A0_H
	plb	
	
	; source pointer
	lda #<(sample_2bpp_image+54+4*4)
	sta DEMO_PTR_L
	lda #>(sample_2bpp_image+54+4*4)
	sta DEMO_PTR_H
		
	ldy #72 ; line count
	jsr copy_image
	
	rts
	
			
; --- Setup mode
; @param: A = mode byte
; @param: Y = number of lines to set the mode for
; @param: VIDEO_RAM_A0 = address of first line
setup_mode:
	phb
	bank #VIDEO_RAM_BANK
	pha
	ldx #$01 ; mode byte is stored at offsets 0 and 1 within the line
	
-	pla
	sta (VIDEO_RAM_A0),X
	dex
	sta (VIDEO_RAM_A0),X
	inx
	dey
	bne +
	
	plb
	rts
	
+	pha
	lda VIDEO_RAM_A0_L
	clc
	adc #$80
	sta VIDEO_RAM_A0_L
	bcc -
	inc VIDEO_RAM_A0_H
	bra -
	

; --- Copy image
; @param: DEMO_PTR = start of image bitmap data, immediately following palette
; @param: VIDEO_RAM_A0 = address of the bottom-most destination line
; @param: Y = number of lines to copy
copy_image:
	jsr copy_one_line
	
	dey
	bne +
	rts
		
	; increment source pointer
+	lda DEMO_PTR_L
	clc
	adc #$80
	sta DEMO_PTR_L
	bcc +
	inc DEMO_PTR_H
	
	; decrement dest pointer	
+	phb
	bank #VIDEO_RAM_BANK
	lda VIDEO_RAM_A0_L
	sec
	sbc #$80
	sta VIDEO_RAM_A0_L
	bcc +
	plb
	bra copy_image
	
+	dec VIDEO_RAM_A0_H
	plb
	bra copy_image
	
	
; --- Copy one line
copy_one_line:
	; copy the last byte of the source row to dest row index 0
	ldx #$7F
	lda (DEMO_PTR), X
	phb
	bank #VIDEO_RAM_BANK
	ldx #$00
	sta (VIDEO_RAM_A0), X
	plb
	
	; copy the rest of the line's image data 
	; (byte 0-1 of source image is not drawn)
	ldx #$03
-	dex
	lda (DEMO_PTR), X
	inx
	phb
	bank #VIDEO_RAM_BANK
	sta (VIDEO_RAM_A0), X
	plb
	inx
	bpl -	
	
	rts
	
	
; --- Setup palette 
; @param: DEMO_PTR = start of palette data
; @param: DEMO_PALETTE_SIZE = number of entries in the palette
; @param: DEMO_PALETTE_BASE = offset to add to each palette index
setup_palette:	
	ldy #$00 ; palette index
	ldx #$02 ; offset into palette data
	
.set_entry:	
	tya
	clc
	adc DEMO_PALETTE_BASE
	sta VIDEO_ADR_PAL_INDEX
	
	; red
	lda (DEMO_PTR),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR
	dex
	
	; green
	lda (DEMO_PTR),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR
	dex
	
	; blue
	lda (DEMO_PTR),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR
	
	inx
	inx
	
	lda DEMO_PTR_L
	clc
	adc #$04
	sta DEMO_PTR_L
	bcc +
	inc DEMO_PTR_H	
	
+	iny
	cpy DEMO_PALETTE_SIZE 
	bne .set_entry
	
	rts
	
	
; --- Pack image 2bpp
; @param: DEMO_PTR = source address of bitmap data to pack
; 2bpp BMP files are stored in 4bpp format, just with a smaller palette.
; This routine packs the data into true 2bpp format.
pack_image_2bpp:
	; 00AB00CD 00EF00GH -> ABCDEFGH
	ldx #1 
	
	lda (DEMO_PTR),X
	and #$03
	sta DEMO_TEMP ; now contains 000000GH
	lda (DEMO_PTR),X
	lsr
	lsr
	ora DEMO_TEMP 
	sta DEMO_TEMP ; now contains 0000EFGH
	
	dex
	lda (DEMO_PTR),X
	asl
	asl
	asl
	asl
	ora DEMO_TEMP
	sta DEMO_TEMP ; now contains 00CDEFGH
	lda (DEMO_PTR),X
	asl
	asl
	and #$C0
	ora DEMO_TEMP 
	sta (DEMO_DEST),X ; write ABCDEFGH
	
	; increment source
	lda DEMO_PTR_L
	clc
	adc #$02
	sta DEMO_PTR_L
	bcc +
	inc DEMO_PTR_H
	
+	; increment dest
	inc DEMO_DEST_L
	bne +
	inc DEMO_DEST_H
	
+	; finished if dest pointer = base + 128*80
	lda DEMO_DEST_L
	cmp #<(sample_2bpp_image+54+4*4+128*80)
	bne pack_image_2bpp
	lda DEMO_DEST_H
	cmp #>(sample_2bpp_image+54+4*4+128*80)
	bne pack_image_2bpp	
	
	rts
	
		
; --- Video text print	
; @param0 (A) - low byte of string address
; @param1 (Y) - high byte of string address
; @param2 (X) - line number at which to print the string
; prints a null-terminated text string to a screen line (up to 63 bytes)
video_text_print:
	; src pointer
	sta DEMO_PTR_L
	sty DEMO_PTR_H
	
	; dst pointer
	phb
	bank #VIDEO_RAM_BANK
	stx VIDEO_RAM_A0_L
	clr VIDEO_RAM_A0_H
	
	; dst += lineNum *= 8; lineNum <<= 7
	ldy #10
	
-	asl VIDEO_RAM_A0_L
	bcs +
	asl VIDEO_RAM_A0_H
	bra ++
		
+	asl VIDEO_RAM_A0_H
	inc VIDEO_RAM_A0_H
	
++  dey
	bne -	
 
	; set text mode for the line
	ldx #$00
	lda #$4F ; text, palette F
	sta (VIDEO_RAM_A0),X
	inc VIDEO_RAM_A0_L
	sta (VIDEO_RAM_A0),X
	inc VIDEO_RAM_A0_L
	
	; copy loop
	plb
	ldy #$00 ; source index
	ldx #$00 ; dest index

-	lda (DEMO_PTR),Y
	beq +
		
	phb
	bank #VIDEO_RAM_BANK
	sta (VIDEO_RAM_A0),X
	plb
	
	iny
	inx
	inx
	bra -	

+	rts
    		
    			
; --- Data
demo_text_2bpp:	
!text "2 bits per pixel, 512 pixels per line, 4 colors                ", 0

demo_text_4bpp:	
!text "4 bits per pixel, 256 pixels per line, 16 colors               ", 0

demo_text_8bpp:	
!text "8 bits per pixel, 128 pixels per line, 256 colors              ", 0

sample_8bpp_image:
!binary "256-color-sample.bmp"		
	
sample_4bpp_image:
!binary "16-color-sample.bmp"		
	
sample_2bpp_image:
!binary "4-color-sample.bmp"	
	
color_depth_demo_end_of_program:
	