* = $0000

; --- Jump to init
	nop
	nop
	nop
	jmp datetime_init

; reserve the remainder of zero page for run-time data
A0:
A0_L: !fi 1
A0_H: !fi 1

DATETIME_SELECTION: !fi 1
TEXT_FLAG: !fi 1
PREVIOUS_SECOND: !fi 1

; Contants
DAY_OF_WEEK_X = 10
MONTH_X = DAY_OF_WEEK_X+22
DAY_X = MONTH_X+20
YEAR_X = DAY_X+8
HOUR_X = YEAR_X+10
MINUTE_X = HOUR_X+6
SECOND_X = MINUTE_X+6
AMPM_X = SECOND_X+6
SAVE_X = AMPM_X+14
DATETIME_Y = 40

INSTRUCTION1_X = 12
INSTRUCTION2_X = 12
INSTRUCTION1_Y = 52
INSTRUCTION2_Y = 56

* = $0100
; includes
DRIVER_RAM_BASE = datetime_end_of_program 
!source "lcd.s"
!source "keyboard.s"
!source "console.s"
!source "app.s"
!source "video_defs.s"
!source "rtc.s"
	
; --- Simple init
datetime_init:	
	lda #<datetime_isr
	sta $00FFFE
	lda #>datetime_isr
	sta $00FFFF	
	+app_init
			
	jsr lcd_standard_init
	jsr keyboard_standard_init
	jsr video_zero_vram
	jsr console_standard_init
	
	; setup mixed mode
	lda #$00
	sta VIDEO_ADR_VSIZE ; 240 lines, mode byte per line
		
	phb
	bank #VIDEO_RAM_BANK
	clr VIDEO_RAM_A0_L
	lda #$74
	sta VIDEO_RAM_A0_H
	 	
	ldx #$01 
	
-	lda #$41 ; text, 1 bpp, palette 1
	sta (VIDEO_RAM_A0),X
	dex
	sta (VIDEO_RAM_A0),X
	inx
	
	; decrement row pointer
	lda VIDEO_RAM_A0_L
	sec
	sbc #$80
	sta VIDEO_RAM_A0_L
	bcs -
	dec VIDEO_RAM_A0_H
	bpl -
	
	; override pallette for date/time line
	clr VIDEO_RAM_A0_L
	lda #DATETIME_Y
	sta VIDEO_RAM_A0_H
	lda #$4A ; text, 1 bpp, palette A
	sta (VIDEO_RAM_A0),X
	dex
	sta (VIDEO_RAM_A0),X
	plb
	
	jsr print_title
	
	lda #0
	jsr console_enable_cursor	
	
	lda #8
	sta DATETIME_SELECTION
	
	jsr update_all
	
	; print instructions
	phb
	bank #VIDEO_RAM_BANK
	lda #INSTRUCTION1_X
	sta VIDEO_RAM_CURSOR_PTR_L	
	lda #INSTRUCTION1_Y
	sta VIDEO_RAM_CURSOR_PTR_H
	plb
	
	lda #<instruction1_string
	sta A0_L
	lda #>instruction1_string
	sta A0_H
	ldx #9 ; input field index
	ldy #0
	jsr print_indexed_string
	
	phb
	bank #VIDEO_RAM_BANK
	lda #INSTRUCTION2_X
	sta VIDEO_RAM_CURSOR_PTR_L	
	lda #INSTRUCTION2_Y
	sta VIDEO_RAM_CURSOR_PTR_H
	plb
	
	lda #<instruction2_string
	sta A0_L
	lda #>instruction2_string
	sta A0_H
	ldx #9 ; input field index
	ldy #0
	jsr print_indexed_string
		
	cli
	
	; start the program
	jsr datetime_run 

	halt

; --- Interrupt service routine	
datetime_isr:
	php
	phb
	pha
	phy
	phx
	
	lda APP_ADR_PROGRAM_BANK
	bank
	
	; was the interrupt triggered by the keyboard?
	lda KEYBOARD_ADR_CONTROL
	and #$1
	bne .isr_done
	lda KEYBOARD_ADR_DATA
	
	; extended key?
	ldx KEYBOARD_ADR_EXTENDED_FLAG
	beq ++
	; not break?
	ldx KEYBOARD_ADR_NOT_BREAK_FLAG
	beq ++
	
	; up arrow?
	cmp #$75
	bne +
	pha
	;jsr handle_arrow_up
	lda #'w'
	sta KEYBOARD_ADR_KEY_BUFFER
	lda #1
	sta KEYBOARD_ADR_KEY_READY
	pla
	bra ++
	
	; down arrow
+	cmp #$72
	bne +
	pha
	;jsr handle_arrow_down
	lda #'s'
	sta KEYBOARD_ADR_KEY_BUFFER
	lda #1
	sta KEYBOARD_ADR_KEY_READY
	pla
	bra ++

+	; left arrow
	cmp #$6B
	bne +
	pha
	;jsr handle_arrow_left
	lda #'a'
	sta KEYBOARD_ADR_KEY_BUFFER
	lda #1
	sta KEYBOARD_ADR_KEY_READY
	pla
	bra ++
	
+	; right arrow
	cmp #$74
	bne ++
	pha
	;jsr handle_arrow_right
	lda #'d'
	sta KEYBOARD_ADR_KEY_BUFFER
	lda #1
	sta KEYBOARD_ADR_KEY_READY
	pla
	
++	jsr keyboard_handle_scan_code
	
	plx
	ply
	pla
	plb
	plp

.isr_done:	
	rtl ; returns to page 0 wrapper

	
; --- Video zero VRAM
; @noparams
; Clears contents of VRAM to all zeroes.
video_zero_vram:
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$7E
	sta VIDEO_RAM_A0_H
	
--	ldx #$7F
	lda #0
	
-	sta (VIDEO_RAM_A0),X
	dex
	bpl -
	
	; decrement row pointer
	lda VIDEO_RAM_A0_L
	sec
	sbc #$80
	sta VIDEO_RAM_A0_L
	bcs --
	dec VIDEO_RAM_A0_H
	bpl --
	
	plb 
	rts
	
	
; --- Date-Time Run			
datetime_run:
	; check key
	lda #1
	cmp KEYBOARD_ADR_KEY_READY
	bne ++
	lda KEYBOARD_ADR_KEY_BUFFER
	clr KEYBOARD_ADR_KEY_READY
	cmp #$0D ; test for carriage return
	bne +
	jsr handle_enter
	bra ++
	
	; WASD
+	cmp #'a'
	bne +
	jsr handle_arrow_left
	bra ++
+	cmp #'d'
	bne +
	jsr handle_arrow_right
	bra ++
+	cmp #'w'
	bne +
	jsr handle_arrow_up
	bra ++
+	cmp #'s'
	bne +
	jsr handle_arrow_down
+	
				
++	lda RTC_ADR_SECONDS
	cmp PREVIOUS_SECOND
	beq +

	sta PREVIOUS_SECOND
	;sei
	jsr update_all
	;cli
+	
	jmp datetime_run
	
	
; --- Update by index
; @param: A = index of input field to update
update_by_index:
	asl
	tax
	inx
	
	; push return address
	lda #>(.update_by_index_return-1) ; high byte first
	pha
	lda #<(.update_by_index_return-1)
	pha
	
	; push routine address
	lda update_table,X
	pha
	dex
	lda update_table,X
	pha
	rts ; jump to routine
	
.update_by_index_return
	rts
	
	
; --- Update all
update_all:	
	lda #8
	
-	pha
	jsr update_by_index
	pla
	dec
	bpl -
	
	rts
	
	
update_dayofweek:
	phb
	bank #VIDEO_RAM_BANK
	lda #DAY_OF_WEEK_X
	sta VIDEO_RAM_CURSOR_PTR_L	
	lda #DATETIME_Y
	sta VIDEO_RAM_CURSOR_PTR_H
	plb
	
	; 1 = Sunday, 7 = Saturday
	lda RTC_ADR_DAY_OF_WEEK
	cmp #8 ; < 8?
	bcc +
	lda #0 ; change invalid index to 0

+	tay
	lda #<dayofweek_strings
	sta A0_L
	lda #>dayofweek_strings
	sta A0_H
	
	ldx #0 ; input field index	
	jsr print_indexed_string_with_comma
	
	; pad with spaces
-	cpx #20
	beq +
	phb
	bank #VIDEO_RAM_BANK
	lda #' '
	sta (VIDEO_RAM_CURSOR_PTR),X
	inx
	inx
	plb
	bra -
	
+	rts
	
	
update_month:
	phb
	bank #VIDEO_RAM_BANK
	lda #MONTH_X
	sta VIDEO_RAM_CURSOR_PTR_L	
	lda #DATETIME_Y
	sta VIDEO_RAM_CURSOR_PTR_H
	plb
	
	; month: 000 T NNNN
	lda RTC_ADR_MONTH
	cmp #$0A ; < 10?
	bcc +
	sec
	sbc #6 ; $10 -> 10
	cmp #10
	beq +
	cmp #11
	beq +
	cmp #12
	beq +
	lda #0 ; change invalid index to 0

+	tay
	lda #<month_strings
	sta A0_L
	lda #>month_strings
	sta A0_H
	
	ldx #1 ; input field index
	jsr print_indexed_string
	
	; pad with spaces
-	cpx #18
	beq +
	phb
	bank #VIDEO_RAM_BANK
	lda #' '
	sta (VIDEO_RAM_CURSOR_PTR),X
	inx
	inx
	plb
	bra -
	
+	rts
	
	
update_day:
	phb
	bank #VIDEO_RAM_BANK
	lda #DAY_X
	sta VIDEO_RAM_CURSOR_PTR_L	
	lda #DATETIME_Y
	sta VIDEO_RAM_CURSOR_PTR_H
	plb
	
	; day: 0 0 TT NNNN
	lda RTC_ADR_DAY
	tay
	ldx #2 ; input field index
	jsr print_bcd_with_comma
	
	rts
	
update_year:
	phb
	bank #VIDEO_RAM_BANK
	lda #YEAR_X
	sta VIDEO_RAM_CURSOR_PTR_L	
	lda #DATETIME_Y
	sta VIDEO_RAM_CURSOR_PTR_H
	plb
	
	; year: TTTT NNNN
	lda RTC_ADR_YEAR
	
	; century
	cmp #$A0 ; < 100?
	bcc +
	lda #$20
	bra ++
+	lda #$19
		
++	tay
	ldx #3
	jsr print_bcd
	
	; decade
	phb
	bank #VIDEO_RAM_BANK
	lda #YEAR_X+4
	sta VIDEO_RAM_CURSOR_PTR_L	
	plb
	
	; year: TTTT NNNN
	lda RTC_ADR_YEAR
	
	; century
	cmp #$A0 ; < 100?
	bcc +
	sec
	sbc #$A0
+	tay
	ldx #3
	jsr print_bcd
	
	rts
	

update_hour:
	phb
	bank #VIDEO_RAM_BANK
	lda #HOUR_X
	sta VIDEO_RAM_CURSOR_PTR_L	
	lda #DATETIME_Y
	sta VIDEO_RAM_CURSOR_PTR_H
	plb
	
	; hours: PM/AM 0 TT NNNN
	lda RTC_ADR_HOURS
	and #$7F
	tay
	ldx #4 ; input field index
	jsr print_bcd_with_colon
	
	rts
	

update_minute:
	phb
	bank #VIDEO_RAM_BANK
	lda #MINUTE_X
	sta VIDEO_RAM_CURSOR_PTR_L	
	lda #DATETIME_Y
	sta VIDEO_RAM_CURSOR_PTR_H
	plb
	
	; minutes: 0 TTT NNNN
	lda RTC_ADR_MINUTES
	tay
	ldx #5 ; input field index
	jsr print_bcd_with_colon
	

update_second:
	phb
	bank #VIDEO_RAM_BANK
	lda #SECOND_X
	sta VIDEO_RAM_CURSOR_PTR_L	
	lda #DATETIME_Y
	sta VIDEO_RAM_CURSOR_PTR_H
	plb
	
	; seconds: 0 TTT NNNN
	lda RTC_ADR_SECONDS
	tay
	ldx #6 ; input field index
	jsr print_bcd
	

update_ampm:
	phb
	bank #VIDEO_RAM_BANK
	lda #AMPM_X
	sta VIDEO_RAM_CURSOR_PTR_L	
	lda #DATETIME_Y
	sta VIDEO_RAM_CURSOR_PTR_H
	plb
	
	; hours: PM/AM 0 TT NNNN
	lda RTC_ADR_HOURS
	bmi +
	ldy #0
	bra ++
+	ldy #1

++	lda #<ampm_strings
	sta A0_L
	lda #>ampm_strings
	sta A0_H
	ldx #7 ; input field index
	jsr print_indexed_string
	
	rts
	

update_save:
	phb
	bank #VIDEO_RAM_BANK
	lda #SAVE_X
	sta VIDEO_RAM_CURSOR_PTR_L	
	lda #DATETIME_Y
	sta VIDEO_RAM_CURSOR_PTR_H
	plb
	
	lda #<save_string
	sta A0_L
	lda #>save_string
	sta A0_H
	ldx #8 ; input field index
	ldy #0
	jsr print_indexed_string
	
	rts
	
		
; --- Print indexed string
; @param: X = input field index
; @param: Y = string index
; @param: A0 = string list
; @param: VIDEO_RAM_CURSOR_PTR = destination 	
print_indexed_string:	
	; wait for vblank
-	lda VIDEO_ADR_VBLANK
	bmi -
	
	txa
	eor DATETIME_SELECTION
	bne +
	lda #$80
	sta TEXT_FLAG
	bra ++
+	lda #$00
	sta TEXT_FLAG

	; skip Y strings
++	ldx #0
--	cpy #0
	beq +

	; load next letter, increment pointer
-	lda (A0),X
	pha
	lda A0_L
	clc
	adc #1
	sta A0_L
	lda A0_H
	adc #0
	sta A0_H
	
	; continue if not null
	pla
	bne -	

	; next string
	dey
	bra --
	
	; print the string
+	ldx #0
	ldy #0
	
-	lda (A0),Y
	beq +
	cmp #' '
	beq ++
	clc
	adc TEXT_FLAG
++	phb
	bank #VIDEO_RAM_BANK
	sta (VIDEO_RAM_CURSOR_PTR),X
	plb
	inx
	inx
	iny
	bne -

+	rts


; --- Print indexed string with comma
print_indexed_string_with_comma:
	jsr print_indexed_string
	phb
	bank #VIDEO_RAM_BANK
	lda #','
	sta (VIDEO_RAM_CURSOR_PTR),X
	plb
	inx
	inx
	rts
	
	 
; --- Print BCD
; @param: Y = value to print
print_bcd:
	; wait for vblank
-	lda VIDEO_ADR_VBLANK
	bmi -
	
	txa
	eor DATETIME_SELECTION
	bne +
	lda #$80
	sta TEXT_FLAG
	bra ++
+	lda #$00
	sta TEXT_FLAG

++	ldx #0
	tya
	
	; high nibble
	; A = A >> 4, ROL * 5 followed by AND should be faster than LSR * 4
	rol
	rol
	rol
	rol
	rol
	and #$F
	clc
	adc TEXT_FLAG
	adc #'0'
	phb
	bank #VIDEO_RAM_BANK
	sta (VIDEO_RAM_CURSOR_PTR),X
	plb
	
	; low nibble
	inx
	inx
	tya
	and #$F
	clc
	adc TEXT_FLAG
	adc #'0'
	phb
	bank #VIDEO_RAM_BANK
	sta (VIDEO_RAM_CURSOR_PTR),X
	plb
	
	rts


; --- Print BCD with comma
print_bcd_with_comma:
	jsr print_bcd
	inx
	inx
	phb
	bank #VIDEO_RAM_BANK
	lda #','
	sta (VIDEO_RAM_CURSOR_PTR),X
	plb
	rts


; --- Print BCD with colon
print_bcd_with_colon:
	jsr print_bcd
	inx
	inx
	phb
	bank #VIDEO_RAM_BANK
	lda #':'
	sta (VIDEO_RAM_CURSOR_PTR),X
	plb
	rts
	

; --- Handle arrow up		
handle_arrow_up
	; day of week
	lda DATETIME_SELECTION
	cmp #0
	bne ++
	lda RTC_ADR_DAY_OF_WEEK
	inc
	cmp #8
	bne +
	lda #1
+	sta RTC_ADR_DAY_OF_WEEK		
	
	; month
++	lda DATETIME_SELECTION
	cmp #1
	bne ++
	lda RTC_ADR_MONTH
	inc
	cmp #$0A
	bne +
	lda #$10
+	cmp #$13
	bne +
	lda #1
+	sta RTC_ADR_MONTH		

	; day
++	lda DATETIME_SELECTION
	cmp #2
	bne ++
	lda RTC_ADR_DAY
	inc
	cmp #$0A
	bne +
	lda #$10
+	cmp #$1A
	bne +
	lda #$20
+	cmp #$2A
	bne +
	lda #$30
+	cmp #$32
	bne +	
	lda #1
+	sta RTC_ADR_DAY	
	
	; year
++	lda DATETIME_SELECTION
	cmp #3
	bne ++
	lda RTC_ADR_YEAR
	inc
	pha
	and #$0F
	cmp #$0A
	bne +
	pla
	clc
	adc #6
	pha
+	pla
	sta RTC_ADR_YEAR	
	
	; hour
++	lda DATETIME_SELECTION
	cmp #4
	bne ++
	lda RTC_ADR_HOURS
	inc
	cmp #$0A
	bne +
	lda #$10
+	cmp #$8A
	bne +
	lda #$90
+	cmp #$13
	bne +
	lda #$01
+	cmp #$93
	bne +
	lda #$81
+	sta RTC_ADR_HOURS
		
	; minute
++	lda DATETIME_SELECTION
	cmp #5
	bne ++
	lda RTC_ADR_MINUTES
	inc
	cmp #$0A
	bne +
	lda #$10
+	cmp #$1A
	bne +
	lda #$20
+	cmp #$2A
	bne +
	lda #$30
+	cmp #$3A
	bne +
	lda #$40
+	cmp #$4A
	bne +
	lda #$50
+	cmp #$5A
	bne +
	lda #0
+	sta RTC_ADR_MINUTES	

	; second
++	lda DATETIME_SELECTION
	cmp #6
	bne ++
	lda RTC_ADR_SECONDS
	inc
	cmp #$0A
	bne +
	lda #$10
+	cmp #$1A
	bne +
	lda #$20
+	cmp #$2A
	bne +
	lda #$30
+	cmp #$3A
	bne +
	lda #$40
+	cmp #$4A
	bne +
	lda #$50
+	cmp #$5A
	bne +
	lda #0
+	sta RTC_ADR_SECONDS	

	; am/pm
++	lda DATETIME_SELECTION
	cmp #7
	bne ++
	lda RTC_ADR_HOURS
	eor #$80
	sta RTC_ADR_HOURS
		
	; save
++  lda DATETIME_SELECTION
	cmp #8
	bne ++
	jml app_warm_boot
			
++	lda DATETIME_SELECTION
	jsr update_by_index
	rts
	
; --- Handle arrow down	
handle_arrow_down:
	; day of week
	lda DATETIME_SELECTION
	cmp #0
	bne ++
	lda RTC_ADR_DAY_OF_WEEK
	dec
	cmp #0
	bne +
	lda #7
+	sta RTC_ADR_DAY_OF_WEEK		
	
	; month
++  lda DATETIME_SELECTION
	cmp #1
	bne ++
	lda RTC_ADR_MONTH
	dec
	cmp #$0F
	bne +
	lda #$09
+	cmp #0
	bne +
	lda #$12
+	sta RTC_ADR_MONTH		

	; day
++  lda DATETIME_SELECTION
	cmp #2
	bne ++
	lda RTC_ADR_DAY
	dec
	cmp #$2F
	bne +
	lda #$29
+	cmp #$1F
	bne +
	lda #$19
+	cmp #$0F
	bne +
	lda #$09
+	cmp #0
	bne +
	lda #$31
+	sta RTC_ADR_DAY	
	
	; year
++	lda DATETIME_SELECTION
	cmp #3
	bne ++
	lda RTC_ADR_YEAR
	dec
	pha
	and #$0F
	cmp #$0F
	bne +
	pla
	sec
	sbc #6
	pha
+	pla
	sta RTC_ADR_YEAR
	
	; hour
++	lda DATETIME_SELECTION
	cmp #4
	bne ++
	lda RTC_ADR_HOURS
	dec
	cmp #$0F
	bne +
	lda #$09
+	cmp #$8F
	bne +
	lda #$89
+	cmp #$00
	bne +
	lda #$12
+	cmp #$80
	bne +
	lda #$92
+	sta RTC_ADR_HOURS

	; minute
++  lda DATETIME_SELECTION
	cmp #5
	bne ++
	lda RTC_ADR_MINUTES
	dec
	cmp #$4F
	bne +
	lda #$49
+	cmp #$3F
	bne +
	lda #$39
+	cmp #$2F
	bne +
	lda #$29
+	cmp #$1F
	bne +
	lda #$19
+	cmp #$0F
	bne +
	lda #$09		
+	cmp #0
	bne +
	lda #$59
+	sta RTC_ADR_MINUTES	
		
	; second
++  lda DATETIME_SELECTION
	cmp #6
	bne ++
	lda RTC_ADR_SECONDS
	dec
	cmp #$4F
	bne +
	lda #$49
+	cmp #$3F
	bne +
	lda #$39
+	cmp #$2F
	bne +
	lda #$29
+	cmp #$1F
	bne +
	lda #$19
+	cmp #$0F
	bne +
	lda #$09		
+	cmp #0
	bne +
	lda #$59
+	sta RTC_ADR_SECONDS	
	
	; am/pm
++	lda DATETIME_SELECTION
	cmp #7
	bne ++
	lda RTC_ADR_HOURS
	eor #$80
	sta RTC_ADR_HOURS
		
	; save
++  lda DATETIME_SELECTION
	cmp #8
	bne ++
	jml app_warm_boot
				
++	lda DATETIME_SELECTION
	jsr update_by_index
	rts
	
	
handle_arrow_left:
	lda DATETIME_SELECTION
	bne +
	rts
		
+	dec DATETIME_SELECTION
	jsr update_by_index
	lda DATETIME_SELECTION
	jsr update_by_index
	rts
	

handle_arrow_right:
	lda DATETIME_SELECTION
	cmp #8 ; < 8?
	bcc +
	rts
		
+	inc DATETIME_SELECTION
	jsr update_by_index
	lda DATETIME_SELECTION
	jsr update_by_index
	rts
	

handle_enter:
	lda DATETIME_SELECTION
	cmp #8
	beq +
	rts
	
+	jml app_warm_boot

		 
print_title:
	ldx #$00
	
-	lda datetime_lcdstr,X
	beq +
	
	phx
	jsr lcd_write_char
	plx
	inx
	
	; terminate after 256 chars, even if NULL is not reached
	bne -

+	rts

	
datetime_lcdstr:	
!text "System date and time", 13, "", 0

dayofweek_strings:
!text "      ???", 0, "   Sunday", 0, "   Monday", 0, "  Tuesday", 0, "Wednesday", 0, " Thursday", 0, "   Friday", 0, " Saturday", 0

month_strings:
!text "      ???", 0, "  January", 0, " February", 0, "    March", 0, "    April", 0, "      May", 0, "     June", 0, "     July", 0, "   August", 0, "September", 0, "  October", 0, " November", 0, " December", 0

save_string:
!text "Exit", 0

instruction1_string:
!text "Use left/right arrow keys to move the highlight.", 0

instruction2_string:
!text "Use up/down arrow keys to change the value.", 0

ampm_strings:
!text "AM", 0, "PM", 0

update_table:
!word (update_dayofweek-1)        
!word (update_month-1)
!word (update_day-1)
!word (update_year-1)
!word (update_hour-1)
!word (update_minute-1)
!word (update_second-1)
!word (update_ampm-1)
!word (update_save-1)
	
datetime_end_of_program:
	