* = $0000

; --- Jump to init
	nop
	nop
	nop
	jmp hi_res_demo_init

; reserve the remainder of zero page for run-time data
DEMO_PTR:
DEMO_PTR_L: !fi 1
DEMO_PTR_H  !fi 1
VIDEO_ADR_TEMP: !fi 1

DEMO_ADR_LAST_SECOND: !fi 1
DEMO_ADR_IDLE_SECONDS: !fi 1
DEMO_IDLE_TIMEOUT = 30

VIDEO_RAM_A0 = $7FFE
VIDEO_RAM_A0_L = $7FFE
VIDEO_RAM_A0_H = $7FFF

* = $0100
; includes
DRIVER_RAM_BASE = hi_res_demo_end_of_program 
!source "app.s"
!source "lcd.s"
!source "keyhit.s"
!source "video_defs.s"
	
; --- init
hi_res_demo_init:	
	lda #<hi_res_demo_isr
	sta $00FFFE
	lda #>hi_res_demo_isr
	sta $00FFFF	
	+app_init
			
	jsr lcd_standard_init
	jsr keyhit_standard_init
	jsr print_title
	
	cli
	
	; start the program
	jsr hi_res_sample
	
	; wait for key
	clr DEMO_ADR_IDLE_SECONDS
-	lda KEYBOARD_ADR_KEY_READY
	bne +
	
	lda $003F20 ; RTC_ADR_SECONDS
	cmp DEMO_ADR_LAST_SECOND
	beq -
	sta DEMO_ADR_LAST_SECOND
	inc DEMO_ADR_IDLE_SECONDS
	lda DEMO_ADR_IDLE_SECONDS
	cmp #DEMO_IDLE_TIMEOUT
	bcc -
	
	; exit
+	jml app_warm_boot
	
; --- Interrupt service routine	
hi_res_demo_isr:		
	php
	phb
	pha
	phy
	phx
	
	lda APP_ADR_PROGRAM_BANK
	bank
	
	; was the interrupt triggered by the keyboard?
	lda KEYBOARD_ADR_CONTROL
	and #$1
	bne +
	lda KEYBOARD_ADR_DATA
	jsr keyboard_handle_scan_code
			
+	plx
	ply
	pla
	plb
	plp
	
	rtl ; returns to page 0 wrapper
		

; --- Print title string to LCD	 
print_title:
	ldx #$00
	
-	lda demo_titlestr,X
	beq +
	
	phx
	jsr lcd_write_char
	plx
	inx
	
	; terminate after 256 chars, even if NULL is not reached
	bne -

+	rts

	
demo_titlestr:	
!text "High-res image demo", 13, "Press any key to exit", 0
	 

; --- Hi res sample
; @noparams
; 512x480 @ 2 colors
hi_res_sample:
	lda #$C0 ; 480 lines, mode byte per frame
	sta VIDEO_ADR_VSIZE 
	
	jsr video_zero_vram
	jsr video_palette_1bpp
	
	lda #$00 ; 2 colors, palette 0
	phb
	bank #VIDEO_RAM_BANK
	sta VIDEO_ADR_FRAME_MODE_0 
	sta VIDEO_ADR_FRAME_MODE_1 
	sta VIDEO_ADR_FRAME_MODE_2 
	plb
	
	; init source pointer = image_base + $3E (62)
	lda #<(video_image_data+62)
	sta DEMO_PTR_L
	lda #>(video_image_data+62)
	sta DEMO_PTR_H
	
	; init dest pointer = start of line 479 = $7781
	phb
	bank #VIDEO_RAM_BANK
	lda #$81
	sta VIDEO_RAM_A0_L
	lda #$77
	sta VIDEO_RAM_A0_H
	plb
	
.copy_loop:
	jsr copy_one_line
	
	; increment source pointer
	lda DEMO_PTR_L
	clc
	adc #$40
	sta DEMO_PTR_L
	bcc +
	inc DEMO_PTR_H
+	

	; decrement dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda VIDEO_RAM_A0_L
	and #$01
	beq +
	
	; odd line: decrement to previoius even line
	dec VIDEO_RAM_A0_L
	plb
	bra .copy_loop
	
	; even line: decrement to previous odd line and check for end	
+	lda VIDEO_RAM_A0_L
	sec
	sbc #$7F
	sta VIDEO_RAM_A0_L
	bcc +
	plb
	bra .copy_loop
	
+	dec VIDEO_RAM_A0_H
	lda VIDEO_RAM_A0_H
	plb
	bmi .done
	bra .copy_loop

.done	
	rts
	

; --- Copy one line
copy_one_line:	
	ldx #126
	ldy #63
	
-	lda (DEMO_PTR),Y
	;eor #$FF
	phb
	bank #VIDEO_RAM_BANK
	sta (VIDEO_RAM_A0), X
	plb
	
	dex
	dex
	dey
	bpl -
	
	rts
	
			
; --- Video zero VRAM
; @noparams
; Clears contents of VRAM to all zeroes.
video_zero_vram:
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$7E
	sta VIDEO_RAM_A0_H
	
--	ldx #$7F
	lda #0
	
-	sta (VIDEO_RAM_A0),X
	dex
	bpl -
	
	; decrement row pointer
	lda VIDEO_RAM_A0_L
	sec
	sbc #$80
	sta VIDEO_RAM_A0_L
	bcs --
	dec VIDEO_RAM_A0_H
	bpl --
	
	plb 
	rts
	
			
; --- Video setup 1bpp palette
; @noparams
; Sets the RGB value of palette entries 0-15 to be
; foreground color if the least significant bit is 1, background if it's 0.
video_palette_1bpp:
	ldx #$00
	
-	; background color
	txa
	sta VIDEO_ADR_PAL_INDEX
	
	; black background
	;lda #$00
	;sta VIDEO_ADR_PAL_COLOR
	;sta VIDEO_ADR_PAL_COLOR
	;sta VIDEO_ADR_PAL_COLOR
	lda #$33
	sta VIDEO_ADR_PAL_COLOR
	lda #$25
	sta VIDEO_ADR_PAL_COLOR
	lda #$14
	sta VIDEO_ADR_PAL_COLOR
	
	inx
	
	; foreground color
	txa
	sta VIDEO_ADR_PAL_INDEX
	
	; foreground.rgb
	;lda #$3F
	;sta VIDEO_ADR_PAL_COLOR
	;sta VIDEO_ADR_PAL_COLOR
	;sta VIDEO_ADR_PAL_COLOR
	lda #$19
	sta VIDEO_ADR_PAL_COLOR
	lda #$0C
	sta VIDEO_ADR_PAL_COLOR
	lda #$00
	sta VIDEO_ADR_PAL_COLOR
		
	inx
	cpx #$10
	
	bne -
					
	rts	
	
		
; --- Data		
video_image_data:
!binary "hi_res_demo.bmp"	

hi_res_demo_end_of_program:
	