; Hardware/driver routines for BMOW keyboard

!source "keyboard_defs.s"

; dimensional constants
KEYBOARD_SCAN_TABLE_SIZE_TIMES_TWO = 134

; driver RAM addresses
KEYBOARD_ADR_KEY_READY = LCD_ADR_END + 1
KEYBOARD_ADR_KEY_BUFFER = KEYBOARD_ADR_KEY_READY + 1
KEYBOARD_ADR_MODIFIER_BUFFER = KEYBOARD_ADR_KEY_READY + 2

KEYBOARD_ADR_LEFT_SHIFT_DOWN = KEYBOARD_ADR_KEY_READY + 3
KEYBOARD_ADR_RIGHT_SHIFT_DOWN = KEYBOARD_ADR_KEY_READY + 4
KEYBOARD_ADR_LEFT_CONTROL_DOWN = KEYBOARD_ADR_KEY_READY + 5
KEYBOARD_ADR_RIGHT_CONTROL_DOWN = KEYBOARD_ADR_KEY_READY + 6
KEYBOARD_ADR_LEFT_ALT_DOWN = KEYBOARD_ADR_KEY_READY + 7
KEYBOARD_ADR_RIGHT_ALT_DOWN = KEYBOARD_ADR_KEY_READY + 8

KEYBOARD_ADR_NOT_BREAK_FLAG = KEYBOARD_ADR_KEY_READY + 9
KEYBOARD_ADR_EXTENDED_FLAG = KEYBOARD_ADR_KEY_READY + 10
KEYBOARD_ADR_SUPER_EXTENDED_FLAG = KEYBOARD_ADR_KEY_READY + 11
KEYBOARD_ADR_IGNORE_COUNT = KEYBOARD_ADR_KEY_READY + 12

KEYBOARD_ADR_END = KEYBOARD_ADR_IGNORE_COUNT

; --- Unshifted scan code to ASCII conversion table
keyboard_keys_unshifted:
!byte 0x0d,9 ; tab
!byte 0x0e,'`'
!byte 0x15,'q'
!byte 0x16,'1'
!byte 0x1a,'z'
!byte 0x1b,'s'
!byte 0x1c,'a'
!byte 0x1d,'w'
!byte 0x1e,'2'
!byte 0x21,'c'
!byte 0x22,'x'
!byte 0x23,'d'
!byte 0x24,'e'
!byte 0x25,'4'
!byte 0x26,'3'
!byte 0x29,' '
!byte 0x2a,'v'
!byte 0x2b,'f'
!byte 0x2c,'t'
!byte 0x2d,'r'
!byte 0x2e,'5'
!byte 0x31,'n'
!byte 0x32,'b'
!byte 0x33,'h'
!byte 0x34,'g'
!byte 0x35,'y'
!byte 0x36,'6'
!byte 0x39,','
!byte 0x3a,'m'
!byte 0x3b,'j'
!byte 0x3c,'u'
!byte 0x3d,'7'
!byte 0x3e,'8'
!byte 0x41,','
!byte 0x42,'k'
!byte 0x43,'i'
!byte 0x44,'o'
!byte 0x45,'0'
!byte 0x46,'9'
!byte 0x49,'.'
!byte 0x4a,'/'
!byte 0x4b,'l'
!byte 0x4c,';'
!byte 0x4d,'p'
!byte 0x4e,'-'
!byte 0x52,39 ; apostrophe
!byte 0x54,'['
!byte 0x55,'='
!byte 0x5a,13 ; return 
!byte 0x5b,']'
!byte 0x5d,92 ; backslash
!byte 0x61,'<'
!byte 0x66,8
!byte 0x69,'1'
!byte 0x6b,'4'
!byte 0x6c,'7'
!byte 0x70,'0'
!byte 0x71,','
!byte 0x72,'2'
!byte 0x73,'5'
!byte 0x74,'6'
!byte 0x75,'8'
!byte 0x79,'+'
!byte 0x7a,'3'
!byte 0x7b,'-'
!byte 0x7c,'*'
!byte 0x7d,'9'

; --- Shifted scan code to ASCII conversion table
keyboard_keys_shifted:
!byte 0x0d,9 ; tab
!byte 0x0e,'~'
!byte 0x15,'Q'
!byte 0x16,'!'
!byte 0x1a,'Z'
!byte 0x1b,'S'
!byte 0x1c,'A'
!byte 0x1d,'W'
!byte 0x1e,'@'
!byte 0x21,'C'
!byte 0x22,'X'
!byte 0x23,'D'
!byte 0x24,'E'
!byte 0x25,'$'
!byte 0x26,'#'
!byte 0x29,' '
!byte 0x2a,'V'
!byte 0x2b,'F'
!byte 0x2c,'T'
!byte 0x2d,'R'
!byte 0x2e,'%'
!byte 0x31,'N'
!byte 0x32,'B'
!byte 0x33,'H'
!byte 0x34,'G'
!byte 0x35,'Y'
!byte 0x36,'^'
!byte 0x39,'L'
!byte 0x3a,'M'
!byte 0x3b,'J'
!byte 0x3c,'U'
!byte 0x3d,'&'
!byte 0x3e,'*'
!byte 0x41,'<'
!byte 0x42,'K'
!byte 0x43,'I'
!byte 0x44,'O'
!byte 0x45,')'
!byte 0x46,'('
!byte 0x49,'>'
!byte 0x4a,'?'
!byte 0x4b,'L'
!byte 0x4c,':'
!byte 0x4d,'P'
!byte 0x4e,'_'
!byte 0x52,'"'
!byte 0x54,'{'
!byte 0x55,'+'
!byte 0x5a,13 ; return
!byte 0x5b,'}'
!byte 0x5d,'|'
!byte 0x61,'>'
!byte 0x66,8
!byte 0x69,'1'
!byte 0x6b,'4'
!byte 0x6c,'7'
!byte 0x70,'0'
!byte 0x71,','
!byte 0x72,'2'
!byte 0x73,'5'
!byte 0x74,'6'
!byte 0x75,'8'
!byte 0x79,'+'
!byte 0x7a,'3'
!byte 0x7b,'-'
!byte 0x7c,'*'
!byte 0x7d,'9'

; --- Keyboard standard init
; @noparams
; Resets the keyboard state.
keyboard_standard_init:
	clr KEYBOARD_ADR_KEY_READY
	clr KEYBOARD_ADR_LEFT_SHIFT_DOWN
	clr KEYBOARD_ADR_RIGHT_SHIFT_DOWN
	clr KEYBOARD_ADR_LEFT_CONTROL_DOWN
	clr KEYBOARD_ADR_RIGHT_CONTROL_DOWN
	clr KEYBOARD_ADR_LEFT_ALT_DOWN
	clr KEYBOARD_ADR_RIGHT_ALT_DOWN
	clr KEYBOARD_ADR_EXTENDED_FLAG
	clr KEYBOARD_ADR_SUPER_EXTENDED_FLAG
	clr KEYBOARD_ADR_IGNORE_COUNT
	
	lda #1
	sta KEYBOARD_ADR_NOT_BREAK_FLAG

	rts
	
	
; --- Keyboard handle scan code
; @param0 - a one-byte scan code received from the keyboard
; Processes the scan code, updating the modifier key up/down states, and fill the 
; key and modifier buffers if necessary. Sets KEYBOARD_ADR_KEY_READY to 1 if a key
; is ready in the buffer.
keyboard_handle_scan_code:	
	ldx KEYBOARD_ADR_SUPER_EXTENDED_FLAG
	dex
	bne +
	
	; super-extended flag is set
	cmp #$F0
	bne +
	
	; ignore 4 bytes for a super-extended break
	ldx #4
	stx KEYBOARD_ADR_IGNORE_COUNT
	
+	clr KEYBOARD_ADR_SUPER_EXTENDED_FLAG
	
	; ignore bytes
	ldx KEYBOARD_ADR_IGNORE_COUNT
	dex
	bmi +
	
	; store decremented count, and return
	stx KEYBOARD_ADR_IGNORE_COUNT
	rts
	
	; break?
+	cmp #$F0
	bne +

	; clear not break flag, and return
	clr KEYBOARD_ADR_NOT_BREAK_FLAG
	rts
	
	; super-extended?
+	cmp #$E1
	bne +
	
	; set super-extended flag, ignore count = 2, and return
	ldx #1
	stx KEYBOARD_ADR_SUPER_EXTENDED_FLAG
	inx
	stx KEYBOARD_ADR_IGNORE_COUNT
	rts
	
	; extended?
+	cmp #$E0
	bne +
	
	; set extended flag, and return
	ldx #1
	stx KEYBOARD_ADR_EXTENDED_FLAG
	rts

	; handle a "real" key scan code	
+	ldx KEYBOARD_ADR_EXTENDED_FLAG
	dex
	bne +
	jmp .process_extended_code
	
	; non-extended scan code
	
	; left shift?
+	cmp #$12
	bne + 
	ldx KEYBOARD_ADR_NOT_BREAK_FLAG
	stx KEYBOARD_ADR_LEFT_SHIFT_DOWN
	jmp .processing_done
	
	; right shift?
+	cmp #$59
	bne + 
	ldx KEYBOARD_ADR_NOT_BREAK_FLAG
	stx KEYBOARD_ADR_RIGHT_SHIFT_DOWN
	jmp .processing_done
		
	; left control?
+	cmp #$14
	bne + 
	ldx KEYBOARD_ADR_NOT_BREAK_FLAG
	stx KEYBOARD_ADR_LEFT_CONTROL_DOWN
	jmp .processing_done
	
	; left alt?
+	cmp #$11
	bne + 
	ldx KEYBOARD_ADR_NOT_BREAK_FLAG
	stx KEYBOARD_ADR_LEFT_ALT_DOWN
	jmp .processing_done
		
	; non-modifier key: only process non-breaks
+	ldx KEYBOARD_ADR_NOT_BREAK_FLAG
	dex
	bmi .processing_done
	
	; shifted?
	tay ; preserve A
	lda KEYBOARD_ADR_LEFT_SHIFT_DOWN
	ora KEYBOARD_ADR_RIGHT_SHIFT_DOWN
	beq .process_unshifted
	
	; process a shifted key
	tya ; restore A
	ldx #0
	
-	cmp keyboard_keys_shifted,x
	bne +
	
	; load the matching entry from the table
	inx 
	lda keyboard_keys_shifted,x
	bra .process_table_entry
	 
+	inx
	inx
	cpx #KEYBOARD_SCAN_TABLE_SIZE_TIMES_TWO
	bne -
		
	; nothing in the table matched
	jmp .processing_done
	
	; process an unshifted key
.process_unshifted:
	tya ; restore A
	ldx #0
	
-	cmp keyboard_keys_unshifted,x
	bne +
	
	; load the matching entry from the table
	inx 
	lda keyboard_keys_unshifted,x
	bra .process_table_entry
	 
+	inx
	inx
	cpx #KEYBOARD_SCAN_TABLE_SIZE_TIMES_TWO
	bne -
		
	; nothing in the table matched
	jmp .processing_done
	
	; process the entry from the table
.process_table_entry	
	sta KEYBOARD_ADR_KEY_BUFFER
	lda #1
	sta KEYBOARD_ADR_KEY_READY
	
	; construct a bitfield for the modifier states
	lda KEYBOARD_ADR_RIGHT_ALT_DOWN
	asl
	ora KEYBOARD_ADR_LEFT_ALT_DOWN
	asl
	ora KEYBOARD_ADR_RIGHT_CONTROL_DOWN
	asl
	ora KEYBOARD_ADR_LEFT_CONTROL_DOWN
	asl
	ora KEYBOARD_ADR_RIGHT_SHIFT_DOWN
	asl
	ora KEYBOARD_ADR_LEFT_SHIFT_DOWN
	sta KEYBOARD_ADR_MODIFIER_BUFFER
				
	jmp .processing_done
	
	; extended scan code
.process_extended_code	

	; right control?
	cmp #$14
	bne + 
	ldx KEYBOARD_ADR_NOT_BREAK_FLAG
	stx KEYBOARD_ADR_RIGHT_CONTROL_DOWN
	jmp .processing_done
	
	; right alt?
+	cmp #$11
	bne + 
	ldx KEYBOARD_ADR_NOT_BREAK_FLAG
	stx KEYBOARD_ADR_RIGHT_ALT_DOWN
	jmp .processing_done
	
+
.processing_done:
	clr KEYBOARD_ADR_EXTENDED_FLAG
	lda #1
	sta KEYBOARD_ADR_NOT_BREAK_FLAG
	rts
	
; --- Keyboard get character
; @noparams
; @return - key value in A, modifier value in Y
; Blocks until a key is pressed, then returns the key and key modifier values.
keyboard_getchar:
	
	; keep looping until a key is ready
	lda #1
-	cmp KEYBOARD_ADR_KEY_READY
	bne -
	
	; return the key data
	lda KEYBOARD_ADR_KEY_BUFFER
	ldy KEYBOARD_ADR_MODIFIER_BUFFER
	clr KEYBOARD_ADR_KEY_READY
	rts
	
	