; Mini hardware/driver routines for BMOW keyboard
; Returns 1 in KEYBOARD_ADR_KEY_READY when a key is pressed, but
; does not set KEYBOARD_ADR_KEY_BUFFER
!source "keyboard_defs.s"

; driver RAM addresses
KEYBOARD_ADR_KEY_READY = LCD_ADR_END + 1

KEYBOARD_ADR_NOT_BREAK_FLAG = KEYBOARD_ADR_KEY_READY + 2
KEYBOARD_ADR_EXTENDED_FLAG = KEYBOARD_ADR_KEY_READY + 3
KEYBOARD_ADR_SUPER_EXTENDED_FLAG = KEYBOARD_ADR_KEY_READY + 4
KEYBOARD_ADR_IGNORE_COUNT = KEYBOARD_ADR_KEY_READY + 5

KEYBOARD_ADR_END = KEYBOARD_ADR_IGNORE_COUNT

; --- Keyboard standard init
; @noparams
; Resets the keyboard state.
keyhit_standard_init:
	clr KEYBOARD_ADR_KEY_READY
	clr KEYBOARD_ADR_EXTENDED_FLAG
	clr KEYBOARD_ADR_SUPER_EXTENDED_FLAG
	clr KEYBOARD_ADR_IGNORE_COUNT
	
	lda #1
	sta KEYBOARD_ADR_NOT_BREAK_FLAG

	rts
	
	
; --- Keyboard handle scan code
; @param0 - a one-byte scan code received from the keyboard
; Processes the scan code, updating the modifier key up/down states, and fill the 
; key and modifier buffers if necessary. Sets KEYBOARD_ADR_KEY_READY to 1 if a key
; is ready in the buffer.
keyboard_handle_scan_code:	
	ldx KEYBOARD_ADR_SUPER_EXTENDED_FLAG
	dex
	bne +
	
	; super-extended flag is set
	cmp #$F0
	bne +
	
	; ignore 4 bytes for a super-extended break
	ldx #4
	stx KEYBOARD_ADR_IGNORE_COUNT
	
+	clr KEYBOARD_ADR_SUPER_EXTENDED_FLAG
	
	; ignore bytes
	ldx KEYBOARD_ADR_IGNORE_COUNT
	dex
	bmi +
	
	; store decremented count, and return
	stx KEYBOARD_ADR_IGNORE_COUNT
	rts
	
	; break?
+	cmp #$F0
	bne +

	; clear not break flag, and return
	clr KEYBOARD_ADR_NOT_BREAK_FLAG
	rts
	
	; super-extended?
+	cmp #$E1
	bne +
	
	; set super-extended flag, ignore count = 2, and return
	ldx #1
	stx KEYBOARD_ADR_SUPER_EXTENDED_FLAG
	inx
	stx KEYBOARD_ADR_IGNORE_COUNT
	rts
	
	; extended?
+	cmp #$E0
	bne +
	
	; set extended flag, and return
	ldx #1
	stx KEYBOARD_ADR_EXTENDED_FLAG
	rts

	; handle a "real" key scan code	
+	ldx KEYBOARD_ADR_EXTENDED_FLAG
	dex
	bne +
	jmp .process_extended_code
	
	; non-extended scan code
	
	; left shift?
+	cmp #$12
	bne + 
	jmp .processing_done
	
	; right shift?
+	cmp #$59
	bne + 
	jmp .processing_done
		
	; left control?
+	cmp #$14
	bne + 
	jmp .processing_done
	
	; left alt?
+	cmp #$11
	bne + 
	jmp .processing_done
		
	; non-modifier key: only process non-breaks
+	ldx KEYBOARD_ADR_NOT_BREAK_FLAG
	dex
	bmi .processing_done
	
	; we have a keypress!
	lda #1
	sta KEYBOARD_ADR_KEY_READY
				
	jmp .processing_done
	
	; extended scan code
.process_extended_code	

	; right control?
	cmp #$14
	bne + 
	jmp .processing_done
	
	; right alt?
+	cmp #$11
	bne + 
	jmp .processing_done
	
+
.processing_done:
	clr KEYBOARD_ADR_EXTENDED_FLAG
	lda #1
	sta KEYBOARD_ADR_NOT_BREAK_FLAG
	rts
	