; video board display for Microchess
	
CHESS_BOARD_HEIGHT=208
CHESS_BOARD_WIDTH_BYTES=78
CHESS_BOARD_ROW_BYTES=80

CHESS_BOARD_SQUARE_SIZE_PIXELS=26
CHESS_BOARD_SQUARE_WIDTH_BYTES=13
CHESS_BOARD_SQUARE_HEIGHT_DEST_BYTES_L=$00
CHESS_BOARD_SQUARE_HEIGHT_DEST_BYTES_H=$0D
CHESS_BOARD_SQUARE_HEIGHT_SRC_BYTES_L=$20
CHESS_BOARD_SQUARE_HEIGHT_SRC_BYTES_H=$08
CHESS_BOARD_SQUARE_HEIGHT_MINUS1_SRC_BYTES_L=$D0
CHESS_BOARD_SQUARE_HEIGHT_MINUS1_SRC_BYTES_H=$07
CHESS_BOARD_FORMAT=$20
	
; driver RAM addresses
MICROCHESS_VIDEO_BASE = CONSOLE_ADR_END+1
MICROCHESS_VIDEO_ADR_A0_L = MICROCHESS_VIDEO_BASE
MICROCHESS_VIDEO_ADR_A0_H = MICROCHESS_VIDEO_BASE+1
MICROCHESS_VIDEO_ADR_A0 = MICROCHESS_VIDEO_BASE
MICROCHESS_VIDEO_SQUARE_COLOR = MICROCHESS_VIDEO_BASE+2


; --- Video zero VRAM
; @noparams
; Clears contents of VRAM to all zeroes.
microchess_video_zero_vram:
	phb
	bank #$20 
			
	!for RowBase, (CHESS_BOARD_HEIGHT>>1) {
		ldx #$00
-		clr VIDEO_RAM_BASE + ((RowBase-1) << 8), X
		inx
		bne -
	}
	
	plb
	rts
	
; --- init
microchess_video_init:	
	jsr microchess_video_zero_vram
			
	; set the bottom console
	jsr console_enable_mixed_mode
	lda #26
	jsr console_set_row
	
	; set the video resolution
	lda #$00 ; 240 lines, mode byte per line
	sta VIDEO_ADR_VSIZE 
	
	; get the image height
	ldy #CHESS_BOARD_HEIGHT
	
	; init dest pointer
	phb
	bank #VIDEO_RAM_BANK
	clr VIDEO_RAM_A0_L
	clr VIDEO_RAM_A0_H
	plb
	
	; set the line mode 
-	ldx #$01
	lda #CHESS_BOARD_FORMAT
	phb
	bank #VIDEO_RAM_BANK
	sta (VIDEO_RAM_A0), X
	
	; increment dest pointer
	bank #VIDEO_RAM_BANK
	lda VIDEO_RAM_A0_L
	eor #$80
	sta VIDEO_RAM_A0_L
	bne +
	inc VIDEO_RAM_A0_H	
+	plb

	dey
	beq ++
	bra -

++	phb
	bank #VIDEO_RAM_BANK
	clr VIDEO_RAM_A0_L
	
	; set mode for one text row
-	ldx #$00
	lda #$41 ; text, 1bpp, palette 1	
	sta (VIDEO_RAM_A0), X
	inx
	sta (VIDEO_RAM_A0), X
	
	; increment dest pointer
	lda #$04
	clc
	adc VIDEO_RAM_A0_H
	sta VIDEO_RAM_A0_H
	
	cmp #(30*4)
	bne -
	
	plb
	
	; setup image palette
	jsr microchess_video_install_palette
	
	rts
	

; --- microchess video draw board
microchess_video_draw_board:
	LDY   	#$00	; init board location to draw

--	LDX #$1F
	TYA					; scan the pieces for a location match
-   CMP	BOARD,X			; match found?
    BEQ drawpiece		; yes; print the piece's color and type
    DEX					; no
    BPL	-				; if not the last piece, try again

	LDX #$CC			; empty square
drawpiece:
    jsr microchess_video_draw_square

	INY				; 
	TYA				; get row number
    AND #$08		; have we completed the row?	 
    BEQ --			; no, do next column
	CLC				; 
	TYA				; 
	ADC	#$08		; point y to beginning of next row
	TAY				;
	CPY   	#$80	; was that the last row?
	BEQ   	+		; yes, print the LED values
	BNE   	--		; no, do new row
+	RTS	

; --- microchess video draw square
; Y = location to draw
; X = piece (CC = empty)
microchess_video_draw_square:
	; determine light or dark square?
	tya
	and	#$01		; odd or even column?				
	sta temp		; save it
	tya  			; is the row odd or even
	lsr				; shift column right 4 spaces 
	lsr				;
	lsr				;
	lsr				;
	and #$01		; strip LSB  
	clc 
	adc temp		; combine row & col to determine square color  
	and #$01		; is board square white or blk?
	bne	+ 			; blk
	lda #$BB
	sta MICROCHESS_VIDEO_SQUARE_COLOR
	bra ++
+	lda #$33
	sta MICROCHESS_VIDEO_SQUARE_COLOR

++	phx
	jsr microchess_video_init_dest_pointer	
	plx
	
	cpx #$CC
	bne +
	jmp	drawempty
+	
	jsr microchess_video_init_source_pointer
	
	; increment to source chess row
	txa				; is the piece white or black?
	and #$F0
	bne +
	clr temp
	bra ++
+	lda #1
	sta temp
++
	
	lda MICROCHESS_VIDEO_SQUARE_COLOR ; is the square dark or light?
	and #$80
	beq +
	lda temp
	clc
	adc #2
	sta temp
		
+	phx
	ldx temp
	
-	beq ++
	lda MICROCHESS_VIDEO_ADR_A0_L
	clc
	adc #CHESS_BOARD_SQUARE_HEIGHT_SRC_BYTES_L
	sta MICROCHESS_VIDEO_ADR_A0_L
	lda MICROCHESS_VIDEO_ADR_A0_H
	adc #CHESS_BOARD_SQUARE_HEIGHT_SRC_BYTES_H
	sta MICROCHESS_VIDEO_ADR_A0_H
	
	dex
	bra -
	
++	plx
	
	; point to last line of row
	lda MICROCHESS_VIDEO_ADR_A0_L
	clc
	adc #CHESS_BOARD_SQUARE_HEIGHT_MINUS1_SRC_BYTES_L
	sta MICROCHESS_VIDEO_ADR_A0_L
	lda MICROCHESS_VIDEO_ADR_A0_H
	adc #CHESS_BOARD_SQUARE_HEIGHT_MINUS1_SRC_BYTES_H
	sta MICROCHESS_VIDEO_ADR_A0_H
		
	; source column
	txa
	and #$0F ; piece type
	
	; get source offset for piece type
	bne + ; king
	lda #0
	bra ++	
+	cmp #$01 ; queen
	bne +
	lda #(CHESS_BOARD_SQUARE_WIDTH_BYTES*1)
	bra ++
+	cmp #$04
	bpl + ; >= 4? no = rook
	lda #(CHESS_BOARD_SQUARE_WIDTH_BYTES*2)	
	bra ++
+	cmp #$06
	bpl + ; >= 6? no = bishop
	lda #(CHESS_BOARD_SQUARE_WIDTH_BYTES*3)	
	bra ++
+	cmp #$08
	bpl +	; >= 8? no = pawn
	lda #(CHESS_BOARD_SQUARE_WIDTH_BYTES*4)	
	bra ++
+   lda #(CHESS_BOARD_SQUARE_WIDTH_BYTES*5)	

++	; add in the column offset
	clc
	adc MICROCHESS_VIDEO_ADR_A0_L
	sta MICROCHESS_VIDEO_ADR_A0_L
	lda MICROCHESS_VIDEO_ADR_A0_H
	adc #0
	sta MICROCHESS_VIDEO_ADR_A0_H
		
	; draw the square
	phy
	ldy #CHESS_BOARD_SQUARE_SIZE_PIXELS
	
--	; copy one line 	
	ldx #(CHESS_BOARD_SQUARE_WIDTH_BYTES-1) 
-	lda (MICROCHESS_VIDEO_ADR_A0), X
	phb
	bank #VIDEO_RAM_BANK
	sta (VIDEO_RAM_A0), X
	plb
	dex
	bpl -	
	
	; decrement source pointer
	lda MICROCHESS_VIDEO_ADR_A0_L
	sec
	sbc #CHESS_BOARD_ROW_BYTES
	sta MICROCHESS_VIDEO_ADR_A0_L
	bcs +
	dec MICROCHESS_VIDEO_ADR_A0_H
+	
	; increment dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda VIDEO_RAM_A0_L
	eor #$80
	sta VIDEO_RAM_A0_L
	and #$80
	bne +
	inc VIDEO_RAM_A0_H	
+	plb

	dey
	beq ++
	bra --

++	ply	
	rts
				
drawempty:	
	; fill empty square
	phy
	ldy #CHESS_BOARD_SQUARE_SIZE_PIXELS
	
--	; copy one line 	
	ldx #(CHESS_BOARD_SQUARE_WIDTH_BYTES-1) 
-	lda MICROCHESS_VIDEO_SQUARE_COLOR
	phb
	bank #VIDEO_RAM_BANK
	sta (VIDEO_RAM_A0), X
	plb
	dex
	bpl -	
	
	; increment dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda VIDEO_RAM_A0_L
	eor #$80
	sta VIDEO_RAM_A0_L
	and #$80
	bne +
	inc VIDEO_RAM_A0_H	
+	plb

	dey
	beq ++
	bra --

++	ply
	rts
	
; --- microchess video draw test
microchess_video_draw_test:
	; init source pointer
	jsr microchess_video_init_source_pointer
	
	ldy microchess_pieces_bitmap+22
	
    ; init dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda #$03 ; skip the last byte, mode byte, and unused byte
	sta VIDEO_RAM_A0_L
	clr VIDEO_RAM_A0_H
	plb
	
--	; copy one line 	
	ldx #(CHESS_BOARD_WIDTH_BYTES-1) 
-	lda (MICROCHESS_VIDEO_ADR_A0), X
	phb
	bank #VIDEO_RAM_BANK
	sta (VIDEO_RAM_A0), X
	plb
	dex
	bpl -	
	
	; decrement source pointer
	lda MICROCHESS_VIDEO_ADR_A0_L
	sec
	sbc #CHESS_BOARD_ROW_BYTES
	sta MICROCHESS_VIDEO_ADR_A0_L
	bcs +
	dec MICROCHESS_VIDEO_ADR_A0_H
+	
	; increment dest pointer
	phb
	bank #VIDEO_RAM_BANK
	lda VIDEO_RAM_A0_L
	eor #$80
	sta VIDEO_RAM_A0_L
	and #$80
	bne +
	inc VIDEO_RAM_A0_H	
+	plb

	dey
	beq ++
	bra --

++		
	rts
	
	
; --- microchess video install palette
;initialize the color palette from offset 54 of the BMP
microchess_video_install_palette:	
	; source pointer
	lda #<(microchess_pieces_bitmap+54)
	sta MICROCHESS_VIDEO_ADR_A0_L
	lda #>(microchess_pieces_bitmap+54)
	sta MICROCHESS_VIDEO_ADR_A0_H
	
	ldy #$00
		
-	tya
	sta VIDEO_ADR_PAL_INDEX
	
	;entries are BGRA format
	ldx #$02
	lda (MICROCHESS_VIDEO_ADR_A0),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; R
	dex
	lda (MICROCHESS_VIDEO_ADR_A0),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; G
	dex
	lda (MICROCHESS_VIDEO_ADR_A0),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; B
	
	; increment source pointer + 4
	lda #$04
	clc
	adc MICROCHESS_VIDEO_ADR_A0_L
	sta MICROCHESS_VIDEO_ADR_A0_L
	bcc +
	inc MICROCHESS_VIDEO_ADR_A0_H	
			
+	iny
	cpy microchess_pieces_bitmap+46 ; all colors installed?
	
	bne -
	
	rts
	
	

; --- microchess video init source pointer
microchess_video_init_source_pointer:   	
	; init source pointer
	lda #<(microchess_pieces_bitmap+58)
	sta MICROCHESS_VIDEO_ADR_A0_L
	lda #>(microchess_pieces_bitmap+58)
	sta MICROCHESS_VIDEO_ADR_A0_H
	
	; add 4 * number of palette entries
	lda microchess_pieces_bitmap+46
	dec
	asl
	asl
	clc
	adc MICROCHESS_VIDEO_ADR_A0_L
	sta MICROCHESS_VIDEO_ADR_A0_L
	bcc +
	inc MICROCHESS_VIDEO_ADR_A0_H
+	lda microchess_pieces_bitmap+46
	dec
	lsr
	lsr
	lsr
	lsr
	lsr
	lsr
	clc
	adc MICROCHESS_VIDEO_ADR_A0_H
	sta MICROCHESS_VIDEO_ADR_A0_H	
	
!if 0 {	
	; get the image height
	ldy microchess_pieces_bitmap+22

	; advance to the last line of source image
	; TODO: perform multiplication instead of repeated addtion
	
	dey
-	lda MICROCHESS_VIDEO_ADR_A0_L
	clc
	adc #CHESS_BOARD_ROW_BYTES
	sta MICROCHESS_VIDEO_ADR_A0_L	
	bcc +
	inc MICROCHESS_VIDEO_ADR_A0_H
+	dey
	bne -
}

	rts
	
		
; --- microchess video init dest pointer
; Y = square index
microchess_video_init_dest_pointer:   
	; init dest pointer to zero
	phb
	bank #VIDEO_RAM_BANK
	lda #$03 ; skip the last byte, mode byte, and unused byte
	sta VIDEO_RAM_A0_L
	clr VIDEO_RAM_A0_H
	plb
	
	tya
	lsr				; shift right 4 spaces to get row index
	lsr
	lsr
	lsr
	tax				; X = row index
	
	; increment dest pointer by row index
-	beq +
	phb
	bank #VIDEO_RAM_BANK
	clc 
	lda VIDEO_RAM_A0_L
	adc #CHESS_BOARD_SQUARE_HEIGHT_DEST_BYTES_L
	sta VIDEO_RAM_A0_L
	lda VIDEO_RAM_A0_H
	adc #CHESS_BOARD_SQUARE_HEIGHT_DEST_BYTES_H
	sta VIDEO_RAM_A0_H			
	plb
	
	dex
	bra -

+	;increment dest pointer by column index
	tya
	and #$07
	tax
	
-	beq +
	phb
	bank #VIDEO_RAM_BANK
	clc 
	lda VIDEO_RAM_A0_L
	adc #CHESS_BOARD_SQUARE_WIDTH_BYTES
	sta VIDEO_RAM_A0_L		
	plb
	
	dex
	bra -
	
+	rts
		
microchess_pieces_bitmap:
!binary "microchess_pieces.bmp"	