* = $0000

; --- Jump to init
	nop
	nop
	nop
	jmp mixed_mode_demo_init

; reserve the remainder of zero page for run-time data
VIDEO_DEMO_PTR:
VIDEO_DEMO_PTR_L: !fi 1
VIDEO_DEMO_PTR_H  !fi 1
VIDEO_ADR_TEMP: !fi 1

DEMO_ADR_LAST_SECOND: !fi 1
DEMO_ADR_IDLE_SECONDS: !fi 1
DEMO_IDLE_TIMEOUT = 30

* = $0100
; includes
DRIVER_RAM_BASE = mixed_mode_demo_end_of_program 
!source "app.s"
!source "lcd.s"
!source "keyhit.s"
!source "video_defs.s"
	
; --- init
mixed_mode_demo_init:	
	lda #<mixed_mode_demo_isr
	sta $00FFFE
	lda #>mixed_mode_demo_isr
	sta $00FFFF	
	+app_init
			
	jsr lcd_standard_init
	jsr keyhit_standard_init
	jsr print_title
	
	cli
	
	; start the program
	jsr video_text_sample
	
	; wait for key
	clr DEMO_ADR_IDLE_SECONDS
-	lda KEYBOARD_ADR_KEY_READY
	bne +
	
	lda $003F20 ; RTC_ADR_SECONDS
	cmp DEMO_ADR_LAST_SECOND
	beq -
	sta DEMO_ADR_LAST_SECOND
	inc DEMO_ADR_IDLE_SECONDS
	lda DEMO_ADR_IDLE_SECONDS
	cmp #DEMO_IDLE_TIMEOUT
	bcc -
	
	; exit
+	jml app_warm_boot
	
; --- Interrupt service routine	
mixed_mode_demo_isr:		
	php
	phb
	pha
	phy
	phx
	
	lda APP_ADR_PROGRAM_BANK
	bank
	
	; was the interrupt triggered by the keyboard?
	lda KEYBOARD_ADR_CONTROL
	and #$1
	bne +
	lda KEYBOARD_ADR_DATA
	jsr keyboard_handle_scan_code
			
+	plx
	ply
	pla
	plb
	plp
	
	rtl ; returns to page 0 wrapper
		
		

; --- Print title string to LCD	 
print_title:
	ldx #$00
	
-	lda demo_titlestr,X
	beq +
	
	phx
	jsr lcd_write_char
	plx
	inx
	
	; terminate after 256 chars, even if NULL is not reached
	bne -

+	rts

	
demo_titlestr:	
!text "Mixed text/graphics demo", "Press any key to exit", 0
	 

; --- Video text sample
; @noparams
; 63x30 text @ 2 colors
video_text_sample:
	lda #$00
	sta VIDEO_ADR_VSIZE ; 240 lines, mode byte per line
	
	jsr video_setup_palette
	jsr video_zero_vram
	
	lda #<video_text_teststring1
	ldy #>video_text_teststring1
	ldx #$0
	jsr video_text_print
	
	lda #<video_text_teststring2
	ldy #>video_text_teststring2
	ldx #1
	jsr video_text_print
	
	lda #<video_text_teststring3
	ldy #>video_text_teststring3
	ldx #2
	jsr video_text_print
		
	lda #<video_text_teststring4
	ldy #>video_text_teststring4
	ldx #4
	jsr video_text_print
	
	lda #<video_text_teststring5
	ldy #>video_text_teststring5
	ldx #5
	jsr video_text_print
	
	lda #<video_text_teststring6
	ldy #>video_text_teststring6
	ldx #6
	jsr video_text_print
	
	lda #<video_text_teststring7
	ldy #>video_text_teststring7
	ldx #7
	jsr video_text_print

	lda #<video_text_teststring10
	ldy #>video_text_teststring10
	ldx #10
	jsr video_text_print
	
	lda #<video_text_teststring8
	ldy #>video_text_teststring8
	ldx #12
	jsr video_text_print
		
	lda #<video_text_teststring9
	ldy #>video_text_teststring9
	ldx #14
	jsr video_text_print
	
	lda #<video_text_teststring11
	ldy #>video_text_teststring11
	ldx #16
	jsr video_text_print
	
	lda #<video_text_teststring12
	ldy #>video_text_teststring12
	ldx #17
	jsr video_text_print
		
	lda #<video_text_teststring13
	ldy #>video_text_teststring13
	ldx #19
	jsr video_text_print
	
	lda #<video_text_teststring14
	ldy #>video_text_teststring14
	ldx #20
	jsr video_text_print
	
	lda #<video_text_teststring15
	ldy #>video_text_teststring15
	ldx #21
	jsr video_text_print
	
	lda #<video_text_teststring16
	ldy #>video_text_teststring16
	ldx #22
	jsr video_text_print
		
	ldx #8	
	jsr video_draw_zigzag
	ldx #9	
	jsr video_draw_zigzag
			
	ldx #24
	jsr video_draw_4bpp_sample
			
	phb
	bank #$20
	
	lda #$40
	sta $0000
	sta $0001
	
	lda #$42
	sta $0400
	sta $0401
	
	lda #$44
	sta $0800
	sta $0801
			
	lda #$4D
	sta $1000
	sta $1001
	
	lda #$45
	sta $1400
	sta $1401
	
	lda #$49
	sta $1800
	sta $1801
	
	lda #$4B
	sta $1C00
	sta $1C01
	
	lda #$46
	sta $2800
	sta $2801
	
	lda #$48
	sta $3000
	sta $3001
	
	lda #$42
	sta $3800
	sta $3801
	
	lda #$41
	sta $4000
	sta $4001
	
	lda #$4F
	sta $4400
	sta $4401
	
	lda #$4A
	sta $4C00
	sta $4C01
	
	lda #$4C
	sta $5000
	sta $5001
	
	lda #$47
	sta $5400
	sta $5401
	
	lda #$4E
	sta $5800
	sta $5801
																												
	plb
	rts
	
		
; --- Video zero VRAM
; @noparams
; Clears contents of VRAM to all zeroes.
video_zero_vram:
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$7E
	sta VIDEO_RAM_A0_H
	
--	ldx #$7F
	lda #0
	
-	sta (VIDEO_RAM_A0),X
	dex
	bpl -
	
	; decrement row pointer
	lda VIDEO_RAM_A0_L
	sec
	sbc #$80
	sta VIDEO_RAM_A0_L
	bcs --
	dec VIDEO_RAM_A0_H
	bpl --
	
	plb 
	rts
	
			
; --- Video setup demo palette
; @noparams
; Sets the RGB value of the first 16 palette entries to be
; white if least significant bit is 1, black if it's 0.
; palette colors
;    0    white on black
;    1	  red on black
;    2    green on black
;    3    blue on black

video_setup_palette:
	ldx #$00
	
-	; background color
	txa
	sta VIDEO_ADR_PAL_INDEX
	and #$10
	bne +
	
	; black background
	lda #0
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	bra ++
	
+	; white backgound
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	
++	inx
	
	; foreground color
	txa
	sta VIDEO_ADR_PAL_INDEX
	
	; foreground.r
	and #$80
	bne +
	
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	bra ++
	
+	lda #0
	sta VIDEO_ADR_PAL_COLOR
	
++	; foreground.g
	txa
	and #$40
	bne +
	
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	bra ++
	
+	lda #0
	sta VIDEO_ADR_PAL_COLOR
	
++	; foreground.b
	txa
	and #$20
	bne +
	
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	bra ++
	
+	lda #0
	sta VIDEO_ADR_PAL_COLOR	
	
++	inx
	
	bne -

	; override palette 1 to pink on dark blue (was all white)
	lda #$0F
-	ora #$10
	sta VIDEO_ADR_PAL_INDEX
	tay
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	lda #$32
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	tya
	and #$0F
	dec
	dec
	bpl -
	
	lda #$0E
-	ora #$10
	sta VIDEO_ADR_PAL_INDEX
	tay
	lda #$00
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	lda #$20
	sta VIDEO_ADR_PAL_COLOR
	tya
	and #$0F
	dec
	dec
	bpl -
	
	; override palette 14 to gray on light green (was all black)
	lda #$0F
-	ora #$E0
	sta VIDEO_ADR_PAL_INDEX
	tay
	lda #$20
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	tya
	and #$0F
	dec
	dec
	bpl -
	
	lda #$0E
-	ora #$E0
	sta VIDEO_ADR_PAL_INDEX
	tay
	lda #$32
	sta VIDEO_ADR_PAL_COLOR
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	lda #$32
	sta VIDEO_ADR_PAL_COLOR
	tya
	and #$0F
	dec
	dec
	bpl -	
	
	; override palette 15 to green-blue on brown (was black on white)
    lda #$0F
-	ora #$F0
	sta VIDEO_ADR_PAL_INDEX
	tay
	lda #$00
	sta VIDEO_ADR_PAL_COLOR
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	lda #$20
	sta VIDEO_ADR_PAL_COLOR
	tya
	and #$0F
	dec
	dec
	bpl -	
		
	lda #$0E
-	ora #$F0
	sta VIDEO_ADR_PAL_INDEX
	tay
	lda #$20
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	lda #$00
	sta VIDEO_ADR_PAL_COLOR
	tya
	and #$0F
	dec
	dec
	bpl -	
		
	; override palette 3 to 16 other colors
	lda #$30
	sta VIDEO_ADR_PAL_INDEX
	lda #$00
	sta VIDEO_ADR_PAL_COLOR
	lda #$00
	sta VIDEO_ADR_PAL_COLOR
	lda #$00
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$31
	sta VIDEO_ADR_PAL_INDEX
	lda #$21
	sta VIDEO_ADR_PAL_COLOR
	lda #$2f
	sta VIDEO_ADR_PAL_COLOR
	lda #$42
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$32
	sta VIDEO_ADR_PAL_INDEX
	lda #$3B
	sta VIDEO_ADR_PAL_COLOR
	lda #$21
	sta VIDEO_ADR_PAL_COLOR
	lda #$25
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$33
	sta VIDEO_ADR_PAL_INDEX
	lda #$7D
	sta VIDEO_ADR_PAL_COLOR
	lda #$23
	sta VIDEO_ADR_PAL_COLOR
	lda #$59
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$34
	sta VIDEO_ADR_PAL_INDEX
	lda #$5C
	sta VIDEO_ADR_PAL_COLOR
	lda #$27
	sta VIDEO_ADR_PAL_COLOR
	lda #$20
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$35
	sta VIDEO_ADR_PAL_INDEX
	lda #$44
	sta VIDEO_ADR_PAL_COLOR
	lda #$5E
	sta VIDEO_ADR_PAL_COLOR
	lda #$63
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$36
	sta VIDEO_ADR_PAL_INDEX
	lda #$3E
	sta VIDEO_ADR_PAL_COLOR
	lda #$35
	sta VIDEO_ADR_PAL_COLOR
	lda #$6D
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$37
	sta VIDEO_ADR_PAL_INDEX
	lda #$39
	sta VIDEO_ADR_PAL_COLOR
	lda #$3C
	sta VIDEO_ADR_PAL_COLOR
	lda #$31
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$38
	sta VIDEO_ADR_PAL_INDEX
	lda #$3B
	sta VIDEO_ADR_PAL_COLOR
	lda #$71
	sta VIDEO_ADR_PAL_COLOR
	lda #$41
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$39
	sta VIDEO_ADR_PAL_INDEX
	lda #$45
	sta VIDEO_ADR_PAL_COLOR
	lda #$71
	sta VIDEO_ADR_PAL_COLOR
	lda #$5F
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$3A
	sta VIDEO_ADR_PAL_INDEX
	lda #$4E
	sta VIDEO_ADR_PAL_COLOR
	lda #$64
	sta VIDEO_ADR_PAL_COLOR
	lda #$2F
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$3B
	sta VIDEO_ADR_PAL_INDEX
	lda #$29
	sta VIDEO_ADR_PAL_COLOR
	lda #$3A
	sta VIDEO_ADR_PAL_COLOR
	lda #$65
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$3C
	sta VIDEO_ADR_PAL_INDEX
	lda #$23
	sta VIDEO_ADR_PAL_COLOR
	lda #$2E
	sta VIDEO_ADR_PAL_COLOR
	lda #$72
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$3D
	sta VIDEO_ADR_PAL_INDEX
	lda #$27
	sta VIDEO_ADR_PAL_COLOR
	lda #$34
	sta VIDEO_ADR_PAL_COLOR
	lda #$69
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$3E
	sta VIDEO_ADR_PAL_INDEX
	lda #$20
	sta VIDEO_ADR_PAL_COLOR
	lda #$59
	sta VIDEO_ADR_PAL_COLOR
	lda #$66
	sta VIDEO_ADR_PAL_COLOR
	
	lda #$3F
	sta VIDEO_ADR_PAL_INDEX
	lda #$4C
	sta VIDEO_ADR_PAL_COLOR
	lda #$2A
	sta VIDEO_ADR_PAL_COLOR
	lda #$52
	sta VIDEO_ADR_PAL_COLOR
					
	rts	
	
	

; --- Video text print	
; @param0 (A) - low byte of string address
; @param1 (Y) - high byte of string address
; @param2 (X) - line number at which to print the string
; prints a null-terminated text string to a screen line (up to 63 bytes)
video_text_print:
	; src pointer
	sta VIDEO_DEMO_PTR_L
	sty VIDEO_DEMO_PTR_H
	
	; dst pointer
	phb
	bank #$20
	stx $7FFE
	clr $7FFF
	
	; dst += lineNum *= 8; lineNum <<= 7
	ldy #10
	
-	asl $7FFE
	bcs +
	asl $7FFF
	bra ++
		
+	asl $7FFF
	inc $7FFF
	
++  dey
	bne -	
 
	; set text mode for the line
	ldx #$00
	lda #$40
	sta ($7FFE),X
	inc $7FFE
	sta ($7FFE),X
	inc $7FFE
	
	; copy loop
	plb
	clr VIDEO_ADR_TEMP
	ldx #$00
	ldy #$00

-	lda (VIDEO_DEMO_PTR),X
	beq +
	
	; invert character escaped with '\'
	cmp #'\'
	bne ++
		
	; set a flag to invert the next character and skip output
	clr VIDEO_ADR_TEMP 
	sec
	ror VIDEO_ADR_TEMP ; sets VIDEO_ADR_TEMP to #$80
	inx
	bra -
	
++	clc
	adc VIDEO_ADR_TEMP
	clr VIDEO_ADR_TEMP
	
	phx
	tyx
	
	phb
	bank #$20
	sta ($7FFE),X
	
	plb
	plx
	
	inx
	iny
	iny
	bra -	

+	
    rts

	
		
; --- Video draw zig-zag	
; @param0 (X) - text line number at which to draw the zig-zag
; prints a zig-zag line
video_draw_zigzag:
	; dst pointer
	phb
	bank #$20
	stx $7FFE
	clr $7FFF
	
	; dst += lineNum *= 8; lineNum <<= 7
	ldy #10
	
-	asl $7FFE
	bcs +
	asl $7FFF
	bra ++
		
+	asl $7FFF
	inc $7FFF
	
++  dey
	bne -	
 
	; set 4bpp mode for the line
	ldx #$00
	lda #$23
	sta ($7FFE),X
	inx
	sta ($7FFE),X
	dex

	inc $7FFF
	sta ($7FFE),X
	inx
	sta ($7FFE),X
	dex

	inc $7FFF
	sta ($7FFE),X
	inx
	sta ($7FFE),X
	dex
	
	inc $7FFF
	sta ($7FFE),X
	inx
	sta ($7FFE),X
	dex
	
	dec $7FFF
	dec $7FFF
	dec $7FFF
	lda #$80
	clc
	adc $7FFE
	sta $7FFE
		
	ldx #$00
	lda #$23		
	sta ($7FFE),X
	inx
	sta ($7FFE),X
	dex

	inc $7FFF
	sta ($7FFE),X
	inx
	sta ($7FFE),X
	dex

	inc $7FFF
	sta ($7FFE),X
	inx
	sta ($7FFE),X
	dex
	
	inc $7FFF
	sta ($7FFE),X
	inx
	sta ($7FFE),X
	dex
	
	dec $7FFF
	dec $7FFF
	dec $7FFF
	lda #$80
	clc
	adc $7FFE
	sta $7FFE
		
	; draw zig zag	
	inc $7FFE
	inc $7FFE
	ldy #125
	
-	tya
	pha
	and #$08
	beq ++++
	pla
	and #$07
	bra +++++
++++ pla
	and #$07
	eor #$07
+++++ beq +
	
	; offset row
	tax
--	lda #$80
	clc
	adc $7FFE
	sta $7FFE
	bcc ++
	inc $7FFF		
++	dex
	bne --
	
+   ; draw the segment
	;lda #$E8
	tya
	rol
	rol
	rol
	tyx
	sta ($7FFE),X
	
	; offset row
	tya
	pha
	and #$08
	beq ++++
	pla
	and #$07
	bra +++++
++++ pla
	and #$07
	eor #$07
+++++ beq +
	
	tax
--	lda #$80
	clc
	adc $7FFE
	sta $7FFE
	bcs ++
	dec $7FFF		
++	dex
	bne --
		
+	dey
	bne -
		
	plb
	rts
	
	
	
; --- Video draw 4bpp sample	
; @param2 (X) - text line number at which to draw the 4bpp sample image
video_draw_4bpp_sample:
	; dst pointer
	phb
	phx
	bank #$20
	stx $7FFE
	clr $7FFF
	
	; dst += lineNum *= 8; lineNum <<= 7
	ldy #10
	
-	asl $7FFE
	bcs +
	asl $7FFF
	bra ++
		
+	asl $7FFF
	inc $7FFF
	
++  dey
	bne -	
	
	; set 4bpp mode for the lines
	ldy #48
-	ldx #$00
	lda #$23
	sta ($7FFE),X
	inx
	sta ($7FFE),X
	dex
	
	dey
	beq +

	lda #$80
	clc
	adc $7FFE
	sta $7FFE
	bcc ++
	inc $7FFF
++	bra -
		
+	; reset the dest pointer
	plx
	;bank #$20
	stx $7FFE
	clr $7FFF
	
	; dst += lineNum *= 8; lineNum <<= 7
	ldy #10
	
-	asl $7FFE
	bcs +
	asl $7FFF
	bra ++
		
+	asl $7FFF
	inc $7FFF
	
++  dey
	bne -
	
	; install the palette	
	plb
	ldx #$02
	ldy #$30
	; src pointer
	lda #<(video_images_leaves+54)
	sta VIDEO_DEMO_PTR_L
	lda #>(video_images_leaves+54)
	sta VIDEO_DEMO_PTR_H
	
-	tya
	sta VIDEO_ADR_PAL_INDEX
	
	lda (VIDEO_DEMO_PTR),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; R
	dex
	lda (VIDEO_DEMO_PTR),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; G
	dex
	lda (VIDEO_DEMO_PTR),X
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR ; B
	inx
	inx
	inx
	inx
	inx
	inx
	iny
	cpy #$40
	bne -
	
	; skip mode bytes
	phb
	bank #$20
	inc $7FFE
	inc $7FFE
	
	; src pointer
	plb
	lda #<(video_images_leaves+118)
	sta VIDEO_DEMO_PTR_L
	lda #>(video_images_leaves+118)
	sta VIDEO_DEMO_PTR_H
		
	; copy 126 bytes
---	ldx #0
-	lda (VIDEO_DEMO_PTR),X
	phb
	bank #$20
	sta ($7FFE),X
	inx
	cpx #126
	plb
	bne -
		
	; add 128 to src pointer for next row
	lda #128
	clc
	adc VIDEO_DEMO_PTR_L
	sta VIDEO_DEMO_PTR_L
	bcc +
	inc VIDEO_DEMO_PTR_H
+	; add 128 to dst pointer for next row
	phb
	bank #$20
	lda #128
	clc
	adc $7FFE
	sta $7FFE
	bcc +
	inc $7FFF
+	; stop when dest high byte >= 80
	lda $7FFF
	plb
	bpl ---
			
	rts
	
video_images_leaves:
!binary "leaves.bmp"

		
; --- Data		
video_text_teststring1:	
!text "                            B M O W                            ", 0
video_text_teststring2:	
!text "                       Big Mess o' Wires                       ", 0
video_text_teststring3:	
!text "   An 8-bit, home-built CPU and computer by Steve Chamberlin   ", 0
video_text_teststring4:	
!text "  64x30 pure text mode, or 63x30 with mixed text and graphics  ", 0
video_text_teststring5:	
!text "                     Each character is 8x16                    ", 0
video_text_teststring6:	
!text "       In mixed mode, each line may have different colors      ", 0
video_text_teststring7:	
!text "The 512x480 display is centered in a standard 640x480 VGA image", 0
video_text_teststring8:	
!text "         Text can be displayed normal or \i\n\v\e\r\t\e\d mode         ", 0
video_text_teststring9:	
!text "  The \i\n\v\e\r\t\e\d text can be used for \h\i\g\h\l\i\g\h\t\i\n\g or cursors \   ", 0
video_text_teststring10:	
!text "         Custom-designed video signal generator circuit        ", 0
video_text_teststring11:	
!text "!", 34, "#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\", 220, "]^_", 0
video_text_teststring12:	
!text "                 `abcdefghijklmnopqrstuvwxyz{|}~               ", 0
video_text_teststring13:	
!text "              'Twas brillig, and the slithy toves              ", 0
video_text_teststring14:	
!text "              Did gyre and gimble in the wabe;                 ", 0
video_text_teststring15:	
!text "              All mimsy were the borogoves,                    ", 0
video_text_teststring16:	
!text "              And the mome raths outgrabe.                     ", 0	
	
mixed_mode_demo_end_of_program:
	