;***************************
;*                         *
;*         APPLE II        *
;*      SYSTEM MONITOR     *
;*                         *
;*     COPYRIGHT 1977 BY   *
;*    APPLE COMPUTER, INC. *
;*                         *
;*    ALL RIGHTS RESERVED  *
;*                         *
;*        S. WOZNIAK       *
;*         A. BAUM         *
;*                         *
;***************************

MONITOR_BASE = 6 ; zero page, preserving bytes for bootstrap init

loc0 = MONITOR_BASE+$00
loc1 = MONITOR_BASE+$01

; schamberlin: added bank address
abank = MONITOR_BASE+$02
asav = MONITOR_BASE+$03
tmpsav = MONITOR_BASE+$04
stksavx = MONITOR_BASE+$05
stksavy = MONITOR_BASE+$06
stksava = MONITOR_BASE+$07

							; unused * 26
							; note offsets below are relied on by the code, do not change without fixing
wndlft = MONITOR_BASE+$20	; Text window left-edge (0-39)
wndwdth = MONITOR_BASE+$21	; Text window width (1-40)
wndtop = MONITOR_BASE+$22	; Text window top-edge (0-23)
wndbtm = MONITOR_BASE+$23	; Text window bottom (1-24)
ch = MONITOR_BASE+$24		; Horizontal cursor-position (0-39)
cv = MONITOR_BASE+$25		; Vertical cursor-position (0-23)
							; unused * 2
basl = MONITOR_BASE+$28		; Address of current text row in video memory
bash = MONITOR_BASE+$29		; Address of current text row in video memory
							; unused * 2
lmnem = MONITOR_BASE+$2C	; Disassembler mnenomic
rtnl = MONITOR_BASE+$2C		; Return address, for single-stepping
rmnem = MONITOR_BASE+$2D	; Disassembler mnenomic
rtnh = MONITOR_BASE+$2D		; Return address, for single-stepping
format = MONITOR_BASE+$2E	; Format specifier for disassembler
length = MONITOR_BASE+$2F	; Instruction length for disassembler
sign = MONITOR_BASE+$2F		; Used by multiplication/division
							; unused
mode = MONITOR_BASE+$31		; Current monitor mode
invflg = MONITOR_BASE+$32	; Text output format [63=INVERSE 255=NORMAL 127=FLASH]
prompt = MONITOR_BASE+$33	; Prompt character
ysav = MONITOR_BASE+$34		; Temporary storage for Y register for monitor program
ysav1 = MONITOR_BASE+$35	; Temporary storage for Y register for monitor program
cswl = MONITOR_BASE+$36		; Output vector
cswh = MONITOR_BASE+$37		; Output vector
kswl = MONITOR_BASE+$38		; Input vector
kswh = MONITOR_BASE+$39		; Input vector
pcl = MONITOR_BASE+$3A		; PC pseudo-register used by monitor program
pch = MONITOR_BASE+$3B		; PC pseudo-register used by monitor program
xqt = MONITOR_BASE+$3C		; Execute area for monitor program
a1l = MONITOR_BASE+$3C		; A1 pseudo-register used by monitor program
a1h = MONITOR_BASE+$3D		; A1 pseudo-register used by monitor program
a2l = MONITOR_BASE+$3E		; A2 pseudo-register used by monitor program
a2h = MONITOR_BASE+$3F		; A2 pseudo-register used by monitor program
a3l = MONITOR_BASE+$40		; A3 pseudo-register used by monitor program
a3h = MONITOR_BASE+$41		; A3 pseudo-register used by monitor program
a4l = MONITOR_BASE+$42		; A4 pseudo-register used by monitor program
a4h = MONITOR_BASE+$43		; A4 pseudo-register used by monitor program
a5l = MONITOR_BASE+$44		; A5 pseudo-register used by monitor program
a5h = MONITOR_BASE+$45		; A5 pseudo-register used by monitor program
acc = MONITOR_BASE+$45		; Saved accumulator 
xreg = MONITOR_BASE+$46		; Saved X
yreg = MONITOR_BASE+$47		; Saved Y
status = MONITOR_BASE+$48	; Saved CC's
spnt = MONITOR_BASE+$49		; Saved SP
							; unused * 4
rndl = MONITOR_BASE+$4E		; Pseudo-random number
rndh = MONITOR_BASE+$4F		; Pseudo-random number
acl = MONITOR_BASE+$50		; Used by multiplication/division
ach = MONITOR_BASE+$51		; Used by multiplication/division
xtndl = MONITOR_BASE+$52	; Used by multiplication/division
xtndh = MONITOR_BASE+$53	; Used by multiplication/division
auxl = MONITOR_BASE+$54		; Used by multiplication/division
auxh = MONITOR_BASE+$55		; Used by multiplication/division
ioadr = MONITOR_BASE+$56	; I/O address for non-default ports?
in = MONITOR_BASE+$57		; Input buffer, 248 characters

MONITOR_ADR_END = in

usradr = $0000				; Address of user subroutine for monitor program
irqloc = $FFFE				; ($00FFFE) Interrupt vector, for simulating IRQ in monitor
spkr = $003F30				; Audio speaker hardware address
pick = $95					; Screen character for lifting existing text?

; schamberlin: added quit
quit:
	jml $000100
	
; schamberlin: added setbank and address ops
	
; A macro to calculate the effective address (a1l),y and set it as SP
!macro a1ysp {	
	SEI
	PHP  ; save registers and CC
	PHX
	PHY
	STA asav ; save A outside of stack
	TSR  ; save SP
	STX stksavx
	STY stksavy
	STA stksava
	LDA 1,S ; restore Y into A
	CLC
	ADC a1l ; A = a1l + Y
	LDY a1h ; Y = a1h + carry
	BCC +
	INY
+	LDX abank ; X = abank
	TRS  ; SP = effective address
}
	
; A macro to calculate the effective address (a3l),y and set it as SP
!macro a3ysp {	
	SEI
	PHP  ; save registers and CC
	PHX
	PHY
	STA asav ; save A outside of stack
	TSR  ; save SP
	STX stksavx
	STY stksavy
	STA stksava
	LDA 1,S ; restore Y into A
	CLC
	ADC a3l ; A = a3l + Y
	LDY a3h ; Y = a3h + carry
	BCC +
	INY
+	LDX abank ; X = abank
	TRS  ; SP = effective address
}
	
; A macro to calculate the effective address (a4l),y and set it as SP
!macro a4ysp {	
	SEI
	PHP  ; save registers and CC
	PHX
	PHY
	STA asav ; save A outside of stack
	TSR  ; save SP
	STX stksavx
	STY stksavy
	STA stksava
	LDA 1,S ; restore Y into A
	CLC
	ADC a4l ; A = a4l + Y
	LDY a4h ; Y = a4h + carry
	BCC +
	INY
+	LDX abank ; X = abank
	TRS  ; SP = effective address
}

; A macro to restore SP from stksav
!macro ressp {	
	LDX stksavx
	LDY stksavy
	LDA stksava
	TRS  ; restore SP
	PLY  ; restore regsiters and CC
	PLX
	PLP
	CLI
}

setbank:
	LDA a2l
	STA abank
	RTS
		
ldaa1y: 
    ; LDA (a1l),y using bank byte
    +a1ysp
	LDA 0,S ; load A from the effective address
	STA asav
	+ressp
	LDA asav
	RTS

ldaa4y: 
    ; LDA (a4l),y using bank byte
    +a4ysp
	LDA 0,S ; load A from the effective address
	STA asav
	+ressp
	LDA asav
	RTS
		
staa3y: 
	; STA (a3l),y using bank byte
	PHA
	+a3ysp
	LDA asav  ; restore original A value
	PHA  ; store A at the effective address
	+ressp
	PLA
	RTS

staa4y: 
	; STA (a4l),y using bank byte
	PHA
	+a4ysp
	LDA asav  ; restore original A value
	PHA  ; store A at the effective address
	+ressp
	PLA
	RTS

cmpa4y: 
	; CMP (a4l),y using bank byte
	PHX
	PHY
	PHA
	STA asav ; save A outside of stack
	TSR  ; save SP
	STX stksavx
	STY stksavy
	STA stksava
	LDA 2,S ; restore Y into A
	CLC
	ADC a4l ; A = a4l + Y
	LDY a4h ; Y = a4h + carry
	BCC +
	INY
+	LDX abank ; X = abank
	TRS  ; SP = effective address
	PLA  ; load value from the effective address
	STA tmpsav
	LDA asav   ; restore original A value
	CMP tmpsav ; do the comparison
	SPX  ; put CC result in tmpsav
	STX tmpsav
	LDX stksavx
	LDY stksavy
	LDA stksava
	TRS  ; restore SP
	LDX tmpsav  ; fetch CC result from tmpsav
	SPX
	PLA  ; restore regsiters
	PLY
	PLX
	RTS

scrn2:
	BCC rtmskz         ;  if even, use lo h                F879: 90 04 SCRN2 BCC rtmskz
	LSR                ;                                   F87B: 4A LSR
	LSR                ;                                   F87C: 4A LSR
	LSR                ;  shift high half byte down        F87D: 4A LSR
	LSR                ;                                   F87E: 4A LSR

rtmskz:
	AND #$0f           ;  mask 4-bits                      F87F: 29 0F RTMSKZ AND #$0f
	RTS                ;                                   F881: 60 RTS

insds1:
	LDX pcl            ;  print pcl,h                      F882: A6 3A INSDS1 LDX pcl
	LDY pch            ;                                   F884: A4 3B LDY pch
	JSR pryx2          ;                                   F886: 20 96 FD JSR pryx2
	JSR prblnk         ;  followed by a blank              F889: 20 48 F9 JSR prblnk
	
	; schamberlin: can't support non-zero bank	
	LDA (pcl,x)        ;  get op code                      F88C: A1 3A LDA (pcl,x)

insds2:
	TAY                ;                                   F88E: A8 INSDS2 TAY
	LSR                ;  even/odd test                    F88F: 4A LSR
	BCC ieven          ;                                   F890: 90 09 BCC ieven
	ROR                ;  bit 1 test                       F892: 6A ROR
	BCS err            ;  xxxxxx11 invalid op              F893: B0 10 BCS err
	CMP #$a2           ;                                   F895: C9 A2 CMP #$a2
	BEQ err            ;  opcode $89 invalid               F897: F0 0C BEQ err
	AND #$87           ;  mask bits                        F899: 29 87 AND #$87

ieven:
	LSR                ;  lsb into carry for l/r test      F89B: 4A IEVEN LSR
	TAX                ;                                   F89C: AA TAX
	LDA fmt1,x         ;  get format index byte            F89D: BD 62 F9 LDA fmt1,x	
	JSR scrn2          ;  r/l h-byte on carry              F8A0: 20 79 F8 JSR scrn2
	BNE getfmt         ;                                   F8A3: D0 04 BNE getfmt

err:	
	LDY #$80           ;  substitute $80 for invalid ops   F8A5: A0 80 ERR ldy
	LDA #$00           ;  set print format index to 0      F8A7: A9 00 LDA #$00

getfmt:
	TAX                ;                                   F8A9: AA GETFMT TAX
	LDA fmt2,x         ;  index into print format table    F8AA: BD A6 F9 LDA fmt2,x
	STA format         ;  save for adr field formatting    F8AD: 85 2E STA format
	AND #$03           ;  mask for 2-bit length (p=1 byte, 1=2 byte, 2=3 byte) F8AF: 29 03 AND #$03
	STA length         ;                                   F8B1: 85 2F STA length
	TYA                ;  opcode                           F8B3: 98 TYA
	AND #$8f           ;  mask for 1xxx1010 test           F8B4: 29 8F AND #$8f
	TAX                ;  save it                          F8B6: AA TAX
	TYA                ;  opcode to a again                F8B7: 98 TYA
	LDY #$03           ;                                   F8B8: A0 03 LDY #$03
	CPX #$8a           ;                                   F8BA: E0 8A CPX #$8a
	BEQ mnndx3         ;                                   F8BC: F0 0B BEQ mnndx3

mnndx1:
	LSR                ;                                   F8BE: 4A MNNDX1 LSR
	BCC mnndx3         ;  form index into mnemonic table   F8BF: 90 08 BCC mnndx3
	LSR                ;                                   F8C1: 4A LSR

mnndx2:
	LSR                ;  1) 1xxx1010->00101xxx            F8C2: 4A MNNDX2 LSR
	ORA #$20           ;  2) xxxyyy01->00111xxx            F8C3: 09 20 ORA #$20
	DEY                ;  3) xxxyyy10->00110xxx            F8C5: 88 DEY
	BNE mnndx2         ;  4) xxxyy100->00100xxx            F8C6: D0 FA BNE mnndx2
	INY                ;  5) xxxxx000->000xxxxx            F8C8: C8 INY

mnndx3:
	DEY                ;                                   F8C9: 88 MNNDX3 DEY
	BNE mnndx1         ;                                   F8CA: D0 F2 BNE mnndx1
	RTS                ;                                   F8CC: 60 RTS
	!byte $ff,$ff,$ff  ;                                   F8CD: FF FF FF DFB $ff,$ff,$ff

instdsp:
	JSR insds1         ;  gen fmt, len bytes               F8D0: 20 82 F8 INSTDSP JSR insds1
	PHA                ;  save mnemonic table index        F8D3: 48 PHA

prntop:
	; schamberlin: can't support non-zero bank	
	LDA (pcl),y        ;                                   F8D4: B1 3A PRNTOP LDA (pcl),y
	
	JSR prbyte         ;                                   F8D6: 20 DA FD JSR prbyte
	LDX #$01           ;  print 2 blanks                   F8D9: A2 01 LDX #$01

prntbl:
	JSR prbl2          ;                                   F8DB: 20 4A F9 PRNTBL JSR prbl2
	CPY length         ;  print inst (1-3 bytes)           F8DE: C4 2F CPY length	
	INY                ;  in a 12 chr field                F8E0: C8 INY	
	BCC prntop         ;                                   F8E1: 90 F1 BCC prntop
	LDX #$03           ;  char count for mnemonic print    F8E3: A2 03 LDX #$03
	CPY #$04           ;                                   F8E5: C0 04 CPY #$04
	BCC prntbl         ;                                   F8E7: 90 F2 BCC prntbl
	PLA                ;  recover mnemonic index           F8E9: 68 PLA
	TAY                ;                                   F8EA: A8 TAY
	LDA mneml,y        ;                                   F8EB: B9 C0 F9 LDA mneml,y
	STA lmnem          ;  fetch 3-char mnemonic            F8EE: 85 2C STA lmnem
	LDA mnemr,y        ;  (packed in 2-bytes)              F8F0: B9 00 FA LDA mnemr,y
	STA rmnem          ;                                   F8F3: 85 2D STA rmnem

prmn1:
	LDA #$00           ;                                   F8F5: A9 00 PRMN1 LDA #$00
	LDY #$05           ;                                   F8F7: A0 05 LDY #$05

prmn2:
	ASL rmnem          ;  shift 5 bits of                  F8F9: 06 2D PRMN2 ASL rmnem
	ROL lmnem          ;  character into a                 F8FB: 26 2C ROL lmnem
	ROL                ;  (clears carry)                   F8FD: 2A ROL
	DEY                ;                                   F8FE: 88 DEY
	BNE prmn2          ;                                   F8FF: D0 F8 BNE prmn2
	CLC
	ADC #$bf           ;  add "?" offset                   F901: 69 BF ADC #$bf
	JSR cout           ;  output a char of mnem            F903: 20 ED FD JSR cout
	DEX                ;                                   F906: CA DEX
	BNE prmn1          ;                                   F907: D0 EC BNE prmn1
	JSR prblnk         ;  output 3 blanks                  F909: 20 48 F9 JSR prblnk
	LDY length         ;                                   F90C: A4 2F LDY length
	LDX #$06           ;  cnt for 6 format bits            F90E: A2 06 LDX #$06

pradr1:
	CPX #$03           ;                                   F910: E0 03 PRADR1 CPX #$03
	BEQ pradr5         ;  if x=3 then addr.                F912: F0 1C BEQ pradr5

pradr2:
	ASL format         ;                                   F914: 06 2E PRADR2 ASL format
	BCC pradr3         ;                                   F916: 90 0E BCC pradr3
	LDA char1-1,x      ;                                   F918: BD B3 F9 LDA char1-1,x
	JSR cout           ;                                   F91B: 20 ED FD JSR cout
	LDA char2-1,x      ;                                   F91E: BD B9 F9 LDA char2-1,x
	BEQ pradr3         ;                                   F921: F0 03 BEQ pradr3
	JSR cout           ;                                   F923: 20 ED FD JSR cout

pradr3:
	DEX                ;                                   F926: CA PRADR3 DEX
	BNE pradr1         ;                                   F927: D0 E7 BNE pradr1
	RTS                ;                                   F929: 60 RTS

pradr4:
	DEY                ;                                   F92A: 88 PRADR4 DEY
	BMI pradr2         ;                                   F92B: 30 E7 BMI pradr2
	JSR prbyte         ;                                   F92D: 20 DA FD JSR prbyte

pradr5:
	LDA format         ;                                   F930: A5 2E PRADR5 LDA format
	CMP #$e8           ;  handle rel adr mode              F932: C9 E8 CMP #$e8
	
	; schamberlin: can't support non-zero bank	
	LDA (pcl),y        ;  special (print target,           F934: B1 3A LDA (pcl),y	
	
	BCC pradr4         ;  not offset)                      F936: 90 F2 BCC pradr4

reladr:
	JSR pcadj3         ;                                   F938: 20 56 F9 RELADR JSR pcadj3
	TAX                ;  pcl,pch+offset+1 to a,y          F93B: AA TAX
	INX                ;                                   F93C: E8 INX
	BNE prntyx         ;  +1 to y,x                        F93D: D0 01 BNE prntyx
	INY                ;                                   F93F: C8 INY

prntyx:
	TYA                ;                                   F940: 98 PRNTYX TYA

prntax:
	JSR prbyte         ;  output target adr                F941: 20 DA FD PRNTAX JSR prbyte

prntx:
	TXA                ;  of branch and return             F944: 8A PRNTX TXA
	JMP prbyte         ;                                   F945: 4C DA FD JMP prbyte

prblnk:
	LDX #$03           ;  blank count                      F948: A2 03 PRBLNK LDX #$03

prbl2:
	LDA #$a0           ;  load a space                     F94A: A9 A0 PRBL2 LDA #$a0

prbl3:
	JSR cout           ;  output a blank                   F94C: 20 ED FD PRBL3 JSR cout
	DEX                ;                                   F94F: CA DEX
	BNE prbl2          ;  loop until count=0               F950: D0 F8 BNE prbl2
	RTS                ;                                   F952: 60 RTS

pcadj:
	SEC                ;  0=1-byte, 1=2-byte               F953: 38 PCADJ SEC

pcadj2:
	LDA length         ;  2=3-byte                         F954: A5 2F PCADJ2 LDA length

pcadj3:
	LDY pch            ;                                   F956: A4 3B PCADJ3 LDY pch
	TAX                ;  test displacement sign           F958: AA TAX
	BPL pcadj4         ;  (for rel branch)                 F959: 10 01 BPL pcadj4
	DEY                ;  extend neg by dec pch            F95B: 88 DEY

pcadj4:
	ADC pcl            ;                                   F95C: 65 3A PCADJ4 ADC pcl
	BCC rts2           ;  pcl+length(or displ)+1 to a      F95E: 90 01 BCC rts2
	INY                ;  carry into y (pch)               F960: C8 INY

rts2:
	RTS                ;                                   F961: 60 RTS2 RTS
                       ;* FMT1 BYTES: XXXXXXY0 INSTRS
                       ;* IF Y=0 THEN LEFT HALF BYTE
                       ;* IF Y=1 THEN RIGHT HALF BYTE
                       ;* (X=INDEX)
	                   ;                                   F962: 04 20 54

fmt1:
	!byte $04,$20,$54,$30,$0d;                             F965: 30 0D FMT1 DFB $04,$20,$54,$30,$0d
	                   ;                                   F967: 80 04 90
	!byte $80,$04,$90,$03,$22;                             F96A: 03 22 DFB $80,$04,$90,$03,$22
	                   ;                                   F96C: 54 33 0D
	!byte $54,$33,$0d,$80,$04;                             F96F: 80 04 DFB $54,$33,$0d,$80,$04
	                   ;                                   F971: 90 04 20
	!byte $90,$04,$20,$54,$33;                             F974: 54 33 DFB $90,$04,$20,$54,$33
	                   ;                                   F976: 0D 80 04
	!byte $0d,$80,$04,$90,$04;                             F979: 90 04 DFB $0d,$80,$04,$90,$04
	                   ;                                   F97B: 20 54 3B
	!byte $20,$54,$3b,$0d,$80;                             F97E: 0D 80 DFB $20,$54,$3b,$0d,$80
	                   ;                                   F980: 04 90 00
	!byte $04,$90,$00,$22,$44;                             F983: 22 44 DFB $04,$90,$00,$22,$44
	                   ;                                   F985: 33 0D C8
	!byte $33,$0d,$c8,$44,$00;                             F988: 44 00 DFB $33,$0d,$c8,$44,$00
	                   ;                                   F98A: 11 22 44
	!byte $11,$22,$44,$33,$0d;                             F98D: 33 0D DFB $11,$22,$44,$33,$0d
	                   ;                                   F98F: C8 44 A9
	!byte $c8,$44,$a9,$01,$22;                             F992: 01 22 DFB $c8,$44,$a9,$01,$22
	                   ;                                   F994: 44 33 0D
	!byte $44,$33,$0d,$80,$04;                             F997: 80 04 DFB $44,$33,$0d,$80,$04
	                   ;                                   F999: 90 01 22
	!byte $90,$01,$22,$44,$33;                             F99C: 44 33 DFB $90,$01,$22,$44,$33
	                   ;                                   F99E: 0D 80 04
	!byte $0d,$80,$04,$90;                                 F9A1: 90 DFB $0d,$80,$04,$90
	                   ;                                   F9A2: 26 31 87
	!byte $26,$31,$87,$9a;  $zzxxxy01 instr's              F9A5: 9A DFB $26,$31,$87,$9a

fmt2:
	!byte $00          ;  err                              F9A6: 00 FMT2 DFB $00
	!byte $21          ;  imm                              F9A7: 21 DFB $21
	!byte $81          ;  z-page                           F9A8: 81 DFB $81
	!byte $82          ;  abs                              F9A9: 82 DFB $82
	!byte $00          ;  implied                          F9AA: 00 DFB $00
	!byte $00          ;  accumulator                      F9AB: 00 DFB $00
	!byte $59          ;  (zpag,x)                         F9AC: 59 DFB $59
	!byte $4d          ;  (zpag),y                         F9AD: 4D DFB $4d
	!byte $91          ;  zpag,x                           F9AE: 91 DFB $91
	!byte $92          ;  abs,x                            F9AF: 92 DFB $92
	!byte $86          ;  abs,y                            F9B0: 86 DFB $86
	!byte $4a          ;  (abs)                            F9B1: 4A DFB $4a
	!byte $85          ;  zpag,y                           F9B2: 85 DFB $85
	!byte $9d          ;  relative                         F9B3: 9D DFB $9d
	                   ;                                   F9B4: AC A9 AC
	                   ;                                   F9B7: A3 A8 A4
	                   
char1:	                   
	!text ",),#($"
	                   ;                                   F9BA: D9 00 D8

char2:
	!byte $d9,$00,$d8,$a4,$a4,$00;                         F9BD: A4 A4 00 CHAR2 DFB $d9,$00,$d8,$a4,$a4,$00
	                   ;*CHAR2: "Y",0,"X$$",0
	                   ;* MNEML IS OF FORM:
	                   ;* (A) XXXXX000
	                   ;* (B) XXXYY100
	                   ;* (C) 1XXX1010
	                   ;* (D) XXXYYY10
	                   ;* (E) XXXYYY01
;* (X=INDEX)
	                   ;                                   F9C0: 1C 8A 1C

mneml:
	!byte $1c,$8a,$1c,$23,$5d,$8b;                         F9C3: 23 5D 8B MNEML DFB $1c,$8a,$1c,$23,$5d,$
	                   ;                                   F9C6: 1B A1 9D
	!byte $1b,$a1,$9d,$8a,$1d,$23;                         F9C9: 8A 1D 23 DFB $1b,$a1,$9d,$8a,$1d,$23
	                   ;                                   F9CC: 9D 8B 1D
	!byte $9d,$8b,$1d,$a1,$00,$29;                         F9CF: A1 00 29 DFB $9d,$8b,$1d,$a1,$00,$29
	                   ;                                   F9D2: 19 AE 69
	!byte $19,$ae,$69,$a8,$19,$23;                         F9D5: A8 19 23 DFB $19,$ae,$69,$a8,$19,$23
	                   ;                                   F9D8: 24 53 1B
	!byte $24,$53,$1b,$23,$24,$53;                         F9DB: 23 24 53 DFB $24,$53,$1b,$23,$24,$53
	!byte $19,$a1      ;  (a) format above                 F9DE: 19 A1 DFB $19,$a1
	                   ;                                   F9E0: 00 1A 5B
	!byte $00,$1a,$5b,$5b,$a5,$69;                         F9E3: 5B A5 69 DFB $00,$1a,$5b,$5b,$a5,$69
	!byte $24,$24      ;  (b) format                       F9E6: 24 24 DFB $24,$24
	                   ;                                   F9E8: AE AE A8
	!byte $ae,$ae,$a8,$ad,$29,$00;                         F9EB: AD 29 00 DFB $ae,$ae,$a8,$ad,$29,$00
	!byte $7c,$00      ;  (c) format                       F9EE: 7C 00 DFB $7c,$00
	                   ;                                   F9F0: 15 9C 6D
	!byte $15,$9c,$6d,$9c,$a5,$69;                         F9F3: 9C A5 69 DFB $15,$9c,$6d,$9c,$a5,$69
	!byte $29,$53      ;  (d) format                       F9F6: 29 53 DFB $29,$53
	                   ;                                   F9F8: 84 13 34
	!byte $84,$13,$34,$11,$a5,$69;                         F9FB: 11 A5 69 DFB $84,$13,$34,$11,$a5,$69
	!byte $23,$a0      ;  (e) format                       F9FE: 23 A0 DFB $23,$a0
	                   ;                                   FA00: D8 62 5A

mnemr:
	!byte $d8,$62,$5a,$48,$26,$62;                         FA03: 48 26 62 MNEMR DFB $d8,$62,$5a,$48,$26,$62
	                   ;                                   FA06: 94 88 54
	!byte $94,$88,$54,$44,$c8,$54;                         FA09: 44 C8 54 DFB $94,$88,$54,$44,$c8,$54
	                   ;                                   FA0C: 68 44 E8
	!byte $68,$44,$e8,$94,$00,$b4;                         FA0F: 94 00 B4 DFB $68,$44,$e8,$94,$00,$b4
	                   ;                                   FA12: 08 84 74
	!byte $08,$84,$74,$b4,$28,$6e;                         FA15: B4 28 6E DFB $08,$84,$74,$b4,$28,$6e
	                   ;                                   FA18: 74 F4 CC
	!byte $74,$f4,$cc,$4a,$72,$f2;                         FA1B: 4A 72 F2 DFB $74,$f4,$cc,$4a,$72,$f2
	!byte $a4,$8a      ;  (a) format                       FA1E: A4 8A DFB $a4,$8a
	                   ;                                   FA20: 00 AA A2
	!byte $00,$aa,$a2,$a2,$74,$74;                         FA23: A2 74 74 DFB $00,$aa,$a2,$a2,$74,$74
	!byte $74,$72      ;  (b) format                       FA26: 74 72 DFB $74,$72
	                   ;                                   FA28: 44 68 B2
	!byte $44,$68,$b2,$32,$b2,$00;                         FA2B: 32 B2 00 DFB $44,$68,$b2,$32,$b2,$00
	!byte $22,$00      ;  (c) format                       FA2E: 22 00 DFB $22,$00
	                   ;                                   FA30: 1A 1A 26
	!byte $1a,$1a,$26,$26,$72,$72;                         FA33: 26 72 72 DFB $1a,$1a,$26,$26,$72,$72
	!byte $88,$c8      ;  (d) format                       FA36: 88 C8 DFB $88,$c8
	                   ;                                   FA38: C4 CA 26
	!byte $c4,$ca,$26,$48,$44,$44;                         FA3B: 48 44 44 DFB $c4,$ca,$26,$48,$44,$44
	!byte $a2,$c8      ;  (e) format                       FA3E: A2 C8 DFB $a2,$c8
	!byte $ff,$ff,$ff  ;                                   FA40: FF FF FF DFB $ff,$ff,$ff

step:
	JSR instdsp        ;  disassemble one inst             FA43: 20 D0 F8 STEP JSR instdsp
	PLA                ;  at (pcl,h)                       FA46: 68 PLA
	STA rtnl           ;  adjust to user                   FA47: 85 2C STA rtnl
	PLA                ;  stack. save                      FA49: 68 PLA
	STA rtnh           ;  rtn adr.                         FA4A: 85 2D STA rtnh
	LDX #$08           ;                                   FA4C: A2 08 LDX #$08

xqinit:
	LDA initbl-1,x     ;  init xeq area                    FA4E: BD 10 FB XQINIT LDA initbl-1,x
	STA xqt,x          ;                                   FA51: 95 3C STA xqt,x
	DEX                ;                                   FA53: CA DEX
	BNE xqinit         ;                                   FA54: D0 F8 BNE xqinit
	LDA (pcl,x)        ;  user opcode byte                 FA56: A1 3A LDA (pcl,x)
	BEQ xbrk           ;  special if break                 FA58: F0 42 BEQ xbrk
	LDY length         ;  len from disassembly             FA5A: A4 2F LDY length
	CMP #$20           ;                                   FA5C: C9 20 CMP #$20
	BEQ xjsr           ;  handle jsr, rts, jmp,            FA5E: F0 59 BEQ xjsr
	CMP #$60           ;  jmp (), rti special              FA60: C9 60 CMP #$60
	BEQ xrts           ;                                   FA62: F0 45 BEQ xrts
	CMP #$4c           ;                                   FA64: C9 4C CMP #$4c
	BEQ xjmp           ;                                   FA66: F0 5C BEQ xjmp
	CMP #$6c           ;                                   FA68: C9 6C CMP #$6c
	BEQ xjmpat         ;                                   FA6A: F0 59 BEQ xjmpat
	CMP #$40           ;                                   FA6C: C9 40 CMP #$40
	BEQ xrti           ;                                   FA6E: F0 35 BEQ xrti
	AND #$1f           ;                                   FA70: 29 1F AND #$1f
	EOR #$14           ;                                   FA72: 49 14 EOR #$14
	CMP #$04           ;  copy user inst to xeq area       FA74: C9 04 CMP #$04
	BEQ xq2            ;  with trailing nops               FA76: F0 02 BEQ xq2
	
xq1:	
	LDA (pcl),y        ;  change rel branch                FA78: B1 3A XQ1 LDA (pcl),y

xq2:	
	sta xqt,y          ;   disp to 4 for                   FA7A: 99 3C 00 XQ2 STA xqt,y
	
	DEY                ;  jmp to branch or                 FA7D: 88 DEY
	BPL xq1            ;  nbranch from xeq.                FA7E: 10 F8 BPL xq1
	JSR restore        ;  restore user reg contents.       FA80: 20 3F FF JSR restore
	JMP xqt            ;  xeq user op from ram             FA83: 4C 3C 00 JMP xqt
	
irq:	
	STA acc            ;  (return to nbranch)              FA86: 85 45 IRQ sta
	PLA                ;                                   FA88: 68 PLA
	PHA                ;  **irq handler                    FA89: 48 PHA
	ASL                ;                                   FA8A: 0A ASL
	ASL                ;                                   FA8B: 0A ASL
	ASL                ;                                   FA8C: 0A ASL
	BMI break          ;  test for break                   FA8D: 30 03 BMI break
	JMP (irqloc)       ;  user routine vector in ram       FA8F: 6C FE 03 JMP (irqloc)

break:
	PLP                ;                                   FA92: 28 BREAK PLP
	JSR sav1           ;  save reg's on break              FA93: 20 4C FF JSR sav1
	PLA                ;  including pc                     FA96: 68 PLA
	STA pcl            ;                                   FA97: 85 3A STA pcl
	PLA                ;                                   FA99: 68 PLA
	STA pch            ;                                   FA9A: 85 3B STA pch

xbrk:
	JSR insds1         ;  print user pc.                   FA9C: 20 82 F8 XBRK JSR insds1
	JSR rgdsp1         ;  and reg's                        FA9F: 20 DA FA JSR rgdsp1
	JMP mon            ;  go to monitor                    FAA2: 4C 65 FF JMP mon

xrti:
	CLC                ;                                   FAA5: 18 XRTI CLC
	PLA                ;  simulate rti by expecting        FAA6: 68 PLA
	STA status         ;  status from stack, then rts      FAA7: 85 48 STA status

xrts:
	PLA                ;  rts simulation                   FAA9: 68 XRTS PLA
	STA pcl            ;  extract pc from stack            FAAA: 85 3A STA pcl
	PLA                ;  and update pc by 1 (len=0)       FAAC: 68 PLA

pcinc2:
	STA pch            ;                                   FAAD: 85 3B PCINC2 STA pch

pcinc3:
	LDA length         ;  update pc by len                 FAAF: A5 2F PCINC3 LDA length
	JSR pcadj3         ;                                   FAB1: 20 56 F9 JSR pcadj3
	STY pch            ;                                   FAB4: 84 3B STY pch
	CLC                ;                                   FAB6: 18 CLC
	BCC newpcl         ;                                   FAB7: 90 14 BCC newpcl

xjsr:
	CLC                ;                                   FAB9: 18 XJSR CLC
	JSR pcadj2         ;  update pc and push               FABA: 20 54 F9 JSR pcadj2
	TAX                ;  onto stach for                   FABD: AA TAX
	TYA                ;  jsr simulate                     FABE: 98 TYA
	PHA                ;                                   FABF: 48 PHA
	TXA                ;                                   FAC0: 8A TXA
	PHA                ;                                   FAC1: 48 PHA
	LDY #$02           ;                                   FAC2: A0 02 LDY #$02

xjmp:
	CLC                ;                                   FAC4: 18 XJMP CLC

xjmpat:
	LDA (pcl),y        ;                                   FAC5: B1 3A XJMPAT LDA (pcl),y
	TAX                ;  load pc for jmp,                 FAC7: AA TAX
	DEY                ;  (jmp) simulate.                  FAC8: 88 DEY	
	LDA (pcl),y        ;                                   FAC9: B1 3A LDA (pcl),y
	STX pch            ;                                   FACB: 86 3B STX pch

newpcl:
	STA pcl            ;                                   FACD: 85 3A NEWPCL STA pcl
	BCS xjmp           ;                                   FACF: B0 F3 BCS xjmp

rtnjmp:
	LDA rtnh           ;                                   FAD1: A5 2D RTNJMP LDA rtnh
	PHA                ;                                   FAD3: 48 PHA
	LDA rtnl           ;                                   FAD4: A5 2C LDA rtnl
	PHA                ;                                   FAD6: 48 PHA

regdsp:
	JSR crout          ;  display user reg                 FAD7: 20 8E FD REGDSP JSR crout

rgdsp1:
	LDA #<acc          ;  contents with                    FADA: A9 45 RGDSP1 LDA #acc
	STA a3l            ;  labels                           FADC: 85 40 STA a3l
	LDA #>acc          ;                                   FADE: A9 00 LDA #acc/256
	STA a3h            ;                                   FAE0: 85 41 STA a3h
	LDX #$fb           ;                                   FAE2: A2 FB LDX #$fb

rdsp1:
	LDA #$a0           ;                                   FAE4: A9 A0 RDSP1 LDA #$a0
	JSR cout           ;                                   FAE6: 20 ED FD JSR cout
	LDA rtbl-$fb,x     ;                                   FAE9: BD 1E FA LDA rtbl-$fb,x
	JSR cout           ;                                   FAEC: 20 ED FD JSR cout
	LDA #$bd           ;                                   FAEF: A9 BD LDA #$bd
	JSR cout           ;                                   FAF1: 20 ED FD JSR cout
	LDA acc+5,x        ;                                   FAF4: B5 4A LDA acc+5,x
	JSR prbyte         ;                                   FAF6: 20 DA FD JSR prbyte
	INX                ;                                   FAF9: E8 INX
	BMI rdsp1          ;                                   FAFA: 30 E8 BMI rdsp1
	RTS                ;                                   FAFC: 60 RTS

branch:
	CLC                ;  branch taken,                    FAFD: 18 BRANCH CLC
	LDY #$01           ;  add len+2 to pc                  FAFE: A0 01 LDY #$01
	LDA (pcl),y        ;                                   FB00: B1 3A LDA (pcl),y
	JSR pcadj3         ;                                   FB02: 20 56 F9 JSR pcadj3
	STA pcl            ;                                   FB05: 85 3A STA pcl
	TYA                ;                                   FB07: 98 TYA
	SEC                ;                                   FB08: 38 SEC
	BCS pcinc2         ;                                   FB09: B0 A2 BCS pcinc2

nbrnch:
	JSR save           ;  normal return after              FB0B: 20 4A FF NBRNCH JSR save
	SEC                ;  xeq user of                      FB0E: 38 SEC
	BCS pcinc3         ;  go update pc                     FB0F: B0 9E BCS pcinc3

initbl:
	NOP                ;                                   FB11: EA INITBL NOP
	NOP                ;  dummy fill for                   FB12: EA NOP
	JMP nbrnch         ;  xeq area                         FB13: 4C 0B FB JMP nbrnch
	JMP branch         ;                                   FB16: 4C FD FA JMP branch

rtbl:
	!byte $c1          ;                                   FB19: C1 RTBL DFB $c1
	!byte $d8          ;                                   FB1A: D8 DFB $d8
	!byte $d9          ;                                   FB1B: D9 DFB $d9
	!byte $d0          ;                                   FB1C: D0 DFB $d0
	!byte $d3          ;                                   FB1D: D3 DFB $d3

init:
	LDA #$00           ;  clr status for debug             FB2F: A9 00 INIT LDA #$00
	STA status         ;  software                         FB31: 85 48 STA status

settxt:
	LDA #$00           ;  full screen window               FB3C: A9 00 LDA #$00

setwnd:
	STA wndtop         ;  set for 40 col window            FB4B: 85 22 SETWND STA wndtop
	LDA #$00           ;  top in a-reg,                    FB4D: A9 00 LDA #$00
	STA wndlft         ;  bttm at line 24                  FB4F: 85 20 STA wndlft
	; schamberlin: changed window width
	LDA #$FF ;#$28           ;                                   FB51: A9 28 LDA #$28
	STA wndwdth        ;                                   FB53: 85 21 STA wndwdth
	LDA #$18           ;                                   FB55: A9 18 LDA #$18
	STA wndbtm         ;  vtab to row 23                   FB57: 85 23 STA wndbtm
	LDA #$17           ;                                   FB59: A9 17 LDA #$17

tabv:
	STA cv             ;  vtabs to row in a-reg            FB5B: 85 25 TABV STA cv
	JMP vtab           ;                                   FB5D: 4C 22 FC JMP vtab

mulpm: 
	JSR md1            ;  abs val of ac aux                FB60: 20 A4 FB MULPM JSR md1

mul:	
	LDY #$10           ;  index for 16 bits                FB63: A0 10 MUL LDY #$10

mul2:
	LDA acl            ;  acx * aux + xtnd                 FB65: A5 50 MUL2 LDA acl
	LSR                ;  to ac, xtnd                      FB67: 4A LSR
	BCC mul4           ;  if no carry,                     FB68: 90 0C BCC mul4
	CLC                ;  no partial prod.                 FB6A: 18 CLC
	LDX #$fe           ;                                   FB6B: A2 FE LDX #$fe

mul3:
	LDA xtndl+2,x      ;  add mplcnd (aux)                 FB6D: B5 54 MUL3 LDA xtndl+2,x
	ADC auxl+2,x       ;  to partial prod                  FB6F: 75 56 ADC auxl+2,x
	STA xtndl+2,x      ;  (xtnd)                           FB71: 95 54 STA xtndl+2,x
	INX                ;                                   FB73: E8 INX
	BNE mul3           ;                                   FB74: D0 F7 BNE mul3

mul4:
	LDX #$03           ;                                   FB76: A2 03 MUL4 LDX #$03

mul5:
	; schamberlin: changed two explicit byte values below to ROR acl,x
	ROR acl,x
	;!byte $76          ;                                   FB78: 76 MUL5 DFB $76
	;!byte $50          ;                                   FB79: 50 DFB $50
	DEX                ;                                   FB7A: CA DEX
	BPL mul5           ;                                   FB7B: 10 FB BPL mul5
	DEY                ;                                   FB7D: 88 DEY
	BNE mul2           ;                                   FB7E: D0 E5 BNE mul2
	RTS                ;                                   FB80: 60 RTS

divpm:
	JSR md1            ;  abs val of ac, aux.              FB81: 20 A4 FB DIVPM JSR md1
	
div:	
	LDY #$10           ;  index for 16 bits                FB84: A0 10 DIV LDY #$10

div2:
	ASL acl            ;                                   FB86: 06 50 DIV2 ASL acl
	ROL ach            ;                                   FB88: 26 51 ROL ach
	ROL xtndl          ;  xtnd/aux                         FB8A: 26 52 ROL xtndl
	ROL xtndh          ;  to ac.                           FB8C: 26 53 ROL xtndh
	SEC                ;                                   FB8E: 38 SEC
	LDA xtndl          ;                                   FB8F: A5 52 LDA xtndl
	SBC auxl           ;  mod to xtnd.                     FB91: E5 54 SBC auxl
	TAX                ;                                   FB93: AA TAX
	LDA xtndh          ;                                   FB94: A5 53 LDA xtndh
	SBC auxh           ;                                   FB96: E5 55 SBC auxh
	BCC div3           ;                                   FB98: 90 06 BCC div3
	STX xtndl          ;                                   FB9A: 86 52 STX xtndl
	STA xtndh          ;                                   FB9C: 85 53 STA xtndh
	INC acl            ;                                   FB9E: E6 50 INC acl

div3:
	DEY                ;                                   FBA0: 88 DIV3 DEY
	BNE div2           ;                                   FBA1: D0 E3 BNE div2
	RTS                ;                                   FBA3: 60 RTS
	
md1:	
	LDY #$00           ;  abs val of ac, aux               FBA4: A0 00 MD1 LDY #$00 
	STY sign           ;  with result sign                 FBA6: 84 2F STY sign
	; schamberlin: change offsets below to be relative to MONITOR_BASE
	LDX #(auxl-MONITOR_BASE)    ;  in lsb of sign.                  FBA8: A2 54 LDX #auxl
	JSR md3            ;                                   FBAA: 20 AF FB JSR md3
	LDX #(acl-MONITOR_BASE)     ;                                   FBAD: A2 50 LDX #acl
	
md3:	
	LDA loc1,x          ;  x specifies ac or aux            FBAF: B5 01 MD3 LDA loc1,x
	BPL mdrts          ;                                   FBB1: 10 0D BPL mdrts
	SEC                ;                                   FBB3: 38 SEC
	TYA                ;                                   FBB4: 98 TYA
	SBC loc0,x          ;  compl specified reg              FBB5: F5 00 SBC loc0,x
	STA loc0,x          ;  if neg.                          FBB7: 95 00 STA loc0,x
	TYA                ;                                   FBB9: 98 TYA
	SBC loc1,x          ;                                   FBBA: F5 01 SBC loc1,x
	STA loc1,x          ;                                   FBBC: 95 01 STA loc1,x
	INC sign           ;                                   FBBE: E6 2F INC sign

mdrts:
	RTS                ;                                   FBC0: 60 MDRTS RTS

bascalc:
	PHA                ;  calc base adr in basl,h          FBC1: 48 BASCALC PHA
	LSR                ;  for given line no                FBC2: 4A LSR
	AND #$03           ;  0&lt=line no.&lt=$17             FBC3: 29 03 AND #$03
	ORA #$04           ;  arg=000abcde, generate           FBC5: 09 04 ORA #$04
	STA bash           ;  bash=000001cd                    FBC7: 85 29 STA bash
	PLA                ;  and                              FBC9: 68 PLA	
	AND #$18           ;  basl=eabab000                    FBCA: 29 18 AND #$18  	
	BCC bsclc2         ;                                   FBCC: 90 02 BCC bsclc2
	ADC #$7f           ;                                   FBCE: 69 7F ADC #$7f
  
bsclc2:
	STA basl           ;                                   FBD0: 85 28 BSCLC2 STA basl
	ASL                ;                                   FBD2: 0A ASL
	ASL                ;                                   FBD3: 0A ASL
	ORA basl           ;                                   FBD4: 05 28 ORA basl
	STA basl           ;                                   FBD6: 85 28 STA basl
	RTS                ;                                   FBD8: 60 RTS

bell1:
	CMP #$87           ;  bell char? (cntrl-g)             FBD9: C9 87 BELL1 CMP #$87
	BNE rts2b          ;  no, return                       FBDB: D0 12 BNE rts2b
	; schamberlin: changed to a single read of spkr for compatability with BMOW simulator
	LDA spkr

rts2b:
	RTS                ;                                   FBEF: 60 RTS2B RTS

stoadv:
	LDY ch             ;  cursor h index to y-reg          FBF0: A4 24 STOADV LDY ch
	; schamberlin: replaced original text out subroutine
	AND #$7f ; mask out the non-inverted flag	
+	PHX
	PHY
	JSR console_write_char       ;  store char in line               FBF2: 91 28 STA (basl),y
	PLY
	PLX
	
advance:
	INC ch             ;  increment cursor h index         FBF4: E6 24 ADVANCE INC ch
	LDA ch             ;  (move right)                     FBF6: A5 24 LDA ch
	CMP wndwdth        ;  beyond window width?             FBF8: C5 21 CMP wndwdth
	BCS cr             ;  yes cr to next line              FBFA: B0 66 BCS cr

rts3:
	RTS                ;  no,return                        FBFC: 60 RTS3 RTS

vidout:
	CMP #$a0           ;  control char?                    FBFD: C9 A0 VIDOUT CMP #$a0
	BCS stoadv         ;  no,output it.                    FBFF: B0 EF BCS stoadv
	TAY                ;  inverse video?                   FC01: A8 TAY
	BPL stoadv         ;  yes, output it.                  FC02: 10 EC BPL stoadv
	CMP #$8d           ;  cr?                              FC04: C9 8D CMP #$8d
	BEQ cr             ;  yes.                             FC06: F0 5A BEQ cr
	CMP #$8a           ;  line feed?                       FC08: C9 8A CMP #$8a
	BEQ lf             ;  if so, do it.                    FC0A: F0 5A BEQ lf
	CMP #$88           ;  back space? (cntrl-h)            FC0C: C9 88 CMP #$88
	; schamberlin: added BEQ case for printing backspace
	BEQ stoadv
	BNE bell1          ;  no, check for bell.              FC0E: D0 C9 BNE bell1
	
bs:	
	; schamberlin: added print of backspace character
	LDA #$88
	JSR cout
	
	DEC ch             ;  decrement cursor h index         FC10: C6 24 BS DEC ch
	BPL rts3           ;  if pos, ok. else move up         FC12: 10 E8 BPL rts3
	LDA wndwdth        ;  set ch to wndwdth-1              FC14: A5 21 LDA wndwdth
	STA ch             ;                                   FC16: 85 24 STA ch
	DEC ch             ;  (rightmost screen pos)           FC18: C6 24 DEC ch
	
up:	
	LDA wndtop         ;  cursor v index                   FC1A: A5 22 UP LDA wndtop
	CMP cv             ;                                   FC1C: C5 25 CMP cv
	BCS rts4           ;  if top line then return          FC1E: B0 0B BCS rts4
	DEC cv             ;  dec cursor v-index               FC20: C6 25 DEC cv

vtab:
	LDA cv             ;  get cursor v-index               FC22: A5 25 VTAB LDA cv

vtabz:
	JSR bascalc        ;  generate base adr                FC24: 20 C1 FB VTABZ JSR bascalc
	ADC wndlft         ;  add window left index            FC27: 65 20 ADC wndlft
	STA basl           ;  to basl                          FC29: 85 28 STA basl

rts4:
	RTS                ;                                   FC2B: 60 RTS4 RTS

esc1:
	EOR #$c0           ;  esc?                             FC2C: 49 C0 ESC1 EOR #$c0
	BEQ home           ;  if so, do home and clear         FC2E: F0 28 BEQ home
	ADC #$fd           ;  esc-a or b check                 FC30: 69 FD ADC #$fd
	BCC advance        ;  a, advance                       FC32: 90 C0 BCC advance
	BEQ bs             ;  b, backspace                     FC34: F0 DA BEQ bs
	ADC #$fd           ;  esc-c or d check                 FC36: 69 FD ADC #$fd
	BCC lf             ;  c, down                          FC38: 90 2C BCC lf
	BEQ up             ;  d, go up                         FC3A: F0 DE BEQ up
	ADC #$fd           ;  esc-e or f check                 FC3C: 69 FD ADC #$fd
	BCC clreol         ;  e, clear to end of line          FC3E: 90 5C BCC clreol
	BNE rts4           ;  not f, return                    FC40: D0 E9 BNE rts4

clreop:
	LDY ch             ;  cursor h to y index              FC42: A4 24 CLREOP LDY ch
	LDA cv             ;  cursor v to a-register           FC44: A5 25 LDA cv

cleop1:
	PHA                ;  save current line on stk         FC46: 48 CLEOP1 PHA
	JSR vtabz          ;  calc base address                FC47: 20 24 FC JSR vtabz
	JSR cleolz         ;  clear to eol, set carry          FC4A: 20 9E FC JSR cleolz
	LDY #$00           ;  clear from h index=0 for rest    FC4D: A0 00 LDY #$00
	PLA                ;  increment current line           FC4F: 68 PLA
	ADC #$00           ;  (carry is set)                   FC50: 69 00 ADC #$00
	CMP wndbtm         ;  done to bottom of window?        FC52: C5 23 CMP wndbtm
	BCC cleop1         ;  no, keep clearing lines          FC54: 90 F0 BCC cleop1
	BCS vtab           ;  yes, tab to current line         FC56: B0 CA BCS vtab

home:
	LDA wndtop         ;  init cursor v                    FC58: A5 22 HOME LDA wndtop
	STA cv             ;  and h-indices                    FC5A: 85 25 STA cv
	LDY #$00           ;                                   FC5C: A0 00 LDY #$00
	STY ch             ;  then clear to end of page        FC5E: 84 24 STY ch
	BEQ cleop1         ;                                   FC60: F0 E4 BEQ cleop1
	
cr:	
	LDA #$00           ;  cursor to left of index          FC62: A9 00 CR LDA #$00
	STA ch             ;  (ret cursor h=0)                 FC64: 85 24 STA ch
	
lf:	
	INC cv             ;  incr cursor v(down 1 line)       FC66: E6 25 LF INC cv
	;schamberlin: early return from scrolling code
	LDA #$0D
	PHX
	PHY
	JSR console_write_char   
	PLY    
	PLX
	RTS

clreol:
	LDY ch             ;  cursor h index                   FC9C: A4 24 CLREOL LDY ch

cleolz:
	LDA #$a0           ;                                   FC9E: A9 A0 CLEOLZ LDA #$a0

cleol2:
	;STA (basl),y       ;  store blanks from 'here'         FCA0: 91 28 CLEOL2 STA (basl),y
	INY                ;  to end of lines (wndwdth)        FCA2: C8 INY
	CPY wndwdth        ;                                   FCA3: C4 21 CPY wndwdth
	BCC cleol2         ;                                   FCA5: 90 F9 BCC cleol2
	RTS                ;                                   FCA7: 60 RTS

nxta4:
	INC a4l            ;  incr 2-byte a4                   FCB4: E6 42 NXTA4 INC a4l
	BNE nxta1          ;  and a1                           FCB6: D0 02 BNE nxta1
	INC a4h            ;                                   FCB8: E6 43 INC a4h

nxta1:
	LDA a1l            ;  incr 2-byte a1.                  FCBA: A5 3C NXTA1 LDA a1l
	CMP a2l            ;                                   FCBC: C5 3E CMP a2l
	LDA a1h            ;  and compare to a2                FCBE: A5 3D LDA a1h	
	SBC a2h            ;                                   FCC0: E5 3F SBC a2h
	INC a1l            ;  (carry set if >=)                FCC2: E6 3C INC a1l
	BNE rts4b          ;                                   FCC4: D0 02 BNE rts4b
	INC a1h            ;                                   FCC6: E6 3D INC a1h

rts4b:
	RTS                ;                                   FCC8: 60 RTS4B RTS

rdkey:
	JMP (kswl)         ;  go to user key-in                 FD18: 6C 38 00 JMP (kswl)

keyin:
	INC rndl           ;                                   FD1B: E6 4E KEYIN INC rndl
	BNE keyin2         ;  incr rnd number                  FD1D: D0 02 BNE keyin2
	INC rndh           ;                                   FD1F: E6 4F INC rndh

keyin2:
	; schamberlin: replaced original keyboard input subroutine below
	LDA #1
	CMP KEYBOARD_ADR_KEY_READY
	BNE keyin
	LDA KEYBOARD_ADR_KEY_BUFFER
	ORA #$80 ; set the non-inverted flag
	;schamberlin: TODO: use KEYBOARD_ADR_MODIFIER_BUFFER to set control bits as needed
	;LDY KEYBOARD_ADR_MODIFIER_BUFFER
	CLR KEYBOARD_ADR_KEY_READY
	
	RTS                ;                                   FD2E: 60 RTS
	
esc:	
	JSR rdkey          ;  get keycode                      FD2F: 20 0C FD ESC JSR rdkey
	JSR esc1           ;  handle esc func.                 FD32: 20 2C FC JSR esc1

rdchar:
	JSR rdkey          ;  read key                         FD35: 20 0C FD RDCHAR JSR rdkey
	CMP #$9b           ;  esc?                             FD38: C9 9B CMP #$9b
	BEQ esc            ;  yes, don't return                FD3A: F0 F3 BEQ esc
	RTS                ;                                   FD3C: 60 RTS

notcr:
	LDA invflg         ;                                   FD3D: A5 32 NOTCR LDA invflg
	PHA                ;                                   FD3F: 48 PHA
	LDA #$ff           ;                                   FD40: A9 FF LDA #$ff
	STA invflg         ;  echo user line                   FD42: 85 32 STA invflg
	LDA in,x           ;  non inverse                      FD44: BD 00 02 LDA in,x
	JSR cout           ;                                   FD47: 20 ED FD JSR cout
	PLA                ;                                   FD4A: 68 PLA
	STA invflg         ;                                   FD4B: 85 32 STA invflg
	LDA in,x           ;                                   FD4D: BD 00 02 LDA in,x
	CMP #$88           ;  check for edit keys              FD50: C9 88 CMP #$88
	BEQ bckspc         ;  bs, ctrl-x                       FD52: F0 1D BEQ bckspc
	CMP #$98           ;                                   FD54: C9 98 CMP #$98
	BEQ cancel         ;                                   FD56: F0 0A BEQ cancel
	CPX #$f8           ;  margin?                          FD58: E0 F8 CPX #$f8
	BCC notcr1         ;                                   FD5A: 90 03 BCC notcr1
	JSR bell           ;  yes, sound bell                  FD5C: 20 3A FF JSR bell

notcr1:
	INX                ;  advance input index              FD5F: E8 NOTCR1 INX
	BNE nxtchar        ;                                   FD60: D0 13 BNE nxtchar

cancel:
	LDA #$dc           ;  backslash after cancelled line   FD62: A9 DC CANCEL LDA #$dc
	JSR cout           ;                                   FD64: 20 ED FD JSR cout

getlnz:
	JSR crout          ;  output cr                        FD67: 20 8E FD GETLNZ JSR crout

getln:
	LDA prompt         ;                                   FD6A: A5 33 GETLN LDA prompt
	JSR cout           ;  output prompt char               FD6C: 20 ED FD JSR cout
	LDX #$01           ;  init input index                 FD6F: A2 01 LDX #$01

bckspc:
	TXA                ;  will backspace to 0              FD71: 8A BCKSPC TXA
	BEQ getlnz         ;                                   FD72: F0 F3 BEQ getlnz
	DEX                ;                                   FD74: CA DEX

nxtchar:
	JSR rdchar         ;                                   FD75: 20 35 FD NXTCHAR JSR rdchar
	CMP #pick          ;  use screen char                  FD78: C9 95 CMP #pick
	BNE captst         ;  for ctrl-u                       FD7A: D0 02 BNE captst
	LDA (basl),y       ;                                   FD7C: B1 28 LDA (basl),y

captst:
	CMP #$e0           ;                                   FD7E: C9 E0 CAPTST CMP #$e0
	BCC addinp         ;  convert to caps                  FD80: 90 02 BCC addinp
	AND #$df           ;                                   FD82: 29 DF AND #$df

addinp:
	STA in,x           ;  add to input buf                 FD84: 9D 00 02 ADDINP STA in,x
	CMP #$8d           ;                                   FD87: C9 8D CMP #$8d
	BNE notcr          ;                                   FD89: D0 B2 BNE notcr
	JSR clreol         ;  clr to eol if cr                 FD8B: 20 9C FC JSR clreol

crout:
	LDA #$8d           ;                                   FD8E: A9 8D CROUT LDA #$8d
	;schamberlin: was BNE cout
	JMP cout           ;                                   FD90: D0 5B BNE cout

pra1:
	LDY a1h            ;  print cr,a1 in hex               FD92: A4 3D PRA1 LDY a1h
	LDX a1l            ;                                   FD94: A6 3C LDX a1l

pryx2:
	JSR crout          ;                                   FD96: 20 8E FD PRYX2 JSR crout
	JSR prntyx         ;                                   FD99: 20 40 F9 JSR prntyx
	LDY #$00           ;                                   FD9C: A0 00 LDY #$00
	LDA #$ad           ;  print '-'                        FD9E: A9 AD LDA #$ad
	JMP cout           ;                                   FDA0: 4C ED FD JMP cout

xam8:
	LDA a1l            ;                                   FDA3: A5 3C XAM8 LDA a1l
	ORA #$07           ;  set to finish at                 FDA5: 09 07 ORA #$07
	STA a2l            ;  mod 8=7                          FDA7: 85 3E STA a2l
	LDA a1h            ;                                   FDA9: A5 3D LDA a1h
	STA a2h            ;                                   FDAB: 85 3F STA a2h

modschk:
	LDA a1l            ;                                   FDAD: A5 3C MODSCHK LDA a1l
	AND #$07           ;                                   FDAF: 29 07 AND #$07
	BNE dataout        ;                                   FDB1: D0 03 BNE dataout
	
xam:	
	JSR pra1           ;                                   FDB3: 20 92 FD XAM JSR pra1

dataout:
	LDA #$a0           ;                                   FDB6: A9 A0 DATAOUT LDA #$a0
	JSR cout           ;  output blank                     FDB8: 20 ED FD JSR cout
	
	; schamberlin: added ldaa1y
	JSR ldaa1y
	;LDA (a1l),y        ;                                   FDBB: B1 3C LDA (a1l),y
	
	JSR prbyte         ;  output byte in hex               FDBD: 20 DA FD JSR prbyte
	JSR nxta1          ;                                   FDC0: 20 BA FC JSR nxta1
	BCC modschk        ;  check if time to,                FDC3: 90 E8 BCC modschk

rts4c:
	RTS                ;  print addr                       FDC5: 60 RTS4C RTS

xampm:
	LSR                ;  determine if mon                 FDC6: 4A XAMPM LSR
	BCC xam            ;  mode is xam                      FDC7: 90 EA BCC xam
	LSR                ;  add, or sub                      FDC9: 4A LSR
	LSR                ;                                   FDCA: 4A LSR
	LDA a2l            ;                                   FDCB: A5 3E LDA a2l	
	BCC add            ;                                   FDCD: 90 02 BCC add
	EOR #$ff           ;  sub: form 2's complement         FDCF: 49 FF EOR #$ff
	
add:	
	ADC a1l            ;                                   FDD1: 65 3C ADD ADC a1l
	PHA                ;                                   FDD3: 48 PHA
	LDA #$bd           ;                                   FDD4: A9 BD LDA #$bd
	JSR cout           ;  print '=', then result           FDD6: 20 ED FD JSR cout
	PLA                ;                                   FDD9: 68 PLA

prbyte:
	PHA                ;  print byte as 2 hex              FDDA: 48 PRBYTE PHA
	LSR                ;  digits, destroys a-reg           FDDB: 4A LSR
	LSR                ;                                   FDDC: 4A LSR
	LSR                ;                                   FDDD: 4A LSR
	LSR                ;                                   FDDE: 4A LSR
	JSR prhexz         ;                                   FDDF: 20 E5 FD JSR prhexz
	PLA                ;                                   FDE2: 68 PLA

prhex:
	AND #$0f           ;  print hex dig in a-reg           FDE3: 29 0F PRHEX AND #$0f

prhexz:
	ORA #$b0           ;  lsb's                            FDE5: 09 B0 PRHEXZ ORA #$b0
	CMP #$ba           ;                                   FDE7: C9 BA CMP #$ba
	BCC cout           ;                                   FDE9: 90 02 BCC cout
	ADC #$06           ;                                   FDEB: 69 06 ADC #$06

cout:
	JMP (cswl)         ;  vector to user output routine    FDED: 6C 36 00 COUT JMP (cswl)

cout1:
	CMP #$a0           ;  char less than #$a0?             FDF0: C9 A0 COUT1 CMP #$a0
	BCC coutz          ;  don't output ctrl's inverse      FDF2: 90 02 BCC coutz
	AND invflg         ;  mask with inverse flag           FDF4: 25 32 AND invflg

coutz:
	STY ysav1          ;  sav y-reg                        FDF6: 84 35 COUTZ STY ysav1
	PHA                ;  sav a-reg                        FDF8: 48 PHA
	JSR vidout         ;  output a-reg as ascii            FDF9: 20 FD FB JSR vidout
	PLA                ;  restore a-reg                    FDFC: 68 PLA
	LDY ysav1          ;  and y-reg                        FDFD: A4 35 LDY ysav1
	RTS                ;  then return                      FDFF: 60 RTS
	
bl1:	
	DEC ysav           ;                                   FE00: C6 34 BL1 DEC ysav
	BEQ xam8           ;                                   FE02: F0 9F BEQ xam8

blank:
	DEX                ;  blank to mon                     FE04: CA BLANK DEX
	BNE setmdz         ;  after blank                      FE05: D0 16 BNE setmdz
	CMP #$ba           ;  data store mode?                 FE07: C9 BA CMP #$ba
	BNE xampm          ;  no, xam, add, or sub             FE09: D0 BB BNE xampm

stor:
	STA mode           ;  keep in store mode               FE0B: 85 31 STOR STA mode
	LDA a2l            ;                                   FE0D: A5 3E LDA a2l
	
	; schamberlin: added staa3y
	JSR staa3y
	;STA (a3l),y        ;  store as low byte as (a3)        FE0F: 91 40 STA (a3l),y
	
	INC a3l            ;                                   FE11: E6 40 INC a3l
	BNE rts5           ;  incr a3, return                  FE13: D0 02 BNE rts5
	INC a3h            ;                                   FE15: E6 41 INC a3h

rts5:
	RTS                ;                                   FE17: 60 RTS5 RTS

setmode:
	LDY ysav           ;  save converted ':', '+',         FE18: A4 34 SETMODE LDY ysav
	LDA in-1,y         ;  '-', '.' as mode.                FE1A: B9 FF 01 LDA in-1,y

setmdz:
	STA mode           ;                                   FE1D: 85 31 SETMDZ STA mode
	RTS                ;                                   FE1F: 60 RTS
	
lt:	
	LDX #$01           ;                                   FE20: A2 01 LT LDX #$01
	
lt2:	
	LDA a2l,x          ;  copy a2 (2 bytes) to             FE22: B5 3E LT2 LDA a2l,x
	STA a4l,x          ;  a4 and a5                        FE24: 95 42 STA a4l,x
	STA a5l,x          ;                                   FE26: 95 44 STA a5l,x
	DEX                ;                                   FE28: CA DEX
	BPL lt2            ;                                   FE29: 10 F7 BPL lt2
	RTS                ;                                   FE2B: 60 RTS

move:
	; schamberlin: added ldaa1y, staa4y
	JSR ldaa1y
	JSR staa4y	
	;LDA (a1l),y        ;  move (a1 to a2) to               FE2C: B1 3C MOVE LDA (a1l),y
	;STA (a4l),y        ;  (a4)                             FE2E: 91 42 STA (a4l),y
	
	JSR nxta4          ;                                   FE30: 20 B4 FC JSR nxta4
	BCC move           ;                                   FE33: 90 F7 BCC move
	RTS                ;                                   FE35: 60 RTS
	
vfy:	
	; schamberlin: added ldaa1y, cmpa4y
	JSR ldaa1y
	JSR cmpa4y
	;LDA (a1l),y        ;   verify (a1 to a2) with          FE36: B1 3C VFY LDA (a1l),y
	;CMP (a4l),y        ;  (a4)                             FE38: D1 42 CMP (a4l),y
	
	BEQ vfyok          ;                                   FE3A: F0 1C BEQ vfyok
	JSR pra1           ;                                   FE3C: 20 92 FD JSR pra1
	
	; schamberlin: added ldaa1y
	JSR ldaa1y
	;LDA (a1l),y        ;                                   FE3F: B1 3C LDA (a1l),y
		
	JSR prbyte         ;                                   FE41: 20 DA FD JSR prbyte
	LDA #$a0           ;                                   FE44: A9 A0 LDA #$a0
	JSR cout           ;                                   FE46: 20 ED FD JSR cout
	LDA #$a8           ;                                   FE49: A9 A8 LDA #$a8
	JSR cout           ;                                   FE4B: 20 ED FD JSR cout	
	
	; schamberlin: added ldaa4y
	JSR ldaa4y
	;LDA (a4l),y        ;                                   FE4E: B1 42 LDA (a4l),y
	
	JSR prbyte         ;                                   FE50: 20 DA FD JSR prbyte
	LDA #$a9           ;                                   FE53: A9 A9 LDA #$a9
	JSR cout           ;                                   FE55: 20 ED FD JSR cout

vfyok:
	JSR nxta4          ;                                   FE58: 20 B4 FC VFYOK JSR nxta4
	BCC vfy            ;                                   FE5B: 90 D9 BCC vfy
	RTS                ;                                   FE5D: 60 RTS
			
list:
	JSR a1pc           ;  move a1 (2 bytes) to             FE5E: 20 75 FE LIST JSR a1pc
	
	; schamberlin: changed LDA #$14 to LDA #$1A for number of output lines
	LDA #$1A           ;  pc if spec'd and                 FE61: A9 14 LDA #$14

list2:
	PHA                ;  disemble 20 instrs               FE63: 48 LIST2 PHA
	JSR instdsp        ;                                   FE64: 20 D0 F8 JSR instdsp
	JSR pcadj          ;  adjust pc each instr             FE67: 20 53 F9 JSR pcadj
	STA pcl            ;                                   FE6A: 85 3A STA pcl
	STY pch            ;                                   FE6C: 84 3B STY pch
	PLA                ;                                   FE6E: 68 PLA
	SEC                ;                                   FE6F: 38 SEC
	SBC #$01           ;  next of 20 instrs                FE70: E9 01 SBC #$01
	BNE list2          ;                                   FE72: D0 EF BNE list2
	RTS                ;                                   FE74: 60 RTS

a1pc:
	TXA                ;  if user spec'd adr               FE75: 8A A1PC TXA
	BEQ a1pcrts        ;  copy from a1 to pc               FE76: F0 07 BEQ a1pcrts

a1pclp:
	LDA a1l,x          ;                                   FE78: B5 3C A1PCLP LDA a1l,x
	STA pcl,x          ;                                   FE7A: 95 3A STA pcl,x
	DEX                ;                                   FE7C: CA DEX
	BPL a1pclp         ;                                   FE7D: 10 F9 BPL a1pclp

a1pcrts:
	RTS                ;                                   FE7F: 60 A1PCRTS RTS

setinv:
	LDY #$3f           ;  set for inverse vid              FE80: A0 3F SETINV LDY #$3f
	; schamberlin: changed BNE to JMP
	JMP setiflg        ;  via cout1                        FE82: D0 02 BNE setiflg

setnorm:
	LDY #$ff           ;  set for normal vid               FE84: A0 FF SETNORM LDY #$ff

setiflg:
	STY invflg         ;                                   FE86: 84 32 SETIFLG STY invflg
	RTS                ;                                   FE88: 60 RTS

setkbd:
	LDA #$00           ;  simulate port #0 input           FE89: A9 00 SETKBD LDA #$00

inport:
	STA a2l            ;  specified (keyin routine)        FE8B: 85 3E INPORT STA a2l

inprt:
	; schamberlin: changed address to be relative to monitor base
	LDX #(kswl-MONITOR_BASE)         ;                                   FE8D: A2 38 INPRT LDX #kswl
	LDY #<keyin        ;                                   FE8F: A0 1B LDY #keyin
	; schamberlin: modified lines below to fix assumption that keyin is on same page as cout1
	LDA #>keyin
	JMP ioprt2
	;BNE ioprt          ;                                   FE91: D0 08 BNE ioprt

setvid:
	LDA #$00           ;  simulate port #0 output          FE93: A9 00 SETVID LDA #$00

outport:
	STA a2l            ;  specified (cout1 routine)        FE95: 85 3E OUTPORT STA a2l

outprt:
	; schamberlin: changed address to be relative to monitor base
	LDX #(cswl-MONITOR_BASE)         ;                                   FE97: A2 36 OUTPRT LDX #cswl
	LDY #<cout1        ;                                   FE99: A0 F0 LDY #cout1

ioprt:
	LDA a2l            ;  set ram in/out vectors           FE9B: A5 3E IOPRT LDA a2l
	AND #$0f           ;                                   FE9D: 29 0F AND #$0f
	BEQ ioprt1         ;                                   FE9F: F0 06 BEQ ioprt1
	ORA #>ioadr        ;                                   FEA1: 09 C0 ORA #ioadr/256
	LDY #$00           ;                                   FEA3: A0 00 LDY #$00
	BEQ ioprt2         ;                                   FEA5: F0 02 BEQ ioprt2

ioprt1:
	LDA #>cout1        ;                                   FEA7: A9 FD IOPRT1 LDA #cout1/256

ioprt2:
	STY loc0,x         ;                                   FEA9: 94 00 IOPRT2 STY loc0,x
	STA loc1,x         ;                                   FEAB: 95 01 STA loc1,x
	RTS                ;                                   FEAD: 60 RTS
	
go:	
	JSR a1pc           ;  adr to pc if spec'd              FEB6: 20 75 FE GO JSR a1pc
	JSR restore        ;  restore meta regs                FEB9: 20 3F FF JSR restore
	
	; schamberlin: can't support non-zero bank here, because the called program's
	; RTS instruction doesn't know how to restore PCBANK	
	JMP (pcl)          ;  go to user subr                  FEBC: 6C 3A 00 JMP (pcl)

regz:
	JMP regdsp         ;  to reg display                   FEBF: 4C D7 FA REGZ JMP regdsp

trace:
	DEC ysav           ;                                   FEC2: C6 34 TRACE DEC ysav

stepz:
	JSR a1pc           ;  adr to pc if spec'd              FEC4: 20 75 FE STEPZ JSR a1pc
	JMP step           ;  take one step                    FEC7: 4C 43 FA JMP step
	
usr:	
	JMP usradr         ;  usradr to usr subr at usradr     FECA: 4C F8 03 USR JMP usradr

crmon:
	JSR bl1            ;  handle a cr as blank             FEF6: 20 00 FE CRMON JSR bl1
	PLA                ;  then pop stack                   FEF9: 68 PLA
	PLA                ;  and rtn to mon                   FEFA: 68 PLA
	; schamberlin: was BNE monz
	JMP monz
	;BNE monz           ;                                   FEFB: D0 6C BNE monz

bell:
	LDA #$87           ;  output bell and return           FF3A: A9 87 BELL LDA #$87
	JMP cout           ;                                   FF3C: 4C ED FD JMP cout

restore:
	LDA status         ;  restore 6502 reg contents        FF3F: A5 48 RESTORE LDA status
	PHA                ;  used by debug software           FF41: 48 PHA
	LDA acc            ;                                   FF42: A5 45 LDA acc

restr1:
	LDX xreg           ;                                   FF44: A6 46 RESTR1 LDX xreg
	LDY yreg           ;                                   FF46: A4 47 LDY yreg
	PLP                ;                                   FF48: 28 PLP
	RTS                ;                                   FF49: 60 RTS

save:
	STA acc            ;  save 6502 reg contents           FF4A: 85 45 SAVE STA acc

sav1:
	STX xreg           ;                                   FF4C: 86 46 SAV1 STX xreg
	STY yreg           ;                                   FF4E: 84 47 STY yreg
	PHP                ;                                   FF50: 08 PHP
	PLA                ;                                   FF51: 68 PLA
	STA status         ;                                   FF52: 85 48 STA status
	; schamberlin: disabled SP save
;	TSX                ;                                   FF54: BA TSX
;	STX spnt           ;                                   FF55: 86 49 STX spnt
;	CLD                ;                                   FF57: D8 CLD
	RTS                ;                                   FF58: 60 RTS

init_monitor:
	JSR setnorm        ;  set screen mode                  FF59: 20 84 FE RESET JSR setnorm
	JSR init           ;  and init kbd/screen              FF5C: 20 2F FB JSR init
	JSR setvid         ;  as i/o dev's                     FF5F: 20 93 FE JSR setvid
	JSR setkbd         ;                                   FF62: 20 89 FE JSR setkbd
	
mon:	
	JSR bell           ;                                   FF66: 20 3A FF JSR bell

monz:
	LDA #$aa           ;  '*' prompt for mon               FF69: A9 AA MONZ LDA #$aa
	STA prompt         ;                                   FF6B: 85 33 STA prompt
	JSR getlnz         ;  read a line                      FF6D: 20 67 FD JSR getlnz
	JSR zmode          ;  clear mon mode, scan idx         FF70: 20 C7 FF JSR zmode

nxtitm:
	JSR getnum         ;  get item, non-hex                FF73: 20 A7 FF NXTITM JSR getnum
	STY ysav           ;  char in a-reg                    FF76: 84 34 STY ysav
	; schamberlin: modified number of command table entries
	LDY #$15           ;  x-reg=0 if no hex input          FF78: A0 17 LDY #$17

chrsrch:
	DEY                ;                                   FF7A: 88 CHRSRCH DEY
	BMI mon            ;  not found, go to mon             FF7B: 30 E8 BMI mon
	CMP chrtbl,y       ;  find cmnd char in tel            FF7D: D9 CC FF CMP chrtbl,y
	BNE chrsrch        ;                                   FF80: D0 F8 BNE chrsrch
	JSR tosub          ;  found, call corresponding        FF82: 20 BE FF JSR tosub
	LDY ysav           ;  subroutine                       FF85: A4 34 LDY ysav
	JMP nxtitm         ;                                   FF87: 4C 73 FF JMP nxtitm
	
dig:	
	LDX #$03           ;                                   FF8A: A2 03 DIG LDX #$03
	ASL                ;                                   FF8C: 0A ASL
	ASL                ;  got hex dig,                     FF8D: 0A ASL
	ASL                ;  shift into a2                    FF8E: 0A ASL
	ASL                ;                                   FF8F: 0A ASL

nxtbit:
	ASL                ;                                   FF90: 0A NXTBIT ASL
	ROL a2l            ;                                   FF91: 26 3E ROL a2l
	ROL a2h            ;                                   FF93: 26 3F ROL a2h
	DEX                ;  leave x=$ff if dig               FF95: CA DEX
	BPL nxtbit         ;                                   FF96: 10 F8 BPL nxtbit

	; schamberlin: the original code here relied on page zero wrap-around, and a X value of FF (-1)
	; schamberlin: added LDX #1 below
	LDX #1
	
nxtbas:
	LDA mode           ;                                   FF98: A5 31 NXTBAS LDA mode
	BNE nxtbs2         ;  if mode is zero                  FF9A: D0 06 BNE nxtbs2
	; schamberlin: changed references to a2h, a1h, a3h below to a2l, a1l, a3l
	LDA a2l,x
	STA a1l,x
	STA a3l,x
	;LDA a2h,x          ;  then copy a2 to                  FF9C: B5 3F LDA a2h,x
	;STA a1h,x          ;  a1 and a3                        FF9E: 95 3D STA a1h,x
	;STA a3h,x          ;                                   FFA0: 95 41 STA a3h,x

nxtbs2:
	DEX                ;                                   FFA2: E8 NXTBS2 INX
	BEQ nxtbas         ;                                   FFA3: F0 F3 BEQ nxtbas
	; schamberlin: added LDX #1 to simulate original end condition
	LDX #1
	
	BNE nxtchr         ;                                   FFA5: D0 06 BNE nxtchr

getnum:
	LDX #$00           ;  clear a2                         FFA7: A2 00 GETNUM LDX #$00
	STX a2l            ;                                   FFA9: 86 3E STX a2l
	STX a2h            ;                                   FFAB: 86 3F STX a2h

nxtchr:
	LDA in,y           ;  get char                         FFAD: B9 00 02 NXTCHR LDA in,y
	INY                ;                                   FFB0: C8 INY
	EOR #$b0           ;                                   FFB1: 49 B0 EOR #$b0
	CMP #$0a           ;                                   FFB3: C9 0A CMP #$0a
	BCC dig            ;  if hex dig, then                 FFB5: 90 D3 BCC dig
	ADC #$88           ;                                   FFB7: 69 88 ADC #$88
	CMP #$fa           ;                                   FFB9: C9 FA CMP #$fa
	BCS dig            ;                                   FFBB: B0 CD BCS dig
	RTS                ;                                   FFBD: 60 RTS

tosub:
	; schamberlin: added the code below to use 2-byte subtbl instead of 1
	; Y = 2Y + 1
	TYA
	ASL
	TAY
	INY
	LDA subtbl,y
	DEY
	;LDA #>go           ;  push high-order                  FFBE: A9 FE TOSUB LDA #go/256
	PHA                ;  subr adr on stk                  FFC0: 48 PHA
	LDA subtbl,y       ;  push low-order                   FFC1: B9 E3 FF LDA subtbl,y
	PHA                ;  subr adr on stk                  FFC4: 48 PHA
	LDA mode           ;                                   FFC5: A5 31 LDA mode

zmode:
	LDY #$00           ;  clr mode, old mode               FFC7: A0 00 ZMODE LDY #$00
	STY mode           ;  to a-reg                         FFC9: 84 31 STY mode
	RTS                ;  go to subr via rts               FFCB: 60 RTS

chrtbl:
	!byte $b2          ;  f("ctrl-y")                      FFCD: B2 DFB $b2
	!byte $be          ;  f("ctrl-e")                      FFCE: BE DFB $be
	!byte $ed          ;  f("t")                           FFCF: ED DFB $ed
	!byte $ef          ;  f("v")                           FFD0: EF DFB $ef
	!byte $c4          ;  f("ctrl-k")                      FFD1: C4 DFB $c4
	!byte $ec          ;  f("s")                           FFD2: EC DFB $ec
	!byte $a9          ;  f("ctrl-p")                      FFD3: A9 DFB $a9
	!byte $a6          ;  f("-")                           FFD5: A6 DFB $a6
	!byte $a4          ;  f("+")                           FFD6: A4 DFB $a4
	!byte $06          ;  f("m") (f=ex-or $b0+$89)         FFD7: 06 DFB $06
	!byte $95          ;  f("<")                           FFD8: 95 DFB $95
	!byte $07          ;  f("n")                           FFD9: 07 DFB $07
	!byte $02          ;  f("i")                           FFDA: 02 DFB $02
	
	; schamberlin: added K (bank) and Q (quit) commands
	!byte $04
	!byte $ea
	
	!byte $05          ;  f("l")                           FFDB: 05 DFB $05
	!byte $00          ;  f("g")                           FFDD: 00 DFB $00
	!byte $93          ;  f(":")                           FFDF: 93 DFB $93
	!byte $a7          ;  f(".")                           FFE0: A7 DFB $a7
	!byte $c6          ;  f("cr")                          FFE1: C6 DFB $c6
	!byte $99          ;  f(blank)                         FFE2: 99 DFB $99

subtbl:
	!word (usr-1)        ;                                   FFE4: C9 DFB usr-1
	!word (regz-1)       ;                                   FFE5: BE DFB regz-1
	!word (trace-1)      ;                                   FFE6: C1 DFB trace-1
	!word (vfy-1)        ;                                   FFE7: 35 DFB vfy-1
	!word (inprt-1)      ;                                   FFE8: 8C DFB inprt-1
	!word (stepz-1)      ;                                   FFE9: C3 DFB stepz-1
	!word (outprt-1)     ;                                   FFEA: 96 DFB outprt-1
	!word (setmode-1)    ;                                   FFEC: 17 DFB setmode-1
	!word (setmode-1)    ;                                   FFED: 17 DFB setmode-1
	!word (move-1)       ;                                   FFEE: 2B DFB move-1
	!word (lt-1)         ;                                   FFEF: 1F DFB lt-1
	!word (setnorm-1)    ;                                   FFF0: 83 DFB setnorm-1
	!word (setinv-1)     ;                                   FFF1: 7F DFB setinv-1
	
	; schamberlin: added K (bank) and Q (quit) commands
	!word (setbank-1)
	!word (quit-1)
	
	!word (list-1)       ;                                   FFF2: 5D DFB list-1
	!word (go-1)         ;                                   FFF4: B5 DFB go-1
	!word (setmode-1)    ;                                   FFF6: 17 DFB setmode-1
	!word (setmode-1)    ;                                   FFF7: 17 DFB setmode-1
	!word (crmon-1)      ;                                   FFF8: F5 DFB crmon-1
	!word (blank-1)      ;                                   FFF9: 03 DFB blank-1

