; Process scheduler for BMOW

; dimensional constants
SCHEDULER_MAX_PROCESSES = 8
SCHEDULER_INFO_SIZE = 4
SCHEDULER_TABLE_SIZE = SCHEDULER_INFO_SIZE * SCHEDULER_MAX_PROCESSES

; --- process info struct:
; flags
; SPLO
; SPHI
; unused
; (We don't need to store SPBANK, as it's always the same as the process ID)
; (PCLO/HI/BANK and ARBANK are pushed on the stack by BRK)
; (CC, A, X, and Y are pushed on the stack by the interrupt handler)

; RAM addresses
SCHEDULER_CURRENT_PROCESS_ID = KEYBOARD_ADR_IGNORE_COUNT + 1
SCHEDULER_TABLE_BASE = SCHEDULER_CURRENT_PROCESS_ID + 1
SCHEDULER_TABLE_TOP = SCHEDULER_TABLE_BASE + SCHEDULER_TABLE_SIZE - 1

; timing constants
SCHEDULER_INTERRUPT_INTERVAL = 7 ; about 2ms

; --- Scheduler standard init
; @noparams
; Resets the scheduler state, and initializes the scheduler interrupt.
scheduler_standard_init:
	clr SCHEDULER_CURRENT_PROCESS_ID
	
	ldx #(SCHEDULER_TABLE_TOP - SCHEDULER_TABLE_BASE)
-	clr SCHEDULER_TABLE_BASE, x
	dex
	cpx #$ff
	bne -
	
	lda #SCHEDULER_INTERRUPT_INTERVAL
	sta RTC_ADR_RATES

	lda RTC_ADR_INTERRUPT_ENABLE	
	ora #$04
	sta RTC_ADR_INTERRUPT_ENABLE	
	
	rts
	
	
; --- Scheduler interrupt handler
; Store SPLO and SPHI in the process table.
; Select the next process, in round-robin fashion, and restore
; its state from the process table.
scheduler_interrupt_handler:
	; master interrupt handler has already pushed CC, A, X, and Y
	
	;lda #'S'
	;jsr lcd_write_char
	;lda #'c'
	;jsr lcd_write_char
	;lda #'h'
	;jsr lcd_write_char
	
	tsr ; A = SPLO, Y=SPHI, X=SPBANK
	
	; X = PID * 4
	pha
	lda SCHEDULER_CURRENT_PROCESS_ID
	asl
	asl
	tax
	pla
	
	; store SPLO, SPHI in the table
	inx
	sta SCHEDULER_TABLE_BASE, x
	inx
	tya
	sta SCHEDULER_TABLE_BASE, x
	
	; skip the unused byte, and go to the next table entry
	inx
	inx
	
	; find the next active process
-	cpx #SCHEDULER_TABLE_SIZE
	bne +
	ldx #0	

	; is the active flag set?	
+	lda SCHEDULER_TABLE_BASE, x
	cmp #1
	beq +
	
	; no, go to the next table entry
	inx
	inx
	inx
	inx
	bra -
	
	; choose this entry as the new active process
+	; PID = table offset / 4
	txa
	lsr
	lsr
	sta SCHEDULER_CURRENT_PROCESS_ID
	
	; restore SPLO, SPHI, SPBANK
	inx
	inx
	lda SCHEDULER_TABLE_BASE, x
	tay
	dex
	lda SCHEDULER_TABLE_BASE, x
	ldx SCHEDULER_CURRENT_PROCESS_ID
	trs
	
	ply
	plx
	pla
	plp
	
	rti
	
	
; --- Start a process
; @param0 the memory bank containing the program to run
; Adds a new entry to the process table for a new program.
scheduler_start_process:	
	; save current stack
	; TODO: replace this with bank accumulator, when new microcode is ready
	; TODO: this isn't thread safe, if called from a user process
	pha
	tsr ; A = SPLO, Y=SPHI, X=SPBANK
	; store SPLO, SPHI in the table
	sta SCHEDULER_TABLE_BASE+1
	sty SCHEDULER_TABLE_BASE+2
	pla
	
!if 0 {
	ldx #0
	
	; is the active flag clear?	
-	lda SCHEDULER_TABLE_BASE, x
	cmp #0
	beq +
	
	; no, go to the next table entry
	inx
	inx
	inx
	inx
	
	; did we reach the end of the table?
	cpx #SCHEDULER_TABLE_SIZE
	bne -
	
	; yes, so a new process can't be started. Just return
	rts
	
	; choose this entry for the new process
}
	; table offset = PID * 4
	asl
	asl
	tax
		
+	; set the active flag
	lda #1
	sta SCHEDULER_TABLE_BASE, x
	
	; store SPLO, SPHI for SP=$FFF7 in the table
	lda #$f7
	inx
	sta SCHEDULER_TABLE_BASE, x
	inx
	lda #$ff
	sta SCHEDULER_TABLE_BASE, x
	
	; PID = table offset / 4 (truncated)
	txa
	lsr
	lsr
	tax
	
	; setup the stack to $FFFF
	ldy #$FF
	lda #$FF
	trs	
	
	; push the data normally pushed by BRK
	phx ; ARBANK
	phx ; PCBANK
	lda #0
	pha ; PCHI
	lda #1 ; push PC = $0001, because RTI will subtract 1
	pha ; PCLO
	
	; push some bytes for the data normally pushed by the interrupt handler.
	; these will become the initial register values when the program starts.
	pha
	pha
	pha
	pha
	
	; restore the original stack
	; TODO: replace this with bank accumulator, when new microcode is ready
	ldx #0
	lda SCHEDULER_TABLE_BASE+1
	ldy SCHEDULER_TABLE_BASE+2
	trs
	pla ; pop the value that be pushed before storing the stack
	
	rts