; Video color tests
!source "video_defs.s"

VIDEO_ADR_BASE = KEYBOARD_ADR_END+1
VIDEO_ADR_SPSAVE = VIDEO_ADR_BASE+1
VIDEO_ADR_A0 = VIDEO_ADR_BASE+1
VIDEO_ADR_A0_L = VIDEO_ADR_BASE+1
VIDEO_ADR_A0_H = VIDEO_ADR_BASE+2
VIDEO_ADR_TEMP = VIDEO_ADR_BASE+3

; --- Video zero VRAM
; @noparams
; Clears contents of VRAM to all zeroes.
video_zero_vram:
	phb
	bank #VIDEO_RAM_BANK
	lda #$80
	sta VIDEO_RAM_A0_L
	lda #$7E
	sta VIDEO_RAM_A0_H
	
--	ldx #$7F
	lda #0
	
-	sta (VIDEO_RAM_A0),X
	dex
	bpl -
	
	; decrement row pointer
	lda VIDEO_RAM_A0_L
	sec
	sbc #$80
	sta VIDEO_RAM_A0_L
	bcs --
	dec VIDEO_RAM_A0_H
	bpl --
	
	plb 
	rts
	

; --- Video color test 1bpp: tweed pattern @ 512480 @ 2 colors
; @noparams
; 512480 @ 2 colors
video_color_test_1bpp:	 	
	lda #$C0
	sta VIDEO_ADR_VSIZE ; 480 lines, mode byte per frame
   
	jsr video_palette_1bpp
	jsr video_zero_vram
   
	phb
	bank #$20
	lda #$00 ; 512 columns, 1bpp
	sta $7A80 
	sta $7A81
	
!for RowBase, 240 {
	ldx #$00
		
-	eor #$51
	rol
	sta VIDEO_RAM_BASE + ((RowBase-1) << 7), X
	inx
	bpl -
}

	plb
	
	rts		
	

; --- Video color test 2bpp: color tweed pattern @ 512240 @ 4 colors
; @noparams
; 512240 @ 4 colors
video_color_test_2bpp:	   
	lda #$40
	sta VIDEO_ADR_VSIZE ; 240 lines, mode byte per frame
   
	jsr video_palette_2bpp
	jsr video_zero_vram
   
    phb
	bank #$20
	lda #$10 ; 512 columns, 2bpp
	sta $7A80 
	sta $7A81
	
!for RowBase, 240 {
	ldx #$00
		
-	eor #$51
	rol
	sta VIDEO_RAM_BASE + ((RowBase-1) << 7), X
	inx
	bpl -
}

	plb
	
	rts	
 
 
; --- Video color test: color bands @ 128240 @ 256 colors
; @noparams
; 128240 @ 256 colors
video_color_test_8bpp:	
	lda #$40
	sta VIDEO_ADR_VSIZE ; 240 lines, mode byte per frame
   
	jsr video_palette_direct_color
	jsr video_zero_vram
   
    phb
	bank #$20
	lda #$30 ; 128 columns, 8bpp
	sta $7A80 
	sta $7A81
   
!for RowBase, 240 {
	ldx #$00
		
-	txa
	adc #$7F
	sta VIDEO_RAM_BASE + ((RowBase-1) << 7), X
	inx
	bpl -
}

	plb
	
	rts
   
 		
; --- Video setup 1bpp palette
; @noparams
; Sets the RGB value of palette entries 0-15 to be
; foreground color if the least significant bit is 1, background if it's 0.
video_palette_1bpp:
	ldx #$00
	
-	; background color
	txa
	sta VIDEO_ADR_PAL_INDEX
	
	; black background
	lda #$00
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	
	inx
	
	; foreground color
	txa
	sta VIDEO_ADR_PAL_INDEX
	
	; foreground.rgb
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
		
	inx
	cpx #$10
	
	bne -
					
	rts	
	
		
; --- Video setup 2bpp palette
; @noparams
; Sets the RGB value of the first 16 palette entries to be
; dependent on the least significant 2 bits:
; 00 = black
; 01 = red
; 10 = green
; 11 = white
video_palette_2bpp:
	; black
	lda #$0C
-	sta VIDEO_ADR_PAL_INDEX
	tay
	lda #0
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	tya
	dec
	dec
	dec
	dec
	bpl -
	
	; white
	ldx #$0F
-	txa
	sta VIDEO_ADR_PAL_INDEX
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	dex
	dex
	dex
	dex
	bpl -

	; red
	ldx #$0D
-	txa
	sta VIDEO_ADR_PAL_INDEX
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	lda #0
	sta VIDEO_ADR_PAL_COLOR
	sta VIDEO_ADR_PAL_COLOR
	dex
	dex
	dex
	dex
	bpl -

	; green
	ldx #$0E
-	txa
	sta VIDEO_ADR_PAL_INDEX
	lda #0
	sta VIDEO_ADR_PAL_COLOR
	lda #$3F
	sta VIDEO_ADR_PAL_COLOR
	lda #0
	sta VIDEO_ADR_PAL_COLOR
	dex
	dex
	dex
	dex
	bpl -
						
	rts	
			
; --- Video direct color palette
; @noparams
; Sets the RGB value of each palette entry, using the entry
; index as a direct color value RRRGGGBB.	
video_palette_direct_color:
	ldx #$00
	
-	txa
	sta VIDEO_ADR_PAL_INDEX
	and #$E0
	lsr
	lsr
	sta VIDEO_ADR_PAL_COLOR
	
	txa
	and #$1C
	asl
	sta VIDEO_ADR_PAL_COLOR
	
	txa
	and #$03
	asl
	asl
	asl
	asl
	sta VIDEO_ADR_PAL_COLOR
	
	inx
	bne -
	
	rts	
	
