; --- memory-mapped addresses
; VRAM is in bank 20, addresses $0000-$7FFF
VIDEO_RAM_BANK = $20
VIDEO_RAM_BASE = $0000

; VSIZE register (write only)
; $00 - 240 lines, mode byte per line
; $40 - 240 lines, mode byte per frame
; $80 - 480 lines, mode byte per line
; $C0 - 480 lines, mode byte per frame

VIDEO_ADR_VSIZE = $003F30

; VBLANK register (read only)
; $80 = drawing frame, $00 = vblank interval
VIDEO_ADR_VBLANK = $003F30

; PALETTE registers (write only)
; palette entry to modify ($00-$FF)
VIDEO_ADR_PAL_INDEX = $003F40

; new RGB value for the entry 
;(must write R,G,B on consecutive writes, only the lowest 6 bits per channel are used)
VIDEO_ADR_PAL_COLOR = $003F44

; mask for pixel data
; use $FF for 8bpp and 4bpp
; use $F3 for 2bpp
; use $F1 for 1bpp
; or setup the palette entries so that the masked bits are "don't care"
VIDEO_ADR_PAL_MASK = $003F48

!ifdef VIDEO_RAM_A0 {
} else {
	; user pointer in VRAM
	VIDEO_RAM_A0 = $7FFE
	VIDEO_RAM_A0_L = $7FFE
	VIDEO_RAM_A0_H = $7FFF
}

; --- mode byte
; Unique mode stored per line when VSIZE = $00 or $80, or one common mode stored per
; frame at $207A80-$207A81 when VSIZE = $40 or $C0
VIDEO_ADR_FRAME_MODE_0 = $207800
VIDEO_ADR_FRAME_MODE_1 = $207A00
VIDEO_ADR_FRAME_MODE_2 = $207A01
VIDEO_ADR_FRAME_MODE_3 = $207A80
VIDEO_ADR_FRAME_MODE_4 = $207A81

; MODE = RMASK TEXT BPP1 BPP0 PAL3 PAL2 PAL1 PAL0
; 
; RMASK = when true and VSIZE = $00 or $40, forces an effective 120 vertical resolution
; TEXT = when true, VRAM data is used as an index into the character ROM, instead of being used directly
; BPP = determines the color depth: 3=256 colors, 2=16 colors, 1=4 colors, 0=2 colors
; PAL = determines the high 4 bits used for color palette lookups, with the low 4 bits supplied by VRAM 
;       data (not used for 256 color mode)

; --- common full-screen video modes
; VSIZE  MODE  Description
;  $40   $4X   64x30 text using palette section X
;  $40   $30   128x240 @ 256 colors 
;  $40   $2X   256x240 @ 16 colors using palette X
;  $40   $1X   512x240 @ 4 colors using palette X
;  $C0   $0X   512x480 @ 2 colors using palette X

; --- common mixed mode video modes
; VSIZE  MODE  Description
;  $00   $4X   63x30 text line using palette X
;  $00   $30   126x240 @ 256 colors line
;  $00   $2X   252x240 @ 16 colors line using palette X
;  $00   $1X   504x240 @ 4 colors line using palette X
;  $80   $4X   63x30 text line using palette X
;  $80   $0X   504x480 @ 2 colors using palette X

; --- interleaving
; For 480 line resolutions when VSIZE = $80 or $C0, the bytes for each pair of lines are interleaved in
; memory. For two lines A and B in the image which look like:
; A0 A1 A2 A3 ...
; B0 B1 B2 B3 ...
; The data must be stored in VRAM as:
; A0 B0 A1 B1 A2 B2 A3 B3 ...

